@@ -54,7 +54,7 @@ typedef struct cmd_help_t {
  * cmd - The command string including leading '/'
  * parser - The function used to parse arguments
  * min_args - Minimum number of arguments
- * max_args - Maximum number of arguments
+ * max_args - Maximum number of arguments, -1 for infinite
  * setting_func - Function to display current settings to the console
  * sub_funcs - Optional list of functions mapped to the first argument
  * func - Main function to call when no arguments, or sub_funcs not implemented
@@ -115,6 +115,7 @@ gboolean cmd_reconnect(ProfWin *window, const char *const command, gchar **args)
 gboolean cmd_room(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_rooms(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_bookmark(ProfWin *window, const char *const command, gchar **args);
+gboolean cmd_bookmark_ignore(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_roster(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_software(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_splash(ProfWin *window, const char *const command, gchar **args);
@@ -146,7 +147,6 @@ gboolean cmd_resource(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_inpblock(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_titlebar(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_titlebar_show_hide(ProfWin *window, const char *const command, gchar **args);
-gboolean cmd_titlebar_use(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_mainwin(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_statusbar(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_inputwin(ProfWin *window, const char *const command, gchar **args);
@@ -201,6 +201,7 @@ gboolean cmd_otr_untrust(ProfWin *window, const char *const command, gchar **arg
 gboolean cmd_otr_secret(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_otr_question(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_otr_answer(ProfWin *window, const char *const command, gchar **args);
+gboolean cmd_otr_sendfile(ProfWin *window, const char *const command, gchar **args);
 
 gboolean cmd_wins(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_wins_unread(ProfWin *window, const char *const command, gchar **args);
@@ -219,6 +220,7 @@ gboolean cmd_omemo_trust(ProfWin *window, const char *const command, gchar **arg
 gboolean cmd_omemo_untrust(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_omemo_policy(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_omemo_clear_device_list(ProfWin *window, const char *const command, gchar **args);
+gboolean cmd_omemo_sendfile(ProfWin *window, const char *const command, gchar **args);
 
 gboolean cmd_save(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_reload(ProfWin *window, const char *const command, gchar **args);
@@ -227,4 +229,11 @@ gboolean cmd_paste(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_color(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_avatar(ProfWin *window, const char *const command, gchar **args);
 gboolean cmd_os(ProfWin *window, const char *const command, gchar **args);
+gboolean cmd_correction(ProfWin *window, const char *const command, gchar **args);
+gboolean cmd_correct(ProfWin *window, const char *const command, gchar **args);
+gboolean cmd_slashguard(ProfWin *window, const char *const command, gchar **args);
+gboolean cmd_serversoftware(ProfWin *window, const char *const command, gchar **args);
+gboolean cmd_urlopen(ProfWin *window, const char *const command, gchar **args);
+gboolean cmd_executable(ProfWin *window, const char *const command, gchar **args);
+
 #endif
