@@ -88,7 +88,6 @@ char* str_replace(const char *string, const char *substr, const char *replacemen
 int str_contains(const char str[], int size, char ch);
 gboolean strtoi_range(char *str, int *saveptr, int min, int max, char **err_msg);
 int utf8_display_len(const char *const str);
-char* file_getline(FILE *stream);
 
 char* release_get_latest(void);
 gboolean release_is_new(char *found_version);
@@ -99,6 +98,7 @@ gboolean is_notify_enabled(void);
 
 GSList* prof_occurrences(const char *const needle, const char *const haystack, int offset, gboolean whole_word,
     GSList **result);
+GSList* get_mentions(gboolean whole_word, gboolean case_sensitive, const char *const message, const char *const nick);
 
 int is_regular_file(const char *path);
 int is_dir(const char *path);
@@ -106,4 +106,6 @@ void get_file_paths_recursive(const char *directory, GSList **contents);
 
 char* get_random_string(int length);
 
+gboolean call_external(gchar **argv, gchar ***const output_ptr, gchar ***const error_ptr);
+
 #endif
