@@ -48,7 +48,7 @@ char* accounts_find_all(const char *const prefix, gboolean previous, void *conte
 char* accounts_find_enabled(const char *const prefix, gboolean previous, void *context);
 void accounts_reset_all_search(void);
 void accounts_reset_enabled_search(void);
-void accounts_add(const char *jid, const char *altdomain, const int port, const char *const tls_policy);
+void accounts_add(const char *jid, const char *altdomain, const int port, const char *const tls_policy, const char *const auth_policy);
 int  accounts_remove(const char *jid);
 gchar** accounts_get_list(void);
 ProfAccount* accounts_get_account(const char *const name);
@@ -67,6 +67,7 @@ void accounts_set_muc_service(const char *const account_name, const char *const
 void accounts_set_muc_nick(const char *const account_name, const char *const value);
 void accounts_set_otr_policy(const char *const account_name, const char *const value);
 void accounts_set_tls_policy(const char *const account_name, const char *const value);
+void accounts_set_auth_policy(const char *const account_name, const char *const value);
 void accounts_set_last_presence(const char *const account_name, const char *const value);
 void accounts_set_last_status(const char *const account_name, const char *const value);
 void accounts_set_last_activity(const char *const account_name);
