@@ -3,6 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2020 Michael Vetter <jubalh@idoru.org>
  *
  * This file is part of Profanity.
  *
@@ -90,7 +91,7 @@ files_create_directories(void)
     g_free(xdg_data);
 }
 
-char*
+gchar*
 files_get_inputrc_file(void)
 {
     gchar *xdg_config = _files_get_xdg_config_home();
@@ -100,7 +101,7 @@ files_get_inputrc_file(void)
     g_string_append(inputrc_file, "/profanity/inputrc");
 
     if (g_file_test(inputrc_file->str, G_FILE_TEST_IS_REGULAR)) {
-        char *result = strdup(inputrc_file->str);
+        gchar *result = g_strdup(inputrc_file->str);
         g_string_free(inputrc_file, TRUE);
 
         return result;
@@ -112,44 +113,54 @@ files_get_inputrc_file(void)
 }
 
 char*
-files_get_log_file(void)
+files_get_log_file(char *log_file)
 {
     gchar *xdg_data = _files_get_xdg_data_home();
     GString *logfile = g_string_new(xdg_data);
-    g_string_append(logfile, "/profanity/logs/profanity");
+
+    if (log_file) {
+        g_string_append(logfile, "/profanity/logs/");
+        g_string_append(logfile, log_file);
+    } else {
+        g_string_append(logfile, "/profanity/logs/profanity");
+    }
+
     if (!prefs_get_boolean(PREF_LOG_SHARED)) {
         g_string_append_printf(logfile, "%d", getpid());
     }
+
     g_string_append(logfile, ".log");
-    char *result = strdup(logfile->str);
+
+    char *result = g_strdup(logfile->str);
+
     free(xdg_data);
     g_string_free(logfile, TRUE);
 
     return result;
 }
 
-char*
+gchar*
 files_get_config_path(char *config_base)
 {
     gchar *xdg_config = _files_get_xdg_config_home();
     GString *file_str = g_string_new(xdg_config);
     g_string_append(file_str, "/profanity/");
     g_string_append(file_str, config_base);
-    char *result = strdup(file_str->str);
+    gchar *result = g_strdup(file_str->str);
     g_free(xdg_config);
     g_string_free(file_str, TRUE);
 
     return result;
 }
 
-char*
+gchar*
 files_get_data_path(char *data_base)
 {
     gchar *xdg_data = _files_get_xdg_data_home();
     GString *file_str = g_string_new(xdg_data);
     g_string_append(file_str, "/profanity/");
     g_string_append(file_str, data_base);
-    char *result = strdup(file_str->str);
+    gchar *result = g_strdup(file_str->str);
     g_free(xdg_data);
     g_string_free(file_str, TRUE);
 
@@ -159,12 +170,16 @@ files_get_data_path(char *data_base)
 static char*
 _files_get_xdg_config_home(void)
 {
-    gchar *xdg_config_home = getenv("XDG_CONFIG_HOME");
-    if (xdg_config_home)
+    gchar *xdg_config_home_env = getenv("XDG_CONFIG_HOME");
+    gchar *xdg_config_home = NULL;
+
+    if (xdg_config_home_env) {
+        xdg_config_home = strdup(xdg_config_home_env);
         g_strstrip(xdg_config_home);
+    }
 
     if (xdg_config_home && (strcmp(xdg_config_home, "") != 0)) {
-        return strdup(xdg_config_home);
+        return xdg_config_home;
     } else {
         GString *default_path = g_string_new(getenv("HOME"));
         g_string_append(default_path, "/.config");
@@ -178,12 +193,16 @@ _files_get_xdg_config_home(void)
 static char*
 _files_get_xdg_data_home(void)
 {
-    gchar *xdg_data_home = getenv("XDG_DATA_HOME");
-    if (xdg_data_home)
+    gchar *xdg_data_home_env = getenv("XDG_DATA_HOME");
+    gchar *xdg_data_home = NULL;
+
+    if (xdg_data_home_env) {
+        xdg_data_home = strdup(xdg_data_home_env);
         g_strstrip(xdg_data_home);
+    }
 
     if (xdg_data_home && (strcmp(xdg_data_home, "") != 0)) {
-        return strdup(xdg_data_home);
+        return xdg_data_home;
     } else {
         GString *default_path = g_string_new(getenv("HOME"));
         g_string_append(default_path, "/.local/share");
