@@ -67,7 +67,7 @@
 
 #define INPBLOCK_DEFAULT 1000
 
-static char *prefs_loc;
+static gchar *prefs_loc;
 static GKeyFile *prefs;
 gint log_maxsize = 0;
 
@@ -80,7 +80,7 @@ static const char* _get_key(preference_t pref);
 static gboolean _get_default_boolean(preference_t pref);
 static char* _get_default_string(preference_t pref);
 
-void _prefs_load(void)
+static void _prefs_load(void)
 {
     GError *err = NULL;
     log_maxsize = g_key_file_get_integer(prefs, PREF_GROUP_LOGGING, "maxsize", &err);
@@ -149,6 +149,16 @@ void _prefs_load(void)
         g_key_file_remove_key(prefs, PREF_GROUP_UI, "titlebar.goodbye", NULL);
     }
 
+    // after 0.8.1: titlebar use jid|name -> titlebar show|hide jid|name
+    if (g_key_file_has_key(prefs, PREF_GROUP_UI, "titlebar.muc.title", NULL)) {
+        char *value = g_key_file_get_string(prefs, PREF_GROUP_UI, "titlebar.muc.title", NULL);
+        if (g_strcmp0(value, "name") == 0) {
+            g_key_file_set_boolean(prefs, PREF_GROUP_UI, "titlebar.muc.title.name", TRUE);
+        } else if (g_strcmp0(value, "jid") == 0) {
+            g_key_file_set_boolean(prefs, PREF_GROUP_UI, "titlebar.muc.title.jid", TRUE);
+        }
+    }
+
     _save_prefs();
 
     boolean_choice_ac = autocomplete_new();
@@ -166,9 +176,23 @@ void _prefs_load(void)
     g_strfreev(triggers);
 }
 
+/* Clean up after _prefs_load() */
+static void _prefs_close(void)
+{
+    autocomplete_free(boolean_choice_ac);
+    autocomplete_free(room_trigger_ac);
+}
+
 void
 prefs_reload(void)
 {
+    /*
+     * Current function contains copy-paste, but we wanted to avoid config_file
+     * manipulation from prefs_load/prefs_close
+     */
+
+    _prefs_close();
+
     g_key_file_free(prefs);
     prefs = NULL;
 
@@ -185,7 +209,7 @@ prefs_load(char *config_file)
     if (config_file == NULL) {
         prefs_loc = files_get_config_path(FILE_PROFRC);
     } else {
-        prefs_loc = config_file;
+        prefs_loc = g_strdup(config_file);
     }
 
     if (g_file_test(prefs_loc, G_FILE_TEST_EXISTS)) {
@@ -207,13 +231,12 @@ prefs_save(void)
 void
 prefs_close(void)
 {
-    autocomplete_free(boolean_choice_ac);
-    autocomplete_free(room_trigger_ac);
+    _prefs_close();
 
     g_key_file_free(prefs);
     prefs = NULL;
 
-    free(prefs_loc);
+    g_free(prefs_loc);
     prefs_loc = NULL;
 }
 
@@ -868,82 +891,67 @@ prefs_get_roster_size(void)
     }
 }
 
-char
+static char*
+_prefs_get_encryption_char(const char *const ch, const char *const pref_group, const char *const key)
+{
+    char *result = NULL;
+
+    char *resultstr = g_key_file_get_string(prefs, pref_group, key, NULL);
+    if (!resultstr) {
+        result = strdup(ch);
+    } else {
+        result = resultstr;
+    }
+
+    return result;
+}
+
+static gboolean
+_prefs_set_encryption_char(const char *const ch, const char *const pref_group, const char *const key)
+{
+    if (g_utf8_strlen(ch, 4) == 1) {
+        g_key_file_set_string(prefs, pref_group, key, ch);
+        return TRUE;
+    } else {
+        log_error("Could not set %s encryption char to: %s", key, ch);
+    }
+    return FALSE;
+}
+
+char*
 prefs_get_otr_char(void)
 {
-    char result = '~';
-
-    char *resultstr = g_key_file_get_string(prefs, PREF_GROUP_OTR, "otr.char", NULL);
-    if (!resultstr) {
-        result =  '~';
-    } else {
-        result = resultstr[0];
-    }
-    free(resultstr);
-
-    return result;
+    return _prefs_get_encryption_char("~", PREF_GROUP_OTR, "otr.char");
 }
 
-void
-prefs_set_otr_char(char ch)
+gboolean
+prefs_set_otr_char(char *ch)
 {
-    char str[2];
-    str[0] = ch;
-    str[1] = '\0';
-
-    g_key_file_set_string(prefs, PREF_GROUP_OTR, "otr.char", str);
+    return _prefs_set_encryption_char(ch, PREF_GROUP_OTR, "otr.char");
 }
 
-char
+char*
 prefs_get_pgp_char(void)
 {
-    char result = '~';
-
-    char *resultstr = g_key_file_get_string(prefs, PREF_GROUP_PGP, "pgp.char", NULL);
-    if (!resultstr) {
-        result =  '~';
-    } else {
-        result = resultstr[0];
-    }
-    free(resultstr);
-
-    return result;
+    return _prefs_get_encryption_char("~", PREF_GROUP_PGP, "pgp.char");
 }
 
-void
-prefs_set_pgp_char(char ch)
+gboolean
+prefs_set_pgp_char(char *ch)
 {
-    char str[2];
-    str[0] = ch;
-    str[1] = '\0';
-
-    g_key_file_set_string(prefs, PREF_GROUP_PGP, "pgp.char", str);
+    return _prefs_set_encryption_char(ch, PREF_GROUP_PGP, "pgp.char");
 }
 
-char
+char*
 prefs_get_omemo_char(void)
 {
-    char result = '~';
-
-    char *resultstr = g_key_file_get_string(prefs, PREF_GROUP_OMEMO, "omemo.char", NULL);
-    if (!resultstr) {
-        result =  '~';
-    } else {
-        result = resultstr[0];
-    }
-    free(resultstr);
-
-    return result;
+    return _prefs_get_encryption_char("~", PREF_GROUP_OMEMO, "omemo.char");
 }
 
-void
-prefs_set_omemo_char(char ch)
+gboolean
+prefs_set_omemo_char(char *ch)
 {
-    char str[2];
-    str[0] = ch;
-    str[1] = '\0';
-
-    g_key_file_set_string(prefs, PREF_GROUP_OMEMO, "omemo.char", str);
+    return _prefs_set_encryption_char(ch, PREF_GROUP_OMEMO, "omemo.char");
 }
 
 char
@@ -1201,6 +1209,31 @@ prefs_set_roster_presence_indent(gint value)
     g_key_file_set_integer(prefs, PREF_GROUP_UI, "roster.presence.indent", value);
 }
 
+char*
+prefs_get_correction_char(void)
+{
+    char *result = "+";
+
+    char *resultstr = g_key_file_get_string(prefs, PREF_GROUP_UI, "correction.char", NULL);
+    if (!resultstr) {
+        result = strdup("+");
+    } else {
+        result = resultstr;
+    }
+
+    return result;
+}
+
+void
+prefs_set_correction_char(char ch)
+{
+    char str[2];
+    str[0] = ch;
+    str[1] = '\0';
+
+    g_key_file_set_string(prefs, PREF_GROUP_UI, "correction.char", str);
+}
+
 gboolean
 prefs_add_room_notify_trigger(const char * const text)
 {
@@ -1632,7 +1665,7 @@ _save_prefs(void)
 {
     gsize g_data_size;
     gchar *g_prefs_data = g_key_file_to_data(prefs, &g_data_size, NULL);
-    gchar *base = g_path_get_basename(prefs_loc);
+    gchar *base = g_path_get_dirname(prefs_loc);
     gchar *true_loc = get_file_or_linked(prefs_loc, base);
 
     g_file_set_contents(true_loc, g_prefs_data, g_data_size, NULL);
@@ -1712,14 +1745,18 @@ _get_group(preference_t pref)
         case PREF_CONSOLE_PRIVATE:
         case PREF_CONSOLE_CHAT:
         case PREF_COLOR_NICK:
+        case PREF_COLOR_NICK_OWN:
         case PREF_ROSTER_COLOR_NICK:
         case PREF_OCCUPANTS_COLOR_NICK:
         case PREF_STATUSBAR_SHOW_NAME:
         case PREF_STATUSBAR_SHOW_NUMBER:
+        case PREF_STATUSBAR_SHOW_READ:
         case PREF_STATUSBAR_SELF:
         case PREF_STATUSBAR_CHAT:
         case PREF_STATUSBAR_ROOM:
-        case PREF_TITLEBAR_MUC_TITLE:
+        case PREF_TITLEBAR_MUC_TITLE_JID:
+        case PREF_TITLEBAR_MUC_TITLE_NAME:
+        case PREF_SLASH_GUARD:
             return PREF_GROUP_UI;
         case PREF_STATES:
         case PREF_OUTTYPE:
@@ -1746,6 +1783,8 @@ _get_group(preference_t pref)
         case PREF_GRLOG:
         case PREF_LOG_ROTATE:
         case PREF_LOG_SHARED:
+        case PREF_AVATAR_CMD:
+        case PREF_URL_OPEN_CMD:
             return PREF_GROUP_LOGGING;
         case PREF_AUTOAWAY_CHECK:
         case PREF_AUTOAWAY_MODE:
@@ -1760,11 +1799,15 @@ _get_group(preference_t pref)
         case PREF_RECEIPTS_REQUEST:
         case PREF_REVEAL_OS:
         case PREF_TLS_CERTPATH:
+        case PREF_CORRECTION_ALLOW:
+        case PREF_MAM:
             return PREF_GROUP_CONNECTION;
         case PREF_OTR_LOG:
         case PREF_OTR_POLICY:
+        case PREF_OTR_SENDFILE:
             return PREF_GROUP_OTR;
         case PREF_PGP_LOG:
+        case PREF_PGP_SENDFILE:
             return PREF_GROUP_PGP;
         case PREF_BOOKMARK_INVITE:
         case PREF_ROOM_LIST_CACHE:
@@ -1773,6 +1816,7 @@ _get_group(preference_t pref)
             return PREF_GROUP_PLUGINS;
         case PREF_OMEMO_LOG:
         case PREF_OMEMO_POLICY:
+        case PREF_OMEMO_SENDFILE:
             return PREF_GROUP_OMEMO;
         default:
             return NULL;
@@ -1888,6 +1932,8 @@ _get_key(preference_t pref)
             return "log";
         case PREF_OTR_POLICY:
             return "policy";
+        case PREF_OTR_SENDFILE:
+            return "sendfile";
         case PREF_LOG_ROTATE:
             return "rotate";
         case PREF_LOG_SHARED:
@@ -1968,10 +2014,14 @@ _get_key(preference_t pref)
             return "inpblock.dynamic";
         case PREF_ENC_WARN:
             return "enc.warn";
-        case PREF_TITLEBAR_MUC_TITLE:
-            return "titlebar.muc.title";
+        case PREF_TITLEBAR_MUC_TITLE_JID:
+            return "titlebar.muc.title.jid";
+        case PREF_TITLEBAR_MUC_TITLE_NAME:
+            return "titlebar.muc.title.name";
         case PREF_PGP_LOG:
             return "log";
+        case PREF_PGP_SENDFILE:
+            return "sendfile";
         case PREF_TLS_CERTPATH:
             return "tls.certpath";
         case PREF_TLS_SHOW:
@@ -1986,6 +2036,8 @@ _get_key(preference_t pref)
             return "console.chat";
         case PREF_COLOR_NICK:
             return "color.nick";
+        case PREF_COLOR_NICK_OWN:
+            return "color.nick.own";
         case PREF_ROSTER_COLOR_NICK:
             return "color.roster.nick";
         case PREF_OCCUPANTS_COLOR_NICK:
@@ -2000,6 +2052,8 @@ _get_key(preference_t pref)
             return "statusbar.show.name";
         case PREF_STATUSBAR_SHOW_NUMBER:
             return "statusbar.show.number";
+        case PREF_STATUSBAR_SHOW_READ:
+            return "statusbar.show.read";
         case PREF_STATUSBAR_SELF:
             return "statusbar.self";
         case PREF_STATUSBAR_CHAT:
@@ -2010,6 +2064,18 @@ _get_key(preference_t pref)
             return "log";
         case PREF_OMEMO_POLICY:
             return "policy";
+        case PREF_OMEMO_SENDFILE:
+            return "sendfile";
+        case PREF_CORRECTION_ALLOW:
+            return "correction.allow";
+        case PREF_AVATAR_CMD:
+            return "avatar.cmd";
+        case PREF_SLASH_GUARD:
+            return "slashguard";
+        case PREF_MAM:
+            return "mam";
+        case PREF_URL_OPEN_CMD:
+            return "urlopen.cmd";
         default:
             return NULL;
     }
@@ -2059,7 +2125,15 @@ _get_default_boolean(preference_t pref)
         case PREF_BOOKMARK_INVITE:
         case PREF_ROOM_LIST_CACHE:
         case PREF_STATUSBAR_SHOW_NUMBER:
+        case PREF_STATUSBAR_SHOW_READ:
         case PREF_REVEAL_OS:
+        case PREF_CORRECTION_ALLOW:
+        case PREF_RECEIPTS_SEND:
+        case PREF_CARBONS:
+        case PREF_STATES:
+        case PREF_OUTTYPE:
+        case PREF_TITLEBAR_MUC_TITLE_NAME:
+        case PREF_COLOR_NICK_OWN:
             return TRUE;
         default:
             return FALSE;
@@ -2097,8 +2171,6 @@ _get_default_string(preference_t pref)
             return "none";
         case PREF_ROSTER_ROOMS_USE_AS_NAME:
             return "name";
-        case PREF_TITLEBAR_MUC_TITLE:
-            return "name";
         case PREF_ROSTER_ROOMS_ORDER:
             return "name";
         case PREF_ROSTER_ROOMS_UNREAD:
@@ -2139,6 +2211,9 @@ _get_default_string(preference_t pref)
             return "automatic";
         case PREF_COLOR_NICK:
             return "false";
+        case PREF_AVATAR_CMD:
+        case PREF_URL_OPEN_CMD:
+            return "xdg-open";
         default:
             return NULL;
     }
