@@ -56,6 +56,7 @@ typedef enum {
     THEME_TIME,
     THEME_TITLE_TEXT,
     THEME_TITLE_BRACKET,
+    THEME_TITLE_SCROLLED,
     THEME_TITLE_UNENCRYPTED,
     THEME_TITLE_ENCRYPTED,
     THEME_TITLE_UNTRUSTED,
@@ -139,6 +140,7 @@ typedef enum {
     THEME_MAGENTA_BOLD,
     THEME_TEXT_HISTORY,
     THEME_CMD_WINS_UNREAD,
+    THEME_TRACKBAR,
 } theme_item_t;
 
 void theme_init(const char *const theme_name);
