@@ -39,6 +39,8 @@
 #include "xmpp/roster_list.h"
 #include "xmpp/muc.h"
 #include "xmpp/xmpp.h"
+#include "database.h"
+#include "tools/bookmark_ignore.h"
 
 #ifdef HAVE_LIBGPGME
 #include "pgp/gpg.h"
@@ -67,6 +69,8 @@ ev_disconnect_cleanup(void)
 #ifdef HAVE_OMEMO
     omemo_on_disconnect();
 #endif
+    log_database_close();
+    bookmark_ignore_on_disconnect();
 }
 
 gboolean
