@@ -42,6 +42,7 @@
 
 #include "profanity.h"
 #include "log.h"
+#include "database.h"
 #include "config/preferences.h"
 #include "config/tlscerts.h"
 #include "config/account.h"
@@ -50,6 +51,7 @@
 #include "event/common.h"
 #include "plugins/plugins.h"
 #include "ui/window_list.h"
+#include "tools/bookmark_ignore.h"
 #include "xmpp/xmpp.h"
 #include "xmpp/muc.h"
 #include "xmpp/chat_session.h"
@@ -77,6 +79,8 @@ sv_ev_login_account_success(char *account_name, gboolean secured)
 {
     ProfAccount *account = accounts_get_account(account_name);
 
+    bookmark_ignore_on_connect(account->jid);
+
     roster_create();
 
 #ifdef HAVE_LIBOTR
@@ -91,6 +95,8 @@ sv_ev_login_account_success(char *account_name, gboolean secured)
     omemo_on_connect(account);
 #endif
 
+    log_database_init(account);
+
     avatar_pep_subscribe();
 
     ui_handle_login_account_success(account, secured);
@@ -127,11 +133,12 @@ sv_ev_login_account_success(char *account_name, gboolean secured)
 void
 sv_ev_roster_received(void)
 {
+    roster_process_pending_presence();
+
     if (prefs_get_boolean(PREF_ROSTER)) {
         ui_show_roster();
     }
 
-    roster_process_pending_presence();
     char *account_name = session_get_account_name();
 
 #ifdef HAVE_LIBGPGME
@@ -144,13 +151,12 @@ sv_ev_roster_received(void)
         }
         free(err_str);
 
-        // Redraw the screen after entry of the PGP secret key
+        // Redraw the screen after entry of the PGP secret key, but not init
         ProfWin *win = wins_get_current();
         char *theme = prefs_get_string(PREF_THEME);
         win_clear(win);
         theme_init(theme);
         prefs_free_string(theme);
-        ui_init();
         ui_resize();
         ui_show_roster();
     }
@@ -272,7 +278,7 @@ sv_ev_room_subject(const char *const room, const char *const nick, const char *c
 void
 sv_ev_room_history(ProfMessage *message)
 {
-    ProfMucWin *mucwin = wins_get_muc(message->jid->barejid);
+    ProfMucWin *mucwin = wins_get_muc(message->from_jid->barejid);
     if (mucwin) {
         // if this is the first successful connection
         if (ev_is_first_connect()) {
@@ -286,7 +292,7 @@ sv_ev_room_history(ProfMessage *message)
 
         gboolean younger = g_date_time_compare(mucwin->last_msg_timestamp, message->timestamp) < 0 ? TRUE : FALSE;
         if (ev_is_first_connect() || younger ) {
-            mucwin_history(mucwin, message->jid->resourcepart, message->timestamp, message->plain);
+            mucwin_history(mucwin, message);
         }
     }
 }
@@ -294,16 +300,17 @@ sv_ev_room_history(ProfMessage *message)
 static void _log_muc(ProfMessage *message)
 {
     if (message->enc == PROF_MSG_ENC_OMEMO) {
-        groupchat_log_omemo_msg_in(message->jid->barejid, message->jid->resourcepart, message->plain);
+        groupchat_log_omemo_msg_in(message->from_jid->barejid, message->from_jid->resourcepart, message->plain);
     } else {
-        groupchat_log_msg_in(message->jid->barejid, message->jid->resourcepart, message->plain);
+        groupchat_log_msg_in(message->from_jid->barejid, message->from_jid->resourcepart, message->plain);
     }
+    log_database_add_incoming(message);
 }
 
 void
 sv_ev_room_message(ProfMessage *message)
 {
-    ProfMucWin *mucwin = wins_get_muc(message->jid->barejid);
+    ProfMucWin *mucwin = wins_get_muc(message->from_jid->barejid);
     if (!mucwin) {
         return;
     }
@@ -312,28 +319,19 @@ sv_ev_room_message(ProfMessage *message)
 
     // only log message not coming from this client (but maybe same account, different client)
     // our messages are logged when outgoing
-    if (!(g_strcmp0(mynick, message->jid->resourcepart) == 0 && message_is_sent_by_us(message, TRUE))) {
+    if (!(g_strcmp0(mynick, message->from_jid->resourcepart) == 0 && message_is_sent_by_us(message, TRUE))) {
         _log_muc(message);
     }
 
     char *old_plain = message->plain;
-    message->plain = plugins_pre_room_message_display(message->jid->barejid, message->jid->resourcepart, message->plain);
+    message->plain = plugins_pre_room_message_display(message->from_jid->barejid, message->from_jid->resourcepart, message->plain);
 
-    gboolean whole_word = prefs_get_boolean(PREF_NOTIFY_MENTION_WHOLE_WORD);
-    gboolean case_sensitive = prefs_get_boolean(PREF_NOTIFY_MENTION_CASE_SENSITIVE);
-    char *message_search = case_sensitive ? strdup(message->plain) : g_utf8_strdown(message->plain, -1);
-    char *mynick_search = case_sensitive ? strdup(mynick) : g_utf8_strdown(mynick, -1);
-
-    GSList *mentions = NULL;
-    mentions = prof_occurrences(mynick_search, message_search, 0, whole_word, &mentions);
+    GSList *mentions = get_mentions(prefs_get_boolean(PREF_NOTIFY_MENTION_WHOLE_WORD), prefs_get_boolean(PREF_NOTIFY_MENTION_CASE_SENSITIVE), message->plain, mynick);
     gboolean mention = g_slist_length(mentions) > 0;
-    g_free(message_search);
-    g_free(mynick_search);
-
     GList *triggers = prefs_message_get_triggers(message->plain);
 
     _clean_incoming_message(message);
-    mucwin_incoming_msg(mucwin, message, mentions, triggers);
+    mucwin_incoming_msg(mucwin, message, mentions, triggers, TRUE);
 
     g_slist_free(mentions);
 
@@ -346,7 +344,7 @@ sv_ev_room_message(ProfMessage *message)
         is_current = TRUE;
         status_bar_active(num, WIN_MUC, mucwin->roomjid);
 
-        if ((g_strcmp0(mynick, message->jid->resourcepart) != 0) && (prefs_get_boolean(PREF_BEEP))) {
+        if ((g_strcmp0(mynick, message->from_jid->resourcepart) != 0) && (prefs_get_boolean(PREF_BEEP))) {
             beep();
         }
 
@@ -354,11 +352,11 @@ sv_ev_room_message(ProfMessage *message)
     } else {
         status_bar_new(num, WIN_MUC, mucwin->roomjid);
 
-        if ((g_strcmp0(mynick, message->jid->resourcepart) != 0) && (prefs_get_boolean(PREF_FLASH))) {
+        if ((g_strcmp0(mynick, message->from_jid->resourcepart) != 0) && (prefs_get_boolean(PREF_FLASH))) {
             flash();
         }
 
-        cons_show_incoming_room_message(message->jid->resourcepart, mucwin->roomjid, num, mention, triggers, mucwin->unread);
+        cons_show_incoming_room_message(message->from_jid->resourcepart, mucwin->roomjid, num, mention, triggers, mucwin->unread);
 
         mucwin->unread++;
 
@@ -376,9 +374,9 @@ sv_ev_room_message(ProfMessage *message)
     }
     mucwin->last_msg_timestamp  = g_date_time_new_now_local();
 
-    if (prefs_do_room_notify(is_current, mucwin->roomjid, mynick, message->jid->resourcepart, message->plain, mention, triggers != NULL)) {
+    if (prefs_do_room_notify(is_current, mucwin->roomjid, mynick, message->from_jid->resourcepart, message->plain, mention, triggers != NULL)) {
         Jid *jidp = jid_create(mucwin->roomjid);
-        notify_room_message(message->jid->resourcepart, jidp->localpart, num, message->plain);
+        notify_room_message(message->from_jid->resourcepart, jidp->localpart, num, message->plain);
         jid_destroy(jidp);
     }
 
@@ -388,7 +386,7 @@ sv_ev_room_message(ProfMessage *message)
 
     rosterwin_roster();
 
-    plugins_post_room_message_display(message->jid->barejid, message->jid->resourcepart, message->plain);
+    plugins_post_room_message_display(message->from_jid->barejid, message->from_jid->resourcepart, message->plain);
     free(message->plain);
     message->plain = old_plain;
 }
@@ -397,19 +395,20 @@ void
 sv_ev_incoming_private_message(ProfMessage *message)
 {
     char *old_plain = message->plain;
-    message->plain = plugins_pre_priv_message_display(message->jid->fulljid, message->plain);
+    message->plain = plugins_pre_priv_message_display(message->from_jid->fulljid, message->plain);
 
-    ProfPrivateWin *privatewin = wins_get_private(message->jid->fulljid);
+    ProfPrivateWin *privatewin = wins_get_private(message->from_jid->fulljid);
     if (privatewin == NULL) {
-        ProfWin *window = wins_new_private(message->jid->fulljid);
+        ProfWin *window = wins_new_private(message->from_jid->fulljid);
         privatewin = (ProfPrivateWin*)window;
     }
 
     _clean_incoming_message(message);
     privwin_incoming_msg(privatewin, message);
+    log_database_add_incoming(message);
     chat_log_msg_in(message);
 
-    plugins_post_priv_message_display(message->jid->fulljid, message->plain);
+    plugins_post_priv_message_display(message->from_jid->fulljid, message->plain);
 
     free(message->plain);
     message->plain = old_plain;
@@ -420,11 +419,11 @@ void
 sv_ev_delayed_private_message(ProfMessage *message)
 {
     char *old_plain = message->plain;
-    message->plain = plugins_pre_priv_message_display(message->jid->fulljid, message->plain);
+    message->plain = plugins_pre_priv_message_display(message->from_jid->fulljid, message->plain);
 
-    ProfPrivateWin *privatewin = wins_get_private(message->jid->fulljid);
+    ProfPrivateWin *privatewin = wins_get_private(message->from_jid->fulljid);
     if (privatewin == NULL) {
-        ProfWin *window = wins_new_private(message->jid->fulljid);
+        ProfWin *window = wins_new_private(message->from_jid->fulljid);
         privatewin = (ProfPrivateWin*)window;
     }
 
@@ -432,7 +431,7 @@ sv_ev_delayed_private_message(ProfMessage *message)
     privwin_incoming_msg(privatewin, message);
     chat_log_msg_in(message);
 
-    plugins_post_priv_message_display(message->jid->fulljid, message->plain);
+    plugins_post_priv_message_display(message->from_jid->fulljid, message->plain);
 
     free(message->plain);
     message->plain = old_plain;
@@ -441,20 +440,21 @@ sv_ev_delayed_private_message(ProfMessage *message)
 void
 sv_ev_outgoing_carbon(ProfMessage *message)
 {
-    ProfChatWin *chatwin = wins_get_chat(message->jid->barejid);
+    ProfChatWin *chatwin = wins_get_chat(message->to_jid->barejid);
     if (!chatwin) {
-        chatwin = chatwin_new(message->jid->barejid);
+        chatwin = chatwin_new(message->to_jid->barejid);
     }
 
     chat_state_active(chatwin->state);
 
     if (message->plain) {
-        if (message->mucuser) {
+        if (message->type == PROF_MSG_TYPE_MUCPM) {
             // MUC PM, should have resource (nick) in filename
-            chat_log_msg_out(message->jid->barejid, message->plain, message->jid->resourcepart);
+            chat_log_msg_out(message->to_jid->barejid, message->plain, message->from_jid->resourcepart);
         } else {
-            chat_log_msg_out(message->jid->barejid, message->plain, NULL);
+            chat_log_msg_out(message->to_jid->barejid, message->plain, NULL);
         }
+        log_database_add_incoming(message);
     }
 
 #ifdef HAVE_LIBGPGME
@@ -469,12 +469,12 @@ sv_ev_outgoing_carbon(ProfMessage *message)
                 log_error("Couldn't decrypt GPG message and body was empty");
                 return;
             }
-            message->enc = PROF_MSG_ENC_PLAIN;
+            message->enc = PROF_MSG_ENC_NONE;
             message->plain = strdup(message->body);
             chatwin_outgoing_carbon(chatwin, message);
         }
     } else {
-        message->enc = PROF_MSG_ENC_PLAIN;
+        message->enc = PROF_MSG_ENC_NONE;
         message->plain = strdup(message->body);
         chatwin_outgoing_carbon(chatwin, message);
     }
@@ -487,7 +487,7 @@ sv_ev_outgoing_carbon(ProfMessage *message)
     if (message->enc == PROF_MSG_ENC_OMEMO) {
         chatwin_outgoing_carbon(chatwin, message);
     } else {
-        message->enc = PROF_MSG_ENC_PLAIN;
+        message->enc = PROF_MSG_ENC_NONE;
         message->plain = strdup(message->body);
         chatwin_outgoing_carbon(chatwin, message);
     }
@@ -509,12 +509,12 @@ sv_ev_outgoing_carbon(ProfMessage *message)
                 log_error("Couldn't decrypt GPG message and body was empty");
                 return;
             }
-            message->enc = PROF_MSG_ENC_PLAIN;
+            message->enc = PROF_MSG_ENC_NONE;
             message->plain = strdup(message->body);
             chatwin_outgoing_carbon(chatwin, message);
         }
     } else {
-        message->enc = PROF_MSG_ENC_PLAIN;
+        message->enc = PROF_MSG_ENC_NONE;
         message->plain = strdup(message->body);
         chatwin_outgoing_carbon(chatwin, message);
     }
@@ -525,7 +525,7 @@ sv_ev_outgoing_carbon(ProfMessage *message)
 #ifndef HAVE_LIBGPGME
 #ifndef HAVE_OMEMO
     if (message->body) {
-        message->enc = PROF_MSG_ENC_PLAIN;
+        message->enc = PROF_MSG_ENC_NONE;
         message->plain = strdup(message->body);
         chatwin_outgoing_carbon(chatwin, message);
     }
@@ -542,6 +542,7 @@ _sv_ev_incoming_pgp(ProfChatWin *chatwin, gboolean new_win, ProfMessage *message
         message->enc = PROF_MSG_ENC_PGP;
         _clean_incoming_message(message);
         chatwin_incoming_msg(chatwin, message, new_win);
+        log_database_add_incoming(message);
         if (logit) {
             chat_log_pgp_msg_in(message);
         }
@@ -553,10 +554,11 @@ _sv_ev_incoming_pgp(ProfChatWin *chatwin, gboolean new_win, ProfMessage *message
             log_error("Couldn't decrypt GPG message and body was empty");
             return;
         }
-        message->enc = PROF_MSG_ENC_PLAIN;
+        message->enc = PROF_MSG_ENC_NONE;
         message->plain = strdup(message->body);
         _clean_incoming_message(message);
         chatwin_incoming_msg(chatwin, message, new_win);
+        log_database_add_incoming(message);
         chat_log_msg_in(message);
         chatwin->pgp_recv = FALSE;
     }
@@ -568,17 +570,18 @@ static void
 _sv_ev_incoming_otr(ProfChatWin *chatwin, gboolean new_win, ProfMessage *message)
 {
     gboolean decrypted = FALSE;
-    message->plain = otr_on_message_recv(message->jid->barejid, message->jid->resourcepart, message->body, &decrypted);
+    message->plain = otr_on_message_recv(message->from_jid->barejid, message->from_jid->resourcepart, message->body, &decrypted);
     if (message->plain) {
         if (decrypted) {
             message->enc = PROF_MSG_ENC_OTR;
             chatwin->pgp_send = FALSE;
         } else {
-            message->enc = PROF_MSG_ENC_PLAIN;
+            message->enc = PROF_MSG_ENC_NONE;
         }
 
         _clean_incoming_message(message);
         chatwin_incoming_msg(chatwin, message, new_win);
+        log_database_add_incoming(message);
 
         chat_log_otr_msg_in(message);
         otr_free_message(message->plain);
@@ -594,6 +597,7 @@ _sv_ev_incoming_omemo(ProfChatWin *chatwin, gboolean new_win, ProfMessage *messa
 {
     _clean_incoming_message(message);
     chatwin_incoming_msg(chatwin, message, new_win);
+    log_database_add_incoming(message);
     if (logit) {
         chat_log_omemo_msg_in(message);
     }
@@ -605,10 +609,11 @@ static void
 _sv_ev_incoming_plain(ProfChatWin *chatwin, gboolean new_win, ProfMessage *message, gboolean logit)
 {
     if (message->body) {
-        message->enc = PROF_MSG_ENC_PLAIN;
+        message->enc = PROF_MSG_ENC_NONE;
         message->plain = strdup(message->body);
         _clean_incoming_message(message);
         chatwin_incoming_msg(chatwin, message, new_win);
+        log_database_add_incoming(message);
         if (logit) {
             chat_log_msg_in(message);
         }
@@ -620,16 +625,30 @@ void
 sv_ev_incoming_message(ProfMessage *message)
 {
     gboolean new_win = FALSE;
-    ProfChatWin *chatwin = wins_get_chat(message->jid->barejid);
+    ProfChatWin *chatwin;
+    char *looking_for_jid = message->from_jid->barejid;
+
+    if (message->is_mam) {
+        char *mybarejid = connection_get_barejid();
+        if (g_strcmp0(mybarejid, message->from_jid->barejid) == 0) {
+            looking_for_jid = message->to_jid->barejid;
+        }
+        free(mybarejid);
+    }
+
+    chatwin = wins_get_chat(looking_for_jid);
+
     if (!chatwin) {
-        ProfWin *window = wins_new_chat(message->jid->barejid);
+        ProfWin *window = wins_new_chat(looking_for_jid);
         chatwin = (ProfChatWin*)window;
         new_win = TRUE;
 
 #ifdef HAVE_OMEMO
-        if (omemo_automatic_start(message->jid->barejid)) {
-            omemo_start_session(message->jid->barejid);
-            chatwin->is_omemo = TRUE;
+        if (!message->is_mam) {
+            if (omemo_automatic_start(message->from_jid->barejid)) {
+                omemo_start_session(message->from_jid->barejid);
+                chatwin->is_omemo = TRUE;
+            }
         }
 #endif
     }
@@ -640,7 +659,7 @@ sv_ev_incoming_message(ProfMessage *message)
 #ifndef HAVE_OMEMO
     if (message->encrypted) {
         if (chatwin->is_otr) {
-            win_println((ProfWin*)chatwin, THEME_DEFAULT, '-', "PGP encrypted message received whilst in OTR session.");
+            win_println((ProfWin*)chatwin, THEME_DEFAULT, "-", "PGP encrypted message received whilst in OTR session.");
         } else {
             _sv_ev_incoming_pgp(chatwin, new_win, message, TRUE);
         }
@@ -685,7 +704,7 @@ sv_ev_incoming_message(ProfMessage *message)
 #ifdef HAVE_OMEMO
     if (message->encrypted) {
         if (chatwin->is_otr) {
-            win_println((ProfWin*)chatwin, THEME_DEFAULT, '-', "PGP encrypted message received whilst in OTR session.");
+            win_println((ProfWin*)chatwin, THEME_DEFAULT, "-", "PGP encrypted message received whilst in OTR session.");
         } else {
             _sv_ev_incoming_pgp(chatwin, new_win, message, TRUE);
         }
@@ -763,15 +782,15 @@ void
 sv_ev_incoming_carbon(ProfMessage *message)
 {
     gboolean new_win = FALSE;
-    ProfChatWin *chatwin = wins_get_chat(message->jid->barejid);
+    ProfChatWin *chatwin = wins_get_chat(message->from_jid->barejid);
     if (!chatwin) {
-        ProfWin *window = wins_new_chat(message->jid->barejid);
+        ProfWin *window = wins_new_chat(message->from_jid->barejid);
         chatwin = (ProfChatWin*)window;
         new_win = TRUE;
 
 #ifdef HAVE_OMEMO
-        if (omemo_automatic_start(message->jid->barejid)) {
-            omemo_start_session(message->jid->barejid);
+        if (omemo_automatic_start(message->from_jid->barejid)) {
+            omemo_start_session(message->from_jid->barejid);
             chatwin->is_omemo = TRUE;
         }
 #endif
@@ -1122,7 +1141,7 @@ sv_ev_muc_self_online(const char *const room, const char *const nick, gboolean c
 
         if (muc_invites_contain(room)) {
             if (prefs_get_boolean(PREF_BOOKMARK_INVITE) && !bookmark_exists(room)) {
-                bookmark_add(room, nick, muc_invite_password(room), "on");
+                bookmark_add(room, nick, muc_invite_password(room), "on", NULL);
             }
             muc_invites_remove(room);
         }
@@ -1414,7 +1433,12 @@ sv_ev_lastactivity_response(const char *const from, const int seconds, const cha
 void
 sv_ev_bookmark_autojoin(Bookmark *bookmark)
 {
+    if (bookmark_ignored(bookmark)) {
+        return;
+    }
+
     char *nick = NULL;
+
     if (bookmark->nick) {
         nick = strdup(bookmark->nick);
     } else {
