@@ -55,7 +55,7 @@
 #define PROF "prof"
 
 static FILE *logp;
-GString *mainlogfile;
+GString *mainlogfile = NULL;
 
 static GTimeZone *tz;
 static GDateTime *dt;
@@ -144,25 +144,35 @@ log_error(const char *const msg, ...)
 }
 
 void
-log_init(log_level_t filter)
+log_init(log_level_t filter, char *log_file)
 {
     level_filter = filter;
     tz = g_time_zone_new_local();
-    char *log_file = files_get_log_file();
-    logp = fopen(log_file, "a");
-    g_chmod(log_file, S_IRUSR | S_IWUSR);
-    mainlogfile = g_string_new(log_file);
-    free(log_file);
+
+    gchar *lf;
+    lf = files_get_log_file(log_file);
+
+    logp = fopen(lf, "a");
+    g_chmod(lf, S_IRUSR | S_IWUSR);
+    mainlogfile = g_string_new(lf);
+    g_free(lf);
 }
 
 void
 log_reinit(void)
 {
+    char *lf = strdup(mainlogfile->str);
+    char *start = strrchr(lf, '/') +1;
+    char *end = strstr(start, ".log");
+    *end = '\0';
+
     log_close();
-    log_init(level_filter);
+    log_init(level_filter, start);
+
+    free(lf);
 }
 
-char*
+const char*
 get_log_file_location(void)
 {
     return mainlogfile->str;
@@ -178,6 +188,7 @@ void
 log_close(void)
 {
     g_string_free(mainlogfile, TRUE);
+    mainlogfile = NULL;
     g_time_zone_unref(tz);
     if (logp) {
         fclose(logp);
@@ -229,7 +240,7 @@ log_level_from_string(char *log_level)
 static void
 _rotate_log_file(void)
 {
-    gchar *log_file = files_get_log_file();
+    gchar *log_file = strdup(mainlogfile->str);
     size_t len = strlen(log_file);
     gchar *log_file_new = malloc(len + 4);
     int i = 1;
@@ -241,10 +252,18 @@ _rotate_log_file(void)
             break;
     }
 
-    log_close();
-    rename(log_file, log_file_new);
-    log_init(log_get_filter());
+    char *lf = strdup(mainlogfile->str);
+    char *start = strrchr(lf, '/') +1;
+    char *end = strstr(start, ".log");
+    *end = '\0';
 
+    log_close();
+
+    rename(log_file, log_file_new);
+
+    log_init(log_get_filter(), start);
+
+    free(lf);
     free(log_file_new);
     free(log_file);
     log_info("Log has been rotated");
@@ -271,10 +290,9 @@ void
 chat_log_msg_out(const char *const barejid, const char *const msg, const char *const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
-        _chat_log_chat(jidp->barejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
-        jid_destroy(jidp);
+        char *mybarejid = connection_get_barejid();
+        _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
+        free(mybarejid);
     }
 }
 
@@ -282,16 +300,15 @@ void
 chat_log_otr_msg_out(const char *const barejid, const char *const msg, const char *const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
+        char *mybarejid = connection_get_barejid();
         char *pref_otr_log = prefs_get_string(PREF_OTR_LOG);
         if (strcmp(pref_otr_log, "on") == 0) {
-            _chat_log_chat(jidp->barejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
+            _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
         } else if (strcmp(pref_otr_log, "redact") == 0) {
-            _chat_log_chat(jidp->barejid, barejid, "[redacted]", PROF_OUT_LOG, NULL, resource);
+            _chat_log_chat(mybarejid, barejid, "[redacted]", PROF_OUT_LOG, NULL, resource);
         }
         prefs_free_string(pref_otr_log);
-        jid_destroy(jidp);
+        free(mybarejid);
     }
 }
 
@@ -299,16 +316,15 @@ void
 chat_log_pgp_msg_out(const char *const barejid, const char *const msg, const char *const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
+        char *mybarejid = connection_get_barejid();
         char *pref_pgp_log = prefs_get_string(PREF_PGP_LOG);
         if (strcmp(pref_pgp_log, "on") == 0) {
-            _chat_log_chat(jidp->barejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
+            _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
         } else if (strcmp(pref_pgp_log, "redact") == 0) {
-            _chat_log_chat(jidp->barejid, barejid, "[redacted]", PROF_OUT_LOG, NULL, resource);
+            _chat_log_chat(mybarejid, barejid, "[redacted]", PROF_OUT_LOG, NULL, resource);
         }
         prefs_free_string(pref_pgp_log);
-        jid_destroy(jidp);
+        free(mybarejid);
     }
 }
 
@@ -316,16 +332,15 @@ void
 chat_log_omemo_msg_out(const char *const barejid, const char *const msg, const char *const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
+        char *mybarejid = connection_get_barejid();
         char *pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
         if (strcmp(pref_omemo_log, "on") == 0) {
-            _chat_log_chat(jidp->barejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
+            _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
         } else if (strcmp(pref_omemo_log, "redact") == 0) {
-            _chat_log_chat(jidp->barejid, barejid, "[redacted]", PROF_OUT_LOG, NULL, resource);
+            _chat_log_chat(mybarejid, barejid, "[redacted]", PROF_OUT_LOG, NULL, resource);
         }
         prefs_free_string(pref_omemo_log);
-        jid_destroy(jidp);
+        free(mybarejid);
     }
 }
 
@@ -333,24 +348,23 @@ void
 chat_log_otr_msg_in(ProfMessage *message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
+        char *mybarejid = connection_get_barejid();
         char *pref_otr_log = prefs_get_string(PREF_OTR_LOG);
-        if (message->enc == PROF_MSG_ENC_PLAIN || (strcmp(pref_otr_log, "on") == 0)) {
-            if (message->mucuser) {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->jid->resourcepart);
+        if (message->enc == PROF_MSG_ENC_NONE || (strcmp(pref_otr_log, "on") == 0)) {
+            if (message->type == PROF_MSG_TYPE_MUCPM) {
+                _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
             } else {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, NULL);
+                _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, NULL);
             }
         } else if (strcmp(pref_otr_log, "redact") == 0) {
-            if (message->mucuser) {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, message->jid->resourcepart);
+            if (message->type == PROF_MSG_TYPE_MUCPM) {
+                _chat_log_chat(mybarejid, message->from_jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
             } else {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, NULL);
+                _chat_log_chat(mybarejid, message->from_jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, NULL);
             }
         }
         prefs_free_string(pref_otr_log);
-        jid_destroy(jidp);
+        free(mybarejid);
     }
 }
 
@@ -358,24 +372,23 @@ void
 chat_log_pgp_msg_in(ProfMessage *message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
+        char *mybarejid = connection_get_barejid();
         char *pref_pgp_log = prefs_get_string(PREF_PGP_LOG);
         if (strcmp(pref_pgp_log, "on") == 0) {
-            if (message->mucuser) {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->jid->resourcepart);
+            if (message->type == PROF_MSG_TYPE_MUCPM) {
+                _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
             } else {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, NULL);
+                _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, NULL);
             }
         } else if (strcmp(pref_pgp_log, "redact") == 0) {
-            if (message->mucuser) {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, message->jid->resourcepart);
+            if (message->type == PROF_MSG_TYPE_MUCPM) {
+                _chat_log_chat(mybarejid, message->from_jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
             } else {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, NULL);
+                _chat_log_chat(mybarejid, message->from_jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, NULL);
             }
         }
         prefs_free_string(pref_pgp_log);
-        jid_destroy(jidp);
+        free(mybarejid);
     }
 }
 
@@ -383,24 +396,23 @@ void
 chat_log_omemo_msg_in(ProfMessage *message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
+        char *mybarejid = connection_get_barejid();
         char *pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
         if (strcmp(pref_omemo_log, "on") == 0) {
-            if (message->mucuser) {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->jid->resourcepart);
+            if (message->type == PROF_MSG_TYPE_MUCPM) {
+                _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
             } else {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, NULL);
+                _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, NULL);
             }
         } else if (strcmp(pref_omemo_log, "redact") == 0) {
-            if (message->mucuser) {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, message->jid->resourcepart);
+            if (message->type == PROF_MSG_TYPE_MUCPM) {
+                _chat_log_chat(mybarejid, message->from_jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
             } else {
-                _chat_log_chat(jidp->barejid, message->jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, message->jid->resourcepart);
+                _chat_log_chat(mybarejid, message->from_jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
             }
         }
         prefs_free_string(pref_omemo_log);
-        jid_destroy(jidp);
+        free(mybarejid);
     }
 }
 
@@ -408,15 +420,15 @@ void
 chat_log_msg_in(ProfMessage *message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
+        char *mybarejid = connection_get_barejid();
 
-        if (message->mucuser) {
-            _chat_log_chat(jidp->barejid, message->jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->jid->resourcepart);
+        if (message->type == PROF_MSG_TYPE_MUCPM) {
+            _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
         } else {
-            _chat_log_chat(jidp->barejid, message->jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, NULL);
+            _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, NULL);
         }
-        jid_destroy(jidp);
+
+        free(mybarejid);
     }
 }
 
@@ -466,32 +478,32 @@ _chat_log_chat(const char *const login, const char *const other, const char *con
     }
 
     gchar *date_fmt = g_date_time_format(timestamp, "%H:%M:%S");
-    FILE *logp = fopen(dated_log->filename, "a");
+    FILE *chatlogp = fopen(dated_log->filename, "a");
     g_chmod(dated_log->filename, S_IRUSR | S_IWUSR);
-    if (logp) {
+    if (chatlogp) {
         if (direction == PROF_IN_LOG) {
             if (strncmp(msg, "/me ", 4) == 0) {
                 if (resourcepart) {
-                    fprintf(logp, "%s - *%s %s\n", date_fmt, resourcepart, msg + 4);
+                    fprintf(chatlogp, "%s - *%s %s\n", date_fmt, resourcepart, msg + 4);
                 } else {
-                    fprintf(logp, "%s - *%s %s\n", date_fmt, other, msg + 4);
+                    fprintf(chatlogp, "%s - *%s %s\n", date_fmt, other, msg + 4);
                 }
             } else {
                 if (resourcepart) {
-                    fprintf(logp, "%s - %s: %s\n", date_fmt, resourcepart, msg);
+                    fprintf(chatlogp, "%s - %s: %s\n", date_fmt, resourcepart, msg);
                 } else {
-                    fprintf(logp, "%s - %s: %s\n", date_fmt, other, msg);
+                    fprintf(chatlogp, "%s - %s: %s\n", date_fmt, other, msg);
                 }
             }
         } else {
             if (strncmp(msg, "/me ", 4) == 0) {
-                fprintf(logp, "%s - *me %s\n", date_fmt, msg + 4);
+                fprintf(chatlogp, "%s - *me %s\n", date_fmt, msg + 4);
             } else {
-                fprintf(logp, "%s - me: %s\n", date_fmt, msg);
+                fprintf(chatlogp, "%s - me: %s\n", date_fmt, msg);
             }
         }
-        fflush(logp);
-        int result = fclose(logp);
+        fflush(chatlogp);
+        int result = fclose(chatlogp);
         if (result == EOF) {
             log_error("Error closing file %s, errno = %d", dated_log->filename, errno);
         }
@@ -505,11 +517,10 @@ void
 groupchat_log_msg_out(const gchar *const room, const gchar *const msg)
 {
     if (prefs_get_boolean(PREF_GRLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
+        char *mybarejid = connection_get_barejid();
         char *mynick = muc_nick(room);
-        _groupchat_log_chat(jidp->barejid, room, mynick, msg);
-        jid_destroy(jidp);
+        _groupchat_log_chat(mybarejid, room, mynick, msg);
+        free(mybarejid);
     }
 }
 
@@ -517,10 +528,9 @@ void
 groupchat_log_msg_in(const gchar *const room, const gchar *const nick, const gchar *const msg)
 {
     if (prefs_get_boolean(PREF_GRLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
-        _groupchat_log_chat(jidp->barejid, room, nick, msg);
-        jid_destroy(jidp);
+        char *mybarejid = connection_get_barejid();
+        _groupchat_log_chat(mybarejid, room, nick, msg);
+        free(mybarejid);
     }
 }
 
@@ -528,17 +538,18 @@ void
 groupchat_log_omemo_msg_out(const gchar *const room, const gchar *const msg)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
+        char *mybarejid = connection_get_barejid();
         char *pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
         char *mynick = muc_nick(room);
+
         if (strcmp(pref_omemo_log, "on") == 0) {
-            _groupchat_log_chat(jidp->barejid, room, mynick, msg);
+            _groupchat_log_chat(mybarejid, room, mynick, msg);
         } else if (strcmp(pref_omemo_log, "redact") == 0) {
-            _groupchat_log_chat(jidp->barejid, room, mynick, "[redacted]");
+            _groupchat_log_chat(mybarejid, room, mynick, "[redacted]");
         }
+
         prefs_free_string(pref_omemo_log);
-        jid_destroy(jidp);
+        free(mybarejid);
     }
 }
 
@@ -546,16 +557,17 @@ void
 groupchat_log_omemo_msg_in(const gchar *const room, const gchar *const nick, const gchar *const msg)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        const char *jid = connection_get_fulljid();
-        Jid *jidp = jid_create(jid);
+        char *mybarejid = connection_get_barejid();
         char *pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
+
         if (strcmp(pref_omemo_log, "on") == 0) {
-            _groupchat_log_chat(jidp->barejid, room, nick, msg);
+            _groupchat_log_chat(mybarejid, room, nick, msg);
         } else if (strcmp(pref_omemo_log, "redact") == 0) {
-            _groupchat_log_chat(jidp->barejid, room, nick, "[redacted]");
+            _groupchat_log_chat(mybarejid, room, nick, "[redacted]");
         }
+
         prefs_free_string(pref_omemo_log);
-        jid_destroy(jidp);
+        free(mybarejid);
     }
 }
 
@@ -576,76 +588,28 @@ _groupchat_log_chat(const gchar *const login, const gchar *const room, const gch
         g_hash_table_replace(logs, strdup(room), dated_log);
     }
 
-    GDateTime *dt = g_date_time_new_now_local();
+    GDateTime *dt_tmp = g_date_time_new_now_local();
 
-    gchar *date_fmt = g_date_time_format(dt, "%H:%M:%S");
+    gchar *date_fmt = g_date_time_format(dt_tmp, "%H:%M:%S");
 
-    FILE *logp = fopen(dated_log->filename, "a");
+    FILE *grpchatlogp = fopen(dated_log->filename, "a");
     g_chmod(dated_log->filename, S_IRUSR | S_IWUSR);
-    if (logp) {
+    if (grpchatlogp) {
         if (strncmp(msg, "/me ", 4) == 0) {
-            fprintf(logp, "%s - *%s %s\n", date_fmt, nick, msg + 4);
+            fprintf(grpchatlogp, "%s - *%s %s\n", date_fmt, nick, msg + 4);
         } else {
-            fprintf(logp, "%s - %s: %s\n", date_fmt, nick, msg);
+            fprintf(grpchatlogp, "%s - %s: %s\n", date_fmt, nick, msg);
         }
 
-        fflush(logp);
-        int result = fclose(logp);
+        fflush(grpchatlogp);
+        int result = fclose(grpchatlogp);
         if (result == EOF) {
             log_error("Error closing file %s, errno = %d", dated_log->filename, errno);
         }
     }
 
     g_free(date_fmt);
-    g_date_time_unref(dt);
-}
-
-GSList*
-chat_log_get_previous(const gchar *const login, const gchar *const recipient)
-{
-    GSList *history = NULL;
-    GDateTime *now = g_date_time_new_now_local();
-    GDateTime *log_date = g_date_time_new(tz,
-        g_date_time_get_year(session_started),
-        g_date_time_get_month(session_started),
-        g_date_time_get_day_of_month(session_started),
-        g_date_time_get_hour(session_started),
-        g_date_time_get_minute(session_started),
-        g_date_time_get_second(session_started));
-
-    // get data from all logs from the day the session was started to today
-    while (g_date_time_compare(log_date, now) != 1) {
-        char *filename = _get_log_filename(recipient, login, log_date, FALSE);
-
-        FILE *logp = fopen(filename, "r");
-        if (logp) {
-            GString *header = g_string_new("");
-            g_string_append_printf(header, "%d/%d/%d:",
-                g_date_time_get_day_of_month(log_date),
-                g_date_time_get_month(log_date),
-                g_date_time_get_year(log_date));
-            history = g_slist_append(history, header->str);
-            g_string_free(header, FALSE);
-
-            char *line;
-            while ((line = file_getline(logp)) != NULL) {
-                history = g_slist_append(history, line);
-            }
-
-            fclose(logp);
-        }
-
-        free(filename);
-
-        GDateTime *next = g_date_time_add_days(log_date, 1);
-        g_date_time_unref(log_date);
-        log_date = next;
-    }
-
-    g_date_time_unref(log_date);
-    g_date_time_unref(now);
-
-    return history;
+    g_date_time_unref(dt_tmp);
 }
 
 void
