@@ -118,7 +118,7 @@ omemo_init(void)
 {
     log_info("OMEMO: initialising");
     if (omemo_crypto_init() != 0) {
-        cons_show("Error initializing OMEMO crypto");
+        cons_show("Error initializing OMEMO crypto: gcry_check_version() failed");
     }
 
     pthread_mutexattr_init(&omemo_ctx.attr);
@@ -167,7 +167,7 @@ omemo_on_connect(ProfAccount *account)
     };
 
     if (signal_context_set_crypto_provider(omemo_ctx.signal, &crypto_provider) != 0) {
-        cons_show("Error initializing OMEMO crypto");
+        cons_show("Error initializing OMEMO crypto: unable to set crypto provider");
         return;
     }
 
@@ -373,16 +373,16 @@ omemo_publish_crypto_materials(void)
         return;
     }
 
-    Jid *jid = jid_create(connection_get_fulljid());
+    char *barejid = connection_get_barejid();
 
     /* Ensure we get our current device list, and it gets updated with our
      * device_id */
-    g_hash_table_insert(omemo_ctx.device_list_handler, strdup(jid->barejid), _handle_own_device_list);
-    omemo_devicelist_request(jid->barejid);
+    g_hash_table_insert(omemo_ctx.device_list_handler, strdup(barejid), _handle_own_device_list);
+    omemo_devicelist_request(barejid);
 
     omemo_bundle_publish(true);
 
-    jid_destroy(jid);
+    free(barejid);
 }
 
 void
@@ -671,7 +671,7 @@ out:
 }
 
 char *
-omemo_on_message_send(ProfWin *win, const char *const message, gboolean request_receipt, gboolean muc)
+omemo_on_message_send(ProfWin *win, const char *const message, gboolean request_receipt, gboolean muc, const char *const replace_id)
 {
     char *id = NULL;
     int res;
@@ -809,11 +809,11 @@ omemo_on_message_send(ProfWin *win, const char *const message, gboolean request_
     if (muc) {
         ProfMucWin *mucwin = (ProfMucWin *)win;
         assert(mucwin->memcheck == PROFMUCWIN_MEMCHECK);
-        id = message_send_chat_omemo(mucwin->roomjid, omemo_ctx.device_id, keys, iv, AES128_GCM_IV_LENGTH, ciphertext, ciphertext_len, request_receipt, TRUE);
+        id = message_send_chat_omemo(mucwin->roomjid, omemo_ctx.device_id, keys, iv, AES128_GCM_IV_LENGTH, ciphertext, ciphertext_len, request_receipt, TRUE, replace_id);
     } else {
         ProfChatWin *chatwin = (ProfChatWin *)win;
         assert(chatwin->memcheck == PROFCHATWIN_MEMCHECK);
-        id = message_send_chat_omemo(chatwin->barejid, omemo_ctx.device_id, keys, iv, AES128_GCM_IV_LENGTH, ciphertext, ciphertext_len, request_receipt, FALSE);
+        id = message_send_chat_omemo(chatwin->barejid, omemo_ctx.device_id, keys, iv, AES128_GCM_IV_LENGTH, ciphertext, ciphertext_len, request_receipt, FALSE, replace_id);
     }
 
 out:
