@@ -3,6 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2019 - 2020 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -80,16 +81,16 @@ buffer_free(ProfBuff buffer)
 }
 
 void
-buffer_append(ProfBuff buffer, const char show_char, int pad_indent, GDateTime *time,
-    int flags, theme_item_t theme_item, const char *const from, const char *const message, DeliveryReceipt *receipt, const char *const id)
+buffer_append(ProfBuff buffer, const char *show_char, int pad_indent, GDateTime *time, int flags, theme_item_t theme_item, const char *const display_from, const char *const from_jid, const char *const message, DeliveryReceipt *receipt, const char *const id)
 {
     ProfBuffEntry *e = malloc(sizeof(struct prof_buff_entry_t));
-    e->show_char = show_char;
+    e->show_char = strdup(show_char);
     e->pad_indent = pad_indent;
     e->flags = flags;
     e->theme_item = theme_item;
     e->time = g_date_time_ref(time);
-    e->from = from ? strdup(from) : NULL;
+    e->display_from = display_from ? strdup(display_from) : NULL;
+    e->from_jid = from_jid ? strdup(from_jid) : NULL;
     e->message = strdup(message);
     e->receipt = receipt;
     if (id) {
@@ -113,7 +114,9 @@ buffer_remove_entry_by_id(ProfBuff buffer, const char *const id)
     while (entries) {
         ProfBuffEntry *entry = entries->data;
         if (entry->id && (g_strcmp0(entry->id, id) == 0)) {
+            _free_entry(entry);
             buffer->entries = g_slist_delete_link(buffer->entries, entries);
+            break;
         }
         entries = g_slist_next(entries);
     }
@@ -162,8 +165,10 @@ buffer_get_entry_by_id(ProfBuff buffer, const char *const id)
 static void
 _free_entry(ProfBuffEntry *entry)
 {
+    free(entry->show_char);
     free(entry->message);
-    free(entry->from);
+    free(entry->display_from);
+    free(entry->from_jid);
     free(entry->id);
     free(entry->receipt);
     g_date_time_unref(entry->time);
