@@ -74,7 +74,7 @@ cons_debug(const char *const msg, ...)
         va_start(arg, msg);
         GString *fmt_msg = g_string_new(NULL);
         g_string_vprintf(fmt_msg, msg, arg);
-        win_println(console, THEME_DEFAULT, '-', "%s", fmt_msg->str);
+        win_println(console, THEME_DEFAULT, "-", "%s", fmt_msg->str);
         g_string_free(fmt_msg, TRUE);
         va_end(arg);
     }
@@ -88,7 +88,7 @@ cons_show(const char *const msg, ...)
     va_start(arg, msg);
     GString *fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, msg, arg);
-    win_println(console, THEME_DEFAULT, '-', "%s", fmt_msg->str);
+    win_println(console, THEME_DEFAULT, "-", "%s", fmt_msg->str);
     g_string_free(fmt_msg, TRUE);
     va_end(arg);
 }
@@ -112,8 +112,8 @@ cons_show_help(const char *const cmd, CommandHelp *help)
     ProfWin *console = wins_get_console();
 
     cons_show("");
-    win_println(console, THEME_HELP_HEADER, '-', "%s", &cmd[1]);
-    win_print(console, THEME_HELP_HEADER, '-', "");
+    win_println(console, THEME_HELP_HEADER, "-", "%s", &cmd[1]);
+    win_print(console, THEME_HELP_HEADER, "-", "");
     int i;
     for (i = 0; i < strlen(cmd) - 1 ; i++) {
         win_append(console, THEME_HELP_HEADER, "-");
@@ -121,12 +121,12 @@ cons_show_help(const char *const cmd, CommandHelp *help)
     win_appendln(console, THEME_HELP_HEADER, "");
     cons_show("");
 
-    win_println(console, THEME_HELP_HEADER, '-', "Synopsis");
+    win_println(console, THEME_HELP_HEADER, "-", "Synopsis");
     ui_show_lines(console, help->synopsis);
     cons_show("");
 
-    win_println(console, THEME_HELP_HEADER, '-', "Description");
-    win_println(console, THEME_DEFAULT, '-', "%s", help->desc);
+    win_println(console, THEME_HELP_HEADER, "-", "Description");
+    win_println(console, THEME_DEFAULT, "-", "%s", help->desc);
 
     int maxlen = 0;
     for (i = 0; help->args[i][0] != NULL; i++) {
@@ -136,7 +136,7 @@ cons_show_help(const char *const cmd, CommandHelp *help)
 
     if (i > 0) {
         cons_show("");
-        win_println(console, THEME_HELP_HEADER, '-', "Arguments");
+        win_println(console, THEME_HELP_HEADER, "-", "Arguments");
         for (i = 0; help->args[i][0] != NULL; i++) {
             win_println_indent(console, maxlen + 3, "%-*s: %s", maxlen + 1, help->args[i][0], help->args[i][1]);
         }
@@ -144,7 +144,7 @@ cons_show_help(const char *const cmd, CommandHelp *help)
 
     if (g_strv_length((gchar**)help->examples) > 0) {
         cons_show("");
-        win_println(console, THEME_HELP_HEADER, '-', "Examples");
+        win_println(console, THEME_HELP_HEADER, "-", "Examples");
         ui_show_lines(console, help->examples);
     }
 }
@@ -169,7 +169,7 @@ cons_show_error(const char *const msg, ...)
     va_start(arg, msg);
     GString *fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, msg, arg);
-    win_println(console, THEME_ERROR, '-', "%s", fmt_msg->str);
+    win_println(console, THEME_ERROR, "-", "%s", fmt_msg->str);
     g_string_free(fmt_msg, TRUE);
     va_end(arg);
 
@@ -284,7 +284,7 @@ cons_show_typing(const char *const barejid)
         display_usr = barejid;
     }
 
-    win_println(console, THEME_TYPING, '-', "!! %s is typing a message...", display_usr);
+    win_println(console, THEME_TYPING, "-", "!! %s is typing a message...", display_usr);
     cons_alert();
 }
 
@@ -321,27 +321,27 @@ cons_show_incoming_room_message(const char *const nick, const char *const room,
 
     if (g_strcmp0(muc_show, "all") == 0) {
         if (mention) {
-            win_println(console, THEME_MENTION, '-', "<< room mention: %s in %s (win %d)", nick, room, ui_index);
+            win_println(console, THEME_MENTION, "-", "<< room mention: %s in %s (win %d)", nick, room, ui_index);
         } else if (triggers) {
             char *triggers_str = _room_triggers_to_string(triggers);
-            win_println(console, THEME_TRIGGER, '-', "<< room trigger %s: %s in %s (win %d)", triggers_str, nick, room, ui_index);
+            win_println(console, THEME_TRIGGER, "-", "<< room trigger %s: %s in %s (win %d)", triggers_str, nick, room, ui_index);
             free(triggers_str);
         } else {
-            win_println(console, THEME_INCOMING, '-', "<< room message: %s in %s (win %d)", nick, room, ui_index);
+            win_println(console, THEME_INCOMING, "-", "<< room message: %s in %s (win %d)", nick, room, ui_index);
         }
         cons_alert();
 
     } else if (g_strcmp0(muc_show, "first") == 0) {
         if (mention) {
-            win_println(console, THEME_MENTION, '-', "<< room mention: %s in %s (win %d)", nick, room, ui_index);
+            win_println(console, THEME_MENTION, "-", "<< room mention: %s in %s (win %d)", nick, room, ui_index);
             cons_alert();
         } else if (triggers) {
             char *triggers_str = _room_triggers_to_string(triggers);
-            win_println(console, THEME_TRIGGER, '-', "<< room trigger %s: %s in %s (win %d)", triggers_str, nick, room, ui_index);
+            win_println(console, THEME_TRIGGER, "-", "<< room trigger %s: %s in %s (win %d)", triggers_str, nick, room, ui_index);
             free(triggers_str);
             cons_alert();
         } else if (unread == 0) {
-            win_println(console, THEME_INCOMING, '-', "<< room message: %s (win %d)", room, ui_index);
+            win_println(console, THEME_INCOMING, "-", "<< room message: %s (win %d)", room, ui_index);
             cons_alert();
         }
     }
@@ -360,10 +360,10 @@ cons_show_incoming_message(const char *const short_from, const int win_index, in
 
     char *chat_show = prefs_get_string(PREF_CONSOLE_CHAT);
     if (g_strcmp0(chat_show, "all") == 0) {
-        win_println(console, THEME_INCOMING, '-', "<< chat message: %s (win %d)", short_from, ui_index);
+        win_println(console, THEME_INCOMING, "-", "<< chat message: %s (win %d)", short_from, ui_index);
         cons_alert();
     } else if ((g_strcmp0(chat_show, "first") == 0) && unread == 0) {
-        win_println(console, THEME_INCOMING, '-', "<< chat message: %s (win %d)", short_from, ui_index);
+        win_println(console, THEME_INCOMING, "-", "<< chat message: %s (win %d)", short_from, ui_index);
         cons_alert();
     }
 
@@ -382,10 +382,10 @@ cons_show_incoming_private_message(const char *const nick, const char *const roo
 
     char *priv_show = prefs_get_string(PREF_CONSOLE_PRIVATE);
     if (g_strcmp0(priv_show, "all") == 0) {
-        win_println(console, THEME_INCOMING, '-', "<< private message: %s in %s (win %d)", nick, room, ui_index);
+        win_println(console, THEME_INCOMING, "-", "<< private message: %s in %s (win %d)", nick, room, ui_index);
         cons_alert();
     } else if ((g_strcmp0(priv_show, "first") == 0) && unread == 0) {
-        win_println(console, THEME_INCOMING, '-', "<< private message: %s in %s (win %d)", nick, room, ui_index);
+        win_println(console, THEME_INCOMING, "-", "<< private message: %s in %s (win %d)", nick, room, ui_index);
         cons_alert();
     }
 
@@ -405,24 +405,24 @@ cons_about(void)
 
         if (strcmp(PACKAGE_STATUS, "development") == 0) {
 #ifdef HAVE_GIT_VERSION
-            win_println(console, THEME_DEFAULT, '-', "Welcome to Profanity, version %sdev.%s.%s", PACKAGE_VERSION, PROF_GIT_BRANCH, PROF_GIT_REVISION);
+            win_println(console, THEME_DEFAULT, "-", "Welcome to Profanity, version %sdev.%s.%s", PACKAGE_VERSION, PROF_GIT_BRANCH, PROF_GIT_REVISION);
 #else
             win_println(console, THEME_DEFAULT, "Welcome to Profanity, version %sdev", PACKAGE_VERSION);
 #endif
         } else {
-            win_println(console, THEME_DEFAULT, '-', "Welcome to Profanity, version %s", PACKAGE_VERSION);
+            win_println(console, THEME_DEFAULT, "-", "Welcome to Profanity, version %s", PACKAGE_VERSION);
         }
     }
 
-    win_println(console, THEME_DEFAULT, '-', "Copyright (C) 2012 - 2019 James Booth <boothj5web@gmail.com>.");
-    win_println(console, THEME_DEFAULT, '-', "Copyright (C) 2019 - 2020 Michael Vetter <jubalh@iodoru.org>.");
-    win_println(console, THEME_DEFAULT, '-', "License GPLv3+: GNU GPL version 3 or later <https://www.gnu.org/licenses/gpl.html>");
-    win_println(console, THEME_DEFAULT, '-', "");
-    win_println(console, THEME_DEFAULT, '-', "This is free software; you are free to change and redistribute it.");
-    win_println(console, THEME_DEFAULT, '-', "There is NO WARRANTY, to the extent permitted by law.");
-    win_println(console, THEME_DEFAULT, '-', "");
-    win_println(console, THEME_DEFAULT, '-', "Type '/help' to show complete help.");
-    win_println(console, THEME_DEFAULT, '-', "");
+    win_println(console, THEME_DEFAULT, "-", "Copyright (C) 2012 - 2019 James Booth <boothj5web@gmail.com>.");
+    win_println(console, THEME_DEFAULT, "-", "Copyright (C) 2019 - 2020 Michael Vetter <jubalh@iodoru.org>.");
+    win_println(console, THEME_DEFAULT, "-", "License GPLv3+: GNU GPL version 3 or later <https://www.gnu.org/licenses/gpl.html>");
+    win_println(console, THEME_DEFAULT, "-", "");
+    win_println(console, THEME_DEFAULT, "-", "This is free software; you are free to change and redistribute it.");
+    win_println(console, THEME_DEFAULT, "-", "There is NO WARRANTY, to the extent permitted by law.");
+    win_println(console, THEME_DEFAULT, "-", "");
+    win_println(console, THEME_DEFAULT, "-", "Type '/help' to show complete help.");
+    win_println(console, THEME_DEFAULT, "-", "");
 
     if (prefs_get_boolean(PREF_VERCHECK)) {
         cons_check_version(FALSE);
@@ -444,13 +444,13 @@ cons_check_version(gboolean not_available_msg)
 
         if (relase_valid) {
             if (release_is_new(latest_release)) {
-                win_println(console, THEME_DEFAULT, '-', "A new version of Profanity is available: %s", latest_release);
-                win_println(console, THEME_DEFAULT, '-', "Check <https://profanity-im.github.io> for details.");
-                win_println(console, THEME_DEFAULT, '-', "");
+                win_println(console, THEME_DEFAULT, "-", "A new version of Profanity is available: %s", latest_release);
+                win_println(console, THEME_DEFAULT, "-", "Check <https://profanity-im.github.io> for details.");
+                win_println(console, THEME_DEFAULT, "-", "");
             } else {
                 if (not_available_msg) {
-                    win_println(console, THEME_DEFAULT, '-', "No new version available.");
-                    win_println(console, THEME_DEFAULT, '-', "");
+                    win_println(console, THEME_DEFAULT, "-", "No new version available.");
+                    win_println(console, THEME_DEFAULT, "-", "");
                 }
             }
 
@@ -466,7 +466,7 @@ cons_show_login_success(ProfAccount *account, gboolean secured)
     ProfWin *console = wins_get_console();
 
     const char *fulljid = connection_get_fulljid();
-    win_print(console, THEME_DEFAULT, '-', "%s logged in successfully, ", fulljid);
+    win_print(console, THEME_DEFAULT, "-", "%s logged in successfully, ", fulljid);
 
     resource_presence_t presence = accounts_get_login_presence(account->name);
     const char *presence_str = string_from_resource_presence(presence);
@@ -500,9 +500,9 @@ cons_show_wins(gboolean unread)
     GSList *curr = window_strings;
     while (curr) {
         if (g_strstr_len(curr->data, strlen(curr->data), " unread") > 0) {
-            win_println(console, THEME_CMD_WINS_UNREAD, '-', "%s", curr->data);
+            win_println(console, THEME_CMD_WINS_UNREAD, "-", "%s", curr->data);
         } else {
-            win_println(console, THEME_DEFAULT, '-', "%s", curr->data);
+            win_println(console, THEME_DEFAULT, "-", "%s", curr->data);
         }
         curr = g_slist_next(curr);
     }
@@ -548,13 +548,13 @@ cons_show_caps(const char *const fulljid, resource_presence_t presence)
         const char *resource_presence = string_from_resource_presence(presence);
 
         theme_item_t presence_colour = theme_main_presence_attrs(resource_presence);
-        win_print(console, presence_colour, '-', "%s", fulljid);
+        win_print(console, presence_colour, "-", "%s", fulljid);
         win_appendln(console, THEME_DEFAULT, ":");
 
         // show identity
         if (caps->identity) {
             DiscoIdentity *identity = caps->identity;
-            win_print(console, THEME_DEFAULT, '-', "Identity: ");
+            win_print(console, THEME_DEFAULT, "-", "Identity: ");
             if (identity->name) {
                 win_append(console, THEME_DEFAULT, "%s", identity->name);
                 if (identity->category || identity->type) {
@@ -576,7 +576,7 @@ cons_show_caps(const char *const fulljid, resource_presence_t presence)
         if (caps->software_version) {
             SoftwareVersion *software_version = caps->software_version;
             if (software_version->software) {
-                win_print(console, THEME_DEFAULT, '-', "Software: %s", software_version->software);
+                win_print(console, THEME_DEFAULT, "-", "Software: %s", software_version->software);
             }
             if (software_version->software_version) {
                 win_append(console, THEME_DEFAULT, ", %s", software_version->software_version);
@@ -585,7 +585,7 @@ cons_show_caps(const char *const fulljid, resource_presence_t presence)
                 win_newline(console);
             }
             if (software_version->os) {
-                win_print(console, THEME_DEFAULT, '-', "OS: %s", software_version->os);
+                win_print(console, THEME_DEFAULT, "-", "OS: %s", software_version->os);
             }
             if (software_version->os_version) {
                 win_append(console, THEME_DEFAULT, ", %s", software_version->os_version);
@@ -596,10 +596,10 @@ cons_show_caps(const char *const fulljid, resource_presence_t presence)
         }
 
         if (caps->features) {
-            win_println(console, THEME_DEFAULT, '-', "Features:");
+            win_println(console, THEME_DEFAULT, "-", "Features:");
             GSList *feature = caps->features;
             while (feature) {
-                win_println(console, THEME_DEFAULT, '-', " %s", feature->data);
+                win_println(console, THEME_DEFAULT, "-", " %s", feature->data);
                 feature = g_slist_next(feature);
             }
         }
@@ -661,7 +661,7 @@ cons_show_room_list(GSList *rooms, const char *const conference_node)
         cons_show("Chat rooms at %s:", conference_node);
         while (rooms) {
             DiscoItem *room = rooms->data;
-            win_print(console, THEME_DEFAULT, '-', "  %s", room->jid);
+            win_print(console, THEME_DEFAULT, "-", "  %s", room->jid);
             if (room->name) {
                 win_append(console, THEME_DEFAULT, ", (%s)", room->name);
             }
@@ -696,7 +696,11 @@ cons_show_bookmarks(const GList *list)
             if (muc_active(item->barejid) && roomwin) {
                 presence_colour = THEME_ONLINE;
             }
-            win_print(console, presence_colour, '-', "  %s", item->barejid);
+            if (item->name) {
+                win_print(console, presence_colour, "-", "  %s - %s", item->name, item->barejid);
+            } else {
+                win_print(console, presence_colour, "-", "  %s", item->barejid);
+            }
             if (item->nick) {
                 win_append(console, presence_colour, "/%s", item->nick);
             }
@@ -751,7 +755,7 @@ cons_show_disco_info(const char *jid, GSList *identities, GSList *features)
                 identity_str = g_string_append(identity_str, identity->category);
             }
             cons_show(identity_str->str);
-            g_string_free(identity_str, FALSE);
+            g_string_free(identity_str, TRUE);
             identities = g_slist_next(identities);
         }
 
@@ -776,7 +780,7 @@ cons_show_disco_items(GSList *items, const char *const jid)
         cons_show("Service discovery items for %s:", jid);
         while (items) {
             DiscoItem *item = items->data;
-            win_print(console, THEME_DEFAULT, '-', "  %s", item->jid);
+            win_print(console, THEME_DEFAULT, "-", "  %s", item->jid);
             if (item->name) {
                 win_append(console, THEME_DEFAULT, ", (%s)", item->name);
             }
@@ -855,7 +859,7 @@ cons_show_account_list(gchar **accounts)
                     (g_strcmp0(session_get_account_name(), accounts[i]) == 0)) {
                 resource_presence_t presence = accounts_get_last_presence(accounts[i]);
                 theme_item_t presence_colour = theme_main_presence_attrs(string_from_resource_presence(presence));
-                win_println(console, presence_colour, '-', "%s", accounts[i]);
+                win_println(console, presence_colour, "-", "%s", accounts[i]);
             } else {
                 cons_show(accounts[i]);
             }
@@ -904,6 +908,9 @@ cons_show_account(ProfAccount *account)
     if (account->tls_policy) {
         cons_show   ("TLS policy        : %s", account->tls_policy);
     }
+    if (account->auth_policy) {
+        cons_show   ("Auth policy       : %s", account->auth_policy);
+    }
     if (account->last_presence) {
         cons_show   ("Last presence     : %s", account->last_presence);
     }
@@ -974,7 +981,7 @@ cons_show_account(ProfAccount *account)
 
         GList *curr = resources;
         if (curr) {
-            win_println(console, THEME_DEFAULT, '-', "Resources:");
+            win_println(console, THEME_DEFAULT, "-", "Resources:");
 
             // sort in order of availability
             while (curr) {
@@ -992,7 +999,7 @@ cons_show_account(ProfAccount *account)
             Resource *resource = curr->data;
             const char *resource_presence = string_from_resource_presence(resource->presence);
             theme_item_t presence_colour = theme_main_presence_attrs(resource_presence);
-            win_print(console, presence_colour, '-', "  %s (%d), %s", resource->name, resource->priority, resource_presence);
+            win_print(console, presence_colour, "-", "  %s (%d), %s", resource->name, resource->priority, resource_presence);
 
             if (resource->status) {
                 win_append(console, presence_colour, ", \"%s\"", resource->status);
@@ -1006,7 +1013,7 @@ cons_show_account(ProfAccount *account)
                 // show identity
                 if (caps->identity) {
                     DiscoIdentity *identity = caps->identity;
-                    win_print(console, THEME_DEFAULT, '-', "    Identity: ");
+                    win_print(console, THEME_DEFAULT, "-", "    Identity: ");
                     if (identity->name) {
                         win_append(console, THEME_DEFAULT, "%s", identity->name);
                         if (identity->category || identity->type) {
@@ -1028,7 +1035,7 @@ cons_show_account(ProfAccount *account)
                 if (caps->software_version) {
                     SoftwareVersion *software_version = caps->software_version;
                     if (software_version->software) {
-                        win_print(console, THEME_DEFAULT, '-', "    Software: %s", software_version->software);
+                        win_print(console, THEME_DEFAULT, "-", "    Software: %s", software_version->software);
                     }
                     if (software_version->software_version) {
                         win_append(console, THEME_DEFAULT, ", %s", software_version->software_version);
@@ -1037,7 +1044,7 @@ cons_show_account(ProfAccount *account)
                         win_newline(console);
                     }
                     if (software_version->os) {
-                        win_print(console, THEME_DEFAULT, '-', "    OS: %s", software_version->os);
+                        win_print(console, THEME_DEFAULT, "-", "    OS: %s", software_version->os);
                     }
                     if (software_version->os_version) {
                         win_append(console, THEME_DEFAULT, ", %s", software_version->os_version);
@@ -1067,9 +1074,7 @@ cons_show_aliases(GList *aliases)
     }
 
     GList *curr = aliases;
-    if (curr) {
-        cons_show("Command aliases:");
-    }
+    cons_show("Command aliases:");
     while (curr) {
         ProfAlias *alias = curr->data;
         cons_show("  /%s -> %s", alias->name, alias->value);
@@ -1159,9 +1164,17 @@ cons_titlebar_setting(void)
         cons_show("Titlebar presence (/titlebar)       : OFF");
     }
 
-    char *muctitle = prefs_get_string(PREF_TITLEBAR_MUC_TITLE);
-    cons_show("MUC window title (/titlebar)        : %s", muctitle);
-    prefs_free_string(muctitle);
+    if (prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_NAME)) {
+        cons_show("Titlebar show MUC name (/titlebar)       : ON");
+    } else {
+        cons_show("Titlebar show MUC name (/titlebar)       : OFF");
+    }
+
+    if (prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_JID)) {
+        cons_show("Titlebar show MUC JID (/titlebar)       : ON");
+    } else {
+        cons_show("Titlebar show MUC JID (/titlebar)       : OFF");
+    }
 }
 
 void
@@ -1507,6 +1520,7 @@ cons_roster_setting(void)
     } else {
         cons_show("Roster unread (/roster)             : OFF");
     }
+    prefs_free_string(unread);
 
     if (prefs_get_boolean(PREF_ROSTER_ROOMS))
         cons_show("Roster rooms (/roster)              : show");
@@ -1521,6 +1535,7 @@ cons_roster_setting(void)
     } else {
         cons_show("Roster private (/roster)            : OFF");
     }
+    prefs_free_string(priv);
 
     char *rooms_pos = prefs_get_string(PREF_ROSTER_ROOMS_POS);
     cons_show("Roster rooms position (/roster)     : %s", rooms_pos);
@@ -1546,6 +1561,7 @@ cons_roster_setting(void)
     } else {
         cons_show("Roster rooms unread (/roster)       : OFF");
     }
+    prefs_free_string(roomsunread);
 
     int size = prefs_get_roster_size();
     cons_show("Roster size (/roster)               : %d", size);
@@ -1786,39 +1802,45 @@ void
 cons_statusbar_setting(void)
 {
     if (prefs_get_boolean(PREF_STATUSBAR_SHOW_NAME)) {
-        cons_show("Show tab names (/statusbar)         : ON");
+        cons_show("Show tab names (/statusbar)                 : ON");
     } else {
-        cons_show("Show tab names (/statusbar)         : OFF");
+        cons_show("Show tab names (/statusbar)                 : OFF");
     }
     if (prefs_get_boolean(PREF_STATUSBAR_SHOW_NUMBER)) {
-        cons_show("Show tab numbers (/statusbar)       : ON");
+        cons_show("Show tab numbers (/statusbar)               : ON");
     } else {
-        cons_show("Show tab numbers (/statusbar)       : OFF");
+        cons_show("Show tab numbers (/statusbar)               : OFF");
     }
 
-    cons_show("Max tabs (/statusbar)               : %d", prefs_get_statusbartabs());
+    if (prefs_get_boolean(PREF_STATUSBAR_SHOW_NUMBER)) {
+        cons_show("Show tab with no actions (/statusbar)       : ON");
+    } else {
+        cons_show("Show tab with no actions (/statusbar)       : OFF");
+    }
+
+    cons_show("Max tabs (/statusbar)                       : %d", prefs_get_statusbartabs());
 
     gint pref_len = prefs_get_statusbartablen();
     if (pref_len == 0) {
-        cons_show("Max tab length (/statusbar)         : OFF");
+        cons_show("Max tab length (/statusbar)                 : OFF");
     } else {
-        cons_show("Max tab length (/statusbar)         : %d", pref_len);
+        cons_show("Max tab length (/statusbar)                 : %d", pref_len);
     }
 
     char *pref_self = prefs_get_string(PREF_STATUSBAR_SELF);
     if (g_strcmp0(pref_self, "off") == 0) {
-        cons_show("Self statusbar display (/statusbar) : OFF");
+        cons_show("Self statusbar display (/statusbar)         : OFF");
     } else {
-        cons_show("Self statusbar display (/statusbar) : %s", pref_self);
+        cons_show("Self statusbar display (/statusbar)         : %s", pref_self);
     }
     prefs_free_string(pref_self);
 
     char *pref_chat = prefs_get_string(PREF_STATUSBAR_CHAT);
-    cons_show("Chat tab display (/statusbar)       : %s", pref_chat);
+    cons_show("Chat tab display (/statusbar)               : %s", pref_chat);
     prefs_free_string(pref_chat);
 
     char *pref_room = prefs_get_string(PREF_STATUSBAR_ROOM);
-    cons_show("Room tab display (/statusbar)       : %s", pref_room);
+    cons_show("Room tab display (/statusbar)               : %s", pref_room);
     prefs_free_string(pref_room);
 }
 
@@ -1854,14 +1876,14 @@ void
 cons_logging_setting(void)
 {
     if (prefs_get_boolean(PREF_CHLOG))
-        cons_show("Chat logging (/logging chat)        : ON");
+        cons_show("Chat logging (/logging chat)                : ON");
     else
-        cons_show("Chat logging (/logging chat)        : OFF");
+        cons_show("Chat logging (/logging chat)                : OFF");
 
     if (prefs_get_boolean(PREF_GRLOG))
-        cons_show("Groupchat logging (/logging group)  : ON");
+        cons_show("Groupchat logging (/logging group)          : ON");
     else
-        cons_show("Groupchat logging (/logging group)  : OFF");
+        cons_show("Groupchat logging (/logging group)          : OFF");
 }
 
 void
@@ -1998,6 +2020,12 @@ cons_color_setting(void)
 
     prefs_free_string(color_pref);
 
+    if (prefs_get_boolean(PREF_COLOR_NICK_OWN)) {
+        cons_show("Consistent color generation for own nick (/color own)              : ON");
+    } else {
+        cons_show("Consistent color generation for own nick (/color own)              : OFF");
+    }
+
     if (prefs_get_boolean(PREF_ROSTER_COLOR_NICK)) {
         cons_show("Consistent color generation in roster (/roster color)              : ON");
     } else {
@@ -2021,6 +2049,42 @@ cons_os_setting(void)
     }
 }
 
+void
+cons_correction_setting(void)
+{
+    if (prefs_get_boolean(PREF_CORRECTION_ALLOW)) {
+        cons_show("Last Message Correction (XEP-0308) (/correction)                   : ON");
+    } else {
+        cons_show("Last Message Correction (XEP-0308) (/correction)                   : OFF");
+    }
+
+    char *cc = prefs_get_correction_char();
+    cons_show("LMC indication char (/correction char)                             : %s", cc);
+    free(cc);
+}
+
+void
+cons_executable_setting(void)
+{
+    char *avatar = prefs_get_string(PREF_AVATAR_CMD);
+    cons_show("Avatar command (/executable avatar)                                   : %s", avatar);
+    prefs_free_string(avatar);
+
+    char *exec = prefs_get_string(PREF_URL_OPEN_CMD);
+    cons_show("urlopen command (/executable urlopen)                                 : %s", exec);
+    prefs_free_string(exec);
+}
+
+void
+cons_slashguard_setting(void)
+{
+    if (prefs_get_boolean(PREF_SLASH_GUARD)) {
+        cons_show("Slashguard (/slashguard)       : ON");
+    } else {
+        cons_show("Slashguard (/slashguard)       : OFF");
+    }
+}
+
 void
 cons_show_connection_prefs(void)
 {
@@ -2054,8 +2118,15 @@ cons_show_otr_prefs(void)
     }
     prefs_free_string(log_value);
 
-    char ch = prefs_get_otr_char();
-    cons_show("OTR char (/otr char)     : %c", ch);
+    char *ch = prefs_get_otr_char();
+    cons_show("OTR char (/otr char)     : %s", ch);
+    free(ch);
+
+    if (prefs_get_boolean(PREF_OTR_SENDFILE)) {
+        cons_show("Allow sending unencrypted files in an OTR session via /sendfile (/otr sendfile): ON");
+    } else {
+        cons_show("Allow sending unencrypted files in an OTR session via /sendfile (/otr sendfile): OFF");
+    }
 
     cons_alert();
 }
@@ -2076,8 +2147,15 @@ cons_show_pgp_prefs(void)
     }
     prefs_free_string(log_value);
 
-    char ch = prefs_get_pgp_char();
-    cons_show("PGP char (/pgp char)     : %c", ch);
+    char *ch = prefs_get_pgp_char();
+    cons_show("PGP char (/pgp char)     : %s", ch);
+    free(ch);
+
+    if (prefs_get_boolean(PREF_PGP_SENDFILE)) {
+        cons_show("Allow sending unencrypted files via /sendfile while otherwise using PGP (/pgp sendfile): ON");
+    } else {
+        cons_show("Allow sending unencrypted files via /sendfile while otherwise using PGP (/pgp sendfile): OFF");
+    }
 
     cons_alert();
 }
@@ -2102,8 +2180,15 @@ cons_show_omemo_prefs(void)
     }
     prefs_free_string(log_value);
 
-    char ch = prefs_get_omemo_char();
-    cons_show("OMEMO char (/omemo char)     : %c", ch);
+    char *ch = prefs_get_omemo_char();
+    cons_show("OMEMO char (/omemo char)     : %s", ch);
+    free(ch);
+
+    if (prefs_get_boolean(PREF_OMEMO_SENDFILE)) {
+        cons_show("Allow sending unencrypted files in an OMEMO session via /sendfile (/omemo sendfile): ON");
+    } else {
+        cons_show("Allow sending unencrypted files in an OMEMO session via /sendfile (/omemo sendfile): OFF");
+    }
 
     cons_alert();
 }
@@ -2217,7 +2302,7 @@ cons_navigation_help(void)
 {
     ProfWin *console = wins_get_console();
     cons_show("");
-    win_println(console, THEME_HELP_HEADER, '-', "Navigation");
+    win_println(console, THEME_HELP_HEADER, "-", "Navigation");
     cons_show("Alt-1..Alt-0, F1..F10    : Choose window.");
     cons_show("Alt-LEFT, Alt-RIGHT      : Previous/next chat window.");
     cons_show("PAGEUP, PAGEDOWN         : Page the main window.");
@@ -2323,7 +2408,7 @@ _cons_theme_bar_prop(theme_item_t theme, char *prop)
 
     GString *propstr = g_string_new(" ");
     g_string_append_printf(propstr, "%-24s", prop);
-    win_print(console, THEME_TEXT, '-', "%s", propstr->str);
+    win_print(console, THEME_TEXT, "-", "%s", propstr->str);
     g_string_free(propstr, TRUE);
 
     GString *valstr = g_string_new(" ");
@@ -2342,7 +2427,7 @@ _cons_theme_prop(theme_item_t theme, char *prop)
 
     GString *propstr = g_string_new(" ");
     g_string_append_printf(propstr, "%-24s", prop);
-    win_print(console, THEME_TEXT, '-', "%s", propstr->str);
+    win_print(console, THEME_TEXT, "-", "%s", propstr->str);
     g_string_free(propstr, TRUE);
 
     GString *valstr = g_string_new("");
@@ -2360,6 +2445,8 @@ cons_theme_properties(void)
     _cons_theme_bar_prop(THEME_TITLE_TEXT, "titlebar.text");
     _cons_theme_bar_prop(THEME_TITLE_BRACKET, "titlebar.brackets");
 
+    _cons_theme_bar_prop(THEME_TITLE_SCROLLED, "titlebar.scrolled");
+
     _cons_theme_bar_prop(THEME_TITLE_UNENCRYPTED, "titlebar.unencrypted");
     _cons_theme_bar_prop(THEME_TITLE_ENCRYPTED, "titlebar.encrypted");
     _cons_theme_bar_prop(THEME_TITLE_UNTRUSTED, "titlebar.untrusted");
@@ -2468,28 +2555,28 @@ cons_theme_colours(void)
     ProfWin *console = wins_get_console();
     cons_show("Available colours:");
 
-    win_print(console,      THEME_WHITE, '-',   " white   ");
+    win_print(console,      THEME_WHITE, "-",   " white   ");
     win_appendln(console,   THEME_WHITE_BOLD,   " bold_white");
 
-    win_print(console,      THEME_GREEN, '-',   " green   ");
+    win_print(console,      THEME_GREEN, "-",   " green   ");
     win_appendln(console,   THEME_GREEN_BOLD,   " bold_green");
 
-    win_print(console,      THEME_RED, '-',     " red     ");
+    win_print(console,      THEME_RED, "-",     " red     ");
     win_appendln(console,   THEME_RED_BOLD,     " bold_red");
 
-    win_print(console,      THEME_YELLOW, '-',  " yellow  ");
+    win_print(console,      THEME_YELLOW, "-",  " yellow  ");
     win_appendln(console,   THEME_YELLOW_BOLD,  " bold_yellow");
 
-    win_print(console,      THEME_BLUE, '-',    " blue    ");
+    win_print(console,      THEME_BLUE, "-",    " blue    ");
     win_appendln(console,   THEME_BLUE_BOLD,    " bold_blue");
 
-    win_print(console,      THEME_CYAN, '-',    " cyan    ");
+    win_print(console,      THEME_CYAN, "-",    " cyan    ");
     win_appendln(console,   THEME_CYAN_BOLD,    " bold_cyan");
 
-    win_print(console,      THEME_MAGENTA, '-', " magenta ");
+    win_print(console,      THEME_MAGENTA, "-", " magenta ");
     win_appendln(console,   THEME_MAGENTA_BOLD, " bold_magenta");
 
-    win_print(console,      THEME_BLACK, '-',   " black   ");
+    win_print(console,      THEME_BLACK, "-",   " black   ");
     win_appendln(console,   THEME_BLACK_BOLD,   " bold_black");
 
     if (COLORS >= 256) {
@@ -2506,25 +2593,25 @@ static void
 _cons_splash_logo(void)
 {
     ProfWin *console = wins_get_console();
-    win_println(console, THEME_DEFAULT, '-', "Welcome to");
+    win_println(console, THEME_DEFAULT, "-", "Welcome to");
 
-    win_println(console, THEME_SPLASH, '-', "                  ___            _           ");
-    win_println(console, THEME_SPLASH, '-', "                 / __)          (_)_         ");
-    win_println(console, THEME_SPLASH, '-', " ____   ___ ___ | |__ ____ ____  _| |_ _   _ ");
-    win_println(console, THEME_SPLASH, '-', "|  _ \\ / __) _ \\|  __) _  |  _ \\| |  _) | | |");
-    win_println(console, THEME_SPLASH, '-', "| | ) | | | (_) | | | ( | | | | | | |_| |_| |");
-    win_println(console, THEME_SPLASH, '-', "| ||_/|_|  \\___/|_|  \\_||_|_| |_|_|\\___)__  |");
-    win_println(console, THEME_SPLASH, '-', "|_|                                   (____/ ");
-    win_println(console, THEME_SPLASH, '-', "");
+    win_println(console, THEME_SPLASH, "-", "                  ___            _           ");
+    win_println(console, THEME_SPLASH, "-", "                 / __)          (_)_         ");
+    win_println(console, THEME_SPLASH, "-", " ____   ___ ___ | |__ ____ ____  _| |_ _   _ ");
+    win_println(console, THEME_SPLASH, "-", "|  _ \\ / __) _ \\|  __) _  |  _ \\| |  _) | | |");
+    win_println(console, THEME_SPLASH, "-", "| | ) | | | (_) | | | ( | | | | | | |_| |_| |");
+    win_println(console, THEME_SPLASH, "-", "| ||_/|_|  \\___/|_|  \\_||_|_| |_|_|\\___)__  |");
+    win_println(console, THEME_SPLASH, "-", "|_|                                   (____/ ");
+    win_println(console, THEME_SPLASH, "-", "");
 
     if (strcmp(PACKAGE_STATUS, "development") == 0) {
 #ifdef HAVE_GIT_VERSION
-        win_println(console, THEME_DEFAULT, '-', "Version %sdev.%s.%s", PACKAGE_VERSION, PROF_GIT_BRANCH, PROF_GIT_REVISION);
+        win_println(console, THEME_DEFAULT, "-", "Version %sdev.%s.%s", PACKAGE_VERSION, PROF_GIT_BRANCH, PROF_GIT_REVISION);
 #else
         win_println(console, THEME_DEFAULT, "Version %sdev", PACKAGE_VERSION);
 #endif
     } else {
-        win_println(console, THEME_DEFAULT, '-', "Version %s", PACKAGE_VERSION);
+        win_println(console, THEME_DEFAULT, "-", "Version %s", PACKAGE_VERSION);
     }
 }
 
@@ -2551,7 +2638,7 @@ _show_roster_contacts(GSList *list, gboolean show_groups)
         } else {
             presence_colour = theme_main_presence_attrs("offline");
         }
-        win_print(console, presence_colour, '-', "%s", title->str);
+        win_print(console, presence_colour, "-", "%s", title->str);
 
         g_string_free(title, TRUE);
 
@@ -2599,3 +2686,24 @@ _show_roster_contacts(GSList *list, gboolean show_groups)
         curr = g_slist_next(curr);
     }
 }
+
+void
+cons_show_bookmarks_ignore(gchar **list, gsize len)
+{
+    if (len == 0) {
+        cons_show("");
+        cons_show("No ignored bookmarks");
+        return;
+    }
+
+    int i;
+    ProfWin *console = wins_get_console();
+
+    cons_show("");
+    cons_show("Ignored bookmarks:");
+
+    for(i=0; i<len; i++) {
+        win_print(console, THEME_DEFAULT, "-", "  %s", list[i]);
+        win_newline(console);
+    }
+}
