@@ -196,6 +196,15 @@ inp_readline(void)
     }
 
     if (inp_line) {
+        if (!get_password && prefs_get_boolean(PREF_SLASH_GUARD)) {
+            if (strlen(inp_line) > 1) {
+                char *res = (char*) memchr (inp_line+1, '/', 3);
+                if (res) {
+                    cons_show("Your text contains a slash in the first 4 characters");
+                    return NULL;
+                }
+            }
+        }
         return strdup(inp_line);
     } else {
         return NULL;
@@ -317,6 +326,10 @@ _inp_write(char *line, int offset)
 static int
 _inp_edited(const wint_t ch)
 {
+    // Use own state to be thread-safe with possible pthreads. C standard
+    // guarantees that initial value of the state will be zeroed buffer.
+    static mbstate_t mbstate;
+
     // backspace
     if (ch == 127) {
         return 1;
@@ -334,7 +347,10 @@ _inp_edited(const wint_t ch)
 
     // printable
     char bytes[MB_CUR_MAX+1];
-    size_t utf_len = wcrtomb(bytes, ch, (mbstate_t*)NULL);
+    size_t utf_len = wcrtomb(bytes, ch, &mbstate);
+    if (utf_len == (size_t)-1) {
+        return 0;
+    }
     bytes[utf_len] = '\0';
     gunichar unichar = g_utf8_get_char(bytes);
 
@@ -487,10 +503,10 @@ _inp_rl_startup_hook(void)
     rl_variable_bind("disable-completion", "on");
 
     // check for and load ~/.config/profanity/inputrc
-    char *inputrc = files_get_inputrc_file();
+    gchar *inputrc = files_get_inputrc_file();
     if (inputrc) {
         rl_read_init_file(inputrc);
-        free(inputrc);
+        g_free(inputrc);
     }
 
     return 0;
