@@ -50,6 +50,8 @@
 #include "omemo/omemo.h"
 #endif
 
+static void _mucwin_set_last_message(ProfMucWin *mucwin, const char *const id, const char *const message);
+
 ProfMucWin*
 mucwin_new(const char *const barejid)
 {
@@ -74,7 +76,7 @@ mucwin_role_change(ProfMucWin *mucwin, const char *const role, const char *const
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_print(window, THEME_ROOMINFO, '!', "Your role has been changed to: %s", role);
+    win_print(window, THEME_ROOMINFO, "!", "Your role has been changed to: %s", role);
     if (actor) {
         win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
@@ -91,7 +93,7 @@ mucwin_affiliation_change(ProfMucWin *mucwin, const char *const affiliation, con
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_print(window, THEME_ROOMINFO, '!', "Your affiliation has been changed to: %s", affiliation);
+    win_print(window, THEME_ROOMINFO, "!", "Your affiliation has been changed to: %s", affiliation);
     if (actor) {
         win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
@@ -108,7 +110,7 @@ mucwin_role_and_affiliation_change(ProfMucWin *mucwin, const char *const role, c
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_print(window, THEME_ROOMINFO, '!', "Your role and affiliation have been changed, role: %s, affiliation: %s", role, affiliation);
+    win_print(window, THEME_ROOMINFO, "!", "Your role and affiliation have been changed, role: %s, affiliation: %s", role, affiliation);
     if (actor) {
         win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
@@ -126,7 +128,7 @@ mucwin_occupant_role_change(ProfMucWin *mucwin, const char *const nick, const ch
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_print(window, THEME_ROOMINFO, '!', "%s's role has been changed to: %s", nick, role);
+    win_print(window, THEME_ROOMINFO, "!", "%s's role has been changed to: %s", nick, role);
     if (actor) {
         win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
@@ -143,7 +145,7 @@ mucwin_occupant_affiliation_change(ProfMucWin *mucwin, const char *const nick, c
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_print(window, THEME_ROOMINFO, '!', "%s's affiliation has been changed to: %s", nick, affiliation);
+    win_print(window, THEME_ROOMINFO, "!", "%s's affiliation has been changed to: %s", nick, affiliation);
     if (actor) {
         win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
@@ -160,7 +162,7 @@ mucwin_occupant_role_and_affiliation_change(ProfMucWin *mucwin, const char *cons
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_print(window, THEME_ROOMINFO, '!', "%s's role and affiliation have been changed, role: %s, affiliation: %s", nick, role, affiliation);
+    win_print(window, THEME_ROOMINFO, "!", "%s's role and affiliation have been changed, role: %s, affiliation: %s", nick, role, affiliation);
     if (actor) {
         win_append(window, THEME_ROOMINFO, ", by: %s", actor);
     }
@@ -176,8 +178,8 @@ mucwin_room_info_error(ProfMucWin *mucwin, const char *const error)
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_println(window, THEME_DEFAULT, '!', "Room info request failed: %s", error);
-    win_println(window, THEME_DEFAULT, '-', "");
+    win_println(window, THEME_DEFAULT, "!", "Room info request failed: %s", error);
+    win_println(window, THEME_DEFAULT, "-", "");
 }
 
 void
@@ -189,7 +191,7 @@ mucwin_room_disco_info(ProfMucWin *mucwin, GSList *identities, GSList *features)
     if ((identities && (g_slist_length(identities) > 0)) ||
         (features && (g_slist_length(features) > 0))) {
         if (identities) {
-            win_println(window, THEME_DEFAULT, '!', "Identities:");
+            win_println(window, THEME_DEFAULT, "!", "Identities:");
         }
         while (identities) {
             DiscoIdentity *identity = identities->data;  // anme trpe, cat
@@ -205,19 +207,19 @@ mucwin_room_disco_info(ProfMucWin *mucwin, GSList *identities, GSList *features)
             if (identity->category) {
                 identity_str = g_string_append(identity_str, identity->category);
             }
-            win_println(window, THEME_DEFAULT, '!', "%s", identity_str->str);
+            win_println(window, THEME_DEFAULT, "!", "%s", identity_str->str);
             g_string_free(identity_str, TRUE);
             identities = g_slist_next(identities);
         }
 
         if (features) {
-            win_println(window, THEME_DEFAULT, '!', "Features:");
+            win_println(window, THEME_DEFAULT, "!", "Features:");
         }
         while (features) {
-            win_println(window, THEME_DEFAULT, '!', "  %s", features->data);
+            win_println(window, THEME_DEFAULT, "!", "  %s", features->data);
             features = g_slist_next(features);
         }
-        win_println(window, THEME_DEFAULT, '-', "");
+        win_println(window, THEME_DEFAULT, "-", "");
     }
 }
 
@@ -229,16 +231,16 @@ mucwin_roster(ProfMucWin *mucwin, GList *roster, const char *const presence)
     ProfWin *window = (ProfWin*)mucwin;
     if ((roster == NULL) || (g_list_length(roster) == 0)) {
         if (presence == NULL) {
-            win_println(window, THEME_ROOMINFO, '!', "Room is empty.");
+            win_println(window, THEME_ROOMINFO, "!", "Room is empty.");
         } else {
-            win_println(window, THEME_ROOMINFO, '!', "No occupants %s.", presence);
+            win_println(window, THEME_ROOMINFO, "!", "No occupants %s.", presence);
         }
     } else {
         int length = g_list_length(roster);
         if (presence == NULL) {
-            win_print(window, THEME_ROOMINFO, '!', "%d occupants: ", length);
+            win_print(window, THEME_ROOMINFO, "!", "%d occupants: ", length);
         } else {
-            win_print(window, THEME_ROOMINFO, '!', "%d %s: ", length, presence);
+            win_print(window, THEME_ROOMINFO, "!", "%d %s: ", length, presence);
         }
 
         while (roster) {
@@ -264,7 +266,7 @@ mucwin_occupant_offline(ProfMucWin *mucwin, const char *const nick)
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_println(window, THEME_OFFLINE, '!', "<- %s has left the room.", nick);
+    win_println(window, THEME_OFFLINE, "!", "<- %s has left the room.", nick);
 }
 
 void
@@ -285,7 +287,7 @@ mucwin_occupant_kicked(ProfMucWin *mucwin, const char *const nick, const char *c
         g_string_append(message, reason);
     }
 
-    win_println(window, THEME_OFFLINE, '!', "<- %s", message->str);
+    win_println(window, THEME_OFFLINE, "!", "<- %s", message->str);
     g_string_free(message, TRUE);
 }
 
@@ -307,7 +309,7 @@ mucwin_occupant_banned(ProfMucWin *mucwin, const char *const nick, const char *c
         g_string_append(message, reason);
     }
 
-    win_println(window, THEME_OFFLINE, '!', "<- %s", message->str);
+    win_println(window, THEME_OFFLINE, "!", "<- %s", message->str);
     g_string_free(message, TRUE);
 }
 
@@ -318,7 +320,7 @@ mucwin_occupant_online(ProfMucWin *mucwin, const char *const nick, const char *c
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_print(window, THEME_ONLINE, '!', "-> %s has joined the room", nick);
+    win_print(window, THEME_ONLINE, "!", "-> %s has joined the room", nick);
     if (prefs_get_boolean(PREF_MUC_PRIVILEGES)) {
         if (role) {
             win_append(window, THEME_ONLINE, ", role: %s", role);
@@ -346,7 +348,7 @@ mucwin_occupant_nick_change(ProfMucWin *mucwin, const char *const old_nick, cons
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_println(window, THEME_THEM, '!', "** %s is now known as %s", old_nick, nick);
+    win_println(window, THEME_THEM, "!", "** %s is now known as %s", old_nick, nick);
 }
 
 void
@@ -355,50 +357,43 @@ mucwin_nick_change(ProfMucWin *mucwin, const char *const nick)
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_println(window, THEME_ME, '!', "** You are now known as %s", nick);
+    win_println(window, THEME_ME, "!", "** You are now known as %s", nick);
 }
 
 void
-mucwin_history(ProfMucWin *mucwin, const char *const nick, GDateTime *timestamp, const char *const message)
+mucwin_history(ProfMucWin *mucwin, const ProfMessage *const message)
 {
     assert(mucwin != NULL);
 
-    ProfWin *window = (ProfWin*)mucwin;
-    GString *line = g_string_new("");
+    char *nick = message->from_jid->resourcepart;
+    char *mynick = muc_nick(mucwin->roomjid);
+    GSList *mentions = get_mentions(prefs_get_boolean(PREF_NOTIFY_MENTION_WHOLE_WORD), prefs_get_boolean(PREF_NOTIFY_MENTION_CASE_SENSITIVE), message->plain, mynick);
+    GList *triggers = prefs_message_get_triggers(message->plain);
 
-    if (strncmp(message, "/me ", 4) == 0) {
-        g_string_append(line, "*");
-        g_string_append(line, nick);
-        g_string_append(line, " ");
-        g_string_append(line, message + 4);
-    } else {
-        g_string_append(line, nick);
-        g_string_append(line, ": ");
-        g_string_append(line, message);
-    }
+    mucwin_incoming_msg(mucwin, message, mentions, triggers, FALSE);
 
-    win_print_history(window, timestamp, "%s", line->str);
-    g_string_free(line, TRUE);
+    g_slist_free(mentions);
+    g_list_free_full(triggers, free);
 
-    plugins_on_room_history_message(mucwin->roomjid, nick, message, timestamp);
+    plugins_on_room_history_message(mucwin->roomjid, nick, message->plain, message->timestamp);
 }
 
 static void
 _mucwin_print_mention(ProfWin *window, const char *const message, const char *const from, const char *const mynick, GSList *mentions, const char *const ch, int flags)
 {
     int last_pos = 0;
-    int pos = 0;
+    int pos;
     GSList *curr = mentions;
     while (curr) {
         pos = GPOINTER_TO_INT(curr->data);
 
         char *before_str = g_strndup(message + last_pos, pos - last_pos);
         if (strncmp(before_str, "/me ", 4) == 0) {
-            win_print_them(window, THEME_ROOMMENTION, *ch, flags, "");
+            win_print_them(window, THEME_ROOMMENTION, ch, flags, "");
             win_append_highlight(window, THEME_ROOMMENTION, "*%s ", from);
             win_append_highlight(window, THEME_ROOMMENTION, "%s", before_str + 4);
         } else {
-            win_print_them(window, THEME_ROOMMENTION, *ch, flags, from);
+            win_print_them(window, THEME_ROOMMENTION, ch, flags, from);
             win_append_highlight(window, THEME_ROOMMENTION, "%s", before_str);
         }
         g_free(before_str);
@@ -501,34 +496,42 @@ _mucwin_print_triggers(ProfWin *window, const char *const message, GList *trigge
 }
 
 void
-mucwin_outgoing_msg(ProfMucWin *mucwin, const char *const message, const char *const id, prof_enc_t enc_mode)
+mucwin_outgoing_msg(ProfMucWin *mucwin, const char *const message, const char *const id, prof_enc_t enc_mode, const char *const replace_id)
 {
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
     char *mynick = muc_nick(mucwin->roomjid);
 
-    char ch = '-';
+    char *ch;
     if (mucwin->message_char) {
-        ch = mucwin->message_char[0];
+        ch = strdup(mucwin->message_char);
     } else if (enc_mode == PROF_MSG_ENC_OTR) {
         ch = prefs_get_otr_char();
     } else if (enc_mode == PROF_MSG_ENC_PGP) {
         ch = prefs_get_pgp_char();
     } else if (enc_mode == PROF_MSG_ENC_OMEMO) {
         ch = prefs_get_omemo_char();
+    } else {
+        ch = strdup("-");
     }
 
-    win_println_me_message(window, ch, mynick, "%s", message);
+    win_print_outgoing_muc_msg(window, ch, mynick, id, replace_id, message);
+    free(ch);
+
+    // save last id and message for LMC
+    if (id) {
+        _mucwin_set_last_message(mucwin, id, message);
+    }
 }
 
 void
-mucwin_incoming_msg(ProfMucWin *mucwin, ProfMessage *message, GSList *mentions, GList *triggers)
+mucwin_incoming_msg(ProfMucWin *mucwin, const ProfMessage *const message, GSList *mentions, GList *triggers, gboolean filter_reflection)
 {
     assert(mucwin != NULL);
     int flags = 0;
 
-    if (message_is_sent_by_us(message, TRUE)) {
+    if (filter_reflection && message_is_sent_by_us(message, TRUE)) {
         /* Ignore reflection messages */
         return;
     }
@@ -540,27 +543,32 @@ mucwin_incoming_msg(ProfMucWin *mucwin, ProfMessage *message, GSList *mentions,
     ProfWin *window = (ProfWin*)mucwin;
     char *mynick = muc_nick(mucwin->roomjid);
 
-    char ch = '-';
+    char *ch;
     if (mucwin->message_char) {
-        ch = mucwin->message_char[0];
+        ch = strdup(mucwin->message_char);
     } else if (message->enc == PROF_MSG_ENC_OTR) {
         ch = prefs_get_otr_char();
     } else if (message->enc == PROF_MSG_ENC_PGP) {
         ch = prefs_get_pgp_char();
     } else if (message->enc == PROF_MSG_ENC_OMEMO) {
         ch = prefs_get_omemo_char();
+    } else {
+        ch = strdup("-");
     }
 
     win_insert_last_read_position_marker((ProfWin*)mucwin, mucwin->roomjid);
+    wins_add_urls_ac(window, message);
 
     if (g_slist_length(mentions) > 0) {
-        _mucwin_print_mention(window, message->plain, message->jid->resourcepart, mynick, mentions, &ch, flags);
+        _mucwin_print_mention(window, message->plain, message->from_jid->resourcepart, mynick, mentions, ch, flags);
     } else if (triggers) {
-        win_print_them(window, THEME_ROOMTRIGGER, ch, flags, message->jid->resourcepart);
+        win_print_them(window, THEME_ROOMTRIGGER, ch, flags, message->from_jid->resourcepart);
         _mucwin_print_triggers(window, message->plain, triggers);
     } else {
-        win_println_them_message(window, ch, flags, message->jid->resourcepart, "%s", message->plain);
+        win_println_incoming_muc_msg(window, ch, flags, message);
     }
+
+    free(ch);
 }
 
 void
@@ -571,12 +579,12 @@ mucwin_requires_config(ProfMucWin *mucwin)
     ProfWin *window = (ProfWin*)mucwin;
     int num = wins_get_num(window);
 
-    win_println(window, THEME_DEFAULT, '-', "");
-    win_println(window, THEME_ROOMINFO, '!', "Room locked, requires configuration.");
-    win_println(window, THEME_ROOMINFO, '!', "Use '/room accept' to accept the defaults");
-    win_println(window, THEME_ROOMINFO, '!', "Use '/room destroy' to cancel and destroy the room");
-    win_println(window, THEME_ROOMINFO, '!', "Use '/room config' to edit the room configuration");
-    win_println(window, THEME_DEFAULT, '-', "");
+    win_println(window, THEME_DEFAULT, "-", "");
+    win_println(window, THEME_ROOMINFO, "!", "Room locked, requires configuration.");
+    win_println(window, THEME_ROOMINFO, "!", "Use '/room accept' to accept the defaults");
+    win_println(window, THEME_ROOMINFO, "!", "Use '/room destroy' to cancel and destroy the room");
+    win_println(window, THEME_ROOMINFO, "!", "Use '/room config' to edit the room configuration");
+    win_println(window, THEME_DEFAULT, "-", "");
 
     // currently in groupchat window
     if (wins_is_current(window)) {
@@ -596,17 +604,17 @@ mucwin_subject(ProfMucWin *mucwin, const char *const nick, const char *const sub
     ProfWin *window = (ProfWin*)mucwin;
     if (subject) {
         if (nick) {
-            win_print(window, THEME_ROOMINFO, '!', "*%s has set the room subject: ", nick);
+            win_print(window, THEME_ROOMINFO, "!", "*%s has set the room subject: ", nick);
             win_appendln(window, THEME_DEFAULT, "%s", subject);
         } else {
-            win_print(window, THEME_ROOMINFO, '!', "Room subject: ");
+            win_print(window, THEME_ROOMINFO, "!", "Room subject: ");
             win_appendln(window, THEME_DEFAULT, "%s", subject);
         }
     } else {
         if (nick) {
-            win_println(window, THEME_ROOMINFO, '!', "*%s has cleared the room subject.", nick);
+            win_println(window, THEME_ROOMINFO, "!", "*%s has cleared the room subject.", nick);
         } else {
-            win_println(window, THEME_ROOMINFO, '!', "Room subject cleared");
+            win_println(window, THEME_ROOMINFO, "!", "Room subject cleared");
         }
     }
 }
@@ -617,7 +625,7 @@ mucwin_kick_error(ProfMucWin *mucwin, const char *const nick, const char *const
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_println(window, THEME_ERROR, '!', "Error kicking %s: %s", nick, error);
+    win_println(window, THEME_ERROR, "!", "Error kicking %s: %s", nick, error);
 }
 
 void
@@ -628,7 +636,7 @@ mucwin_broadcast(ProfMucWin *mucwin, const char *const message)
     ProfWin *window = (ProfWin*)mucwin;
     int num = wins_get_num(window);
 
-    win_print(window, THEME_ROOMINFO, '!', "Room message: ");
+    win_print(window, THEME_ROOMINFO, "!", "Room message: ");
     win_appendln(window, THEME_DEFAULT, "%s", message);
 
     // currently in groupchat window
@@ -648,7 +656,7 @@ mucwin_affiliation_list_error(ProfMucWin *mucwin, const char *const affiliation,
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_println(window, THEME_ERROR, '!', "Error retrieving %s list: %s", affiliation, error);
+    win_println(window, THEME_ERROR, "!", "Error retrieving %s list: %s", affiliation, error);
 }
 
 void
@@ -658,17 +666,17 @@ mucwin_handle_affiliation_list(ProfMucWin *mucwin, const char *const affiliation
 
     ProfWin *window = (ProfWin*)mucwin;
     if (jids) {
-        win_println(window, THEME_DEFAULT, '!', "Affiliation: %s", affiliation);
+        win_println(window, THEME_DEFAULT, "!", "Affiliation: %s", affiliation);
         GSList *curr_jid = jids;
         while (curr_jid) {
             const char *jid = curr_jid->data;
-            win_println(window, THEME_DEFAULT, '!', "  %s", jid);
+            win_println(window, THEME_DEFAULT, "!", "  %s", jid);
             curr_jid = g_slist_next(curr_jid);
         }
-        win_println(window, THEME_DEFAULT, '!', "");
+        win_println(window, THEME_DEFAULT, "!", "");
     } else {
-        win_println(window, THEME_DEFAULT, '!', "No users found with affiliation: %s", affiliation);
-        win_println(window, THEME_DEFAULT, '!', "");
+        win_println(window, THEME_DEFAULT, "!", "No users found with affiliation: %s", affiliation);
+        win_println(window, THEME_DEFAULT, "!", "");
     }
 }
 
@@ -683,34 +691,34 @@ mucwin_show_affiliation_list(ProfMucWin *mucwin, muc_affiliation_t affiliation)
     if (!occupants) {
         switch (affiliation) {
             case MUC_AFFILIATION_OWNER:
-                win_println(window, THEME_DEFAULT, '!', "No owners found.");
+                win_println(window, THEME_DEFAULT, "!", "No owners found.");
                 break;
             case MUC_AFFILIATION_ADMIN:
-                win_println(window, THEME_DEFAULT, '!', "No admins found.");
+                win_println(window, THEME_DEFAULT, "!", "No admins found.");
                 break;
             case MUC_AFFILIATION_MEMBER:
-                win_println(window, THEME_DEFAULT, '!', "No members found.");
+                win_println(window, THEME_DEFAULT, "!", "No members found.");
                 break;
             case MUC_AFFILIATION_OUTCAST:
-                win_println(window, THEME_DEFAULT, '!', "No outcasts found.");
+                win_println(window, THEME_DEFAULT, "!", "No outcasts found.");
                 break;
             default:
                 break;
         }
-        win_println(window, THEME_DEFAULT, '-', "");
+        win_println(window, THEME_DEFAULT, "-", "");
     } else {
         switch (affiliation) {
             case MUC_AFFILIATION_OWNER:
-                win_println(window, THEME_DEFAULT, '!', "Owners:");
+                win_println(window, THEME_DEFAULT, "!", "Owners:");
                 break;
             case MUC_AFFILIATION_ADMIN:
-                win_println(window, THEME_DEFAULT, '!', "Admins:");
+                win_println(window, THEME_DEFAULT, "!", "Admins:");
                 break;
             case MUC_AFFILIATION_MEMBER:
-                win_println(window, THEME_DEFAULT, '!', "Members:");
+                win_println(window, THEME_DEFAULT, "!", "Members:");
                 break;
             case MUC_AFFILIATION_OUTCAST:
-                win_println(window, THEME_DEFAULT, '!', "Outcasts:");
+                win_println(window, THEME_DEFAULT, "!", "Outcasts:");
                 break;
             default:
                 break;
@@ -721,16 +729,16 @@ mucwin_show_affiliation_list(ProfMucWin *mucwin, muc_affiliation_t affiliation)
             Occupant *occupant = curr_occupant->data;
             if (occupant->affiliation == affiliation) {
                 if (occupant->jid) {
-                    win_println(window, THEME_DEFAULT, '!', "  %s (%s)", occupant->nick, occupant->jid);
+                    win_println(window, THEME_DEFAULT, "!", "  %s (%s)", occupant->nick, occupant->jid);
                 } else {
-                    win_println(window, THEME_DEFAULT, '!', "  %s", occupant->nick);
+                    win_println(window, THEME_DEFAULT, "!", "  %s", occupant->nick);
                 }
             }
 
             curr_occupant = g_slist_next(curr_occupant);
         }
 
-        win_println(window, THEME_DEFAULT, '-', "");
+        win_println(window, THEME_DEFAULT, "-", "");
     }
 }
 
@@ -740,7 +748,7 @@ mucwin_role_list_error(ProfMucWin *mucwin, const char *const role, const char *c
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_println(window, THEME_ERROR, '!', "Error retrieving %s list: %s", role, error);
+    win_println(window, THEME_ERROR, "!", "Error retrieving %s list: %s", role, error);
 }
 
 void
@@ -750,26 +758,26 @@ mucwin_handle_role_list(ProfMucWin *mucwin, const char *const role, GSList *nick
 
     ProfWin *window = (ProfWin*)mucwin;
     if (nicks) {
-        win_println(window, THEME_DEFAULT, '!', "Role: %s", role);
+        win_println(window, THEME_DEFAULT, "!", "Role: %s", role);
         GSList *curr_nick = nicks;
         while (curr_nick) {
             const char *nick = curr_nick->data;
             Occupant *occupant = muc_roster_item(mucwin->roomjid, nick);
             if (occupant) {
                 if (occupant->jid) {
-                    win_println(window, THEME_DEFAULT, '!', "  %s (%s)", nick, occupant->jid);
+                    win_println(window, THEME_DEFAULT, "!", "  %s (%s)", nick, occupant->jid);
                 } else {
-                    win_println(window, THEME_DEFAULT, '!', "  %s", nick);
+                    win_println(window, THEME_DEFAULT, "!", "  %s", nick);
                 }
             } else {
-                win_println(window, THEME_DEFAULT, '!', "  %s", nick);
+                win_println(window, THEME_DEFAULT, "!", "  %s", nick);
             }
             curr_nick = g_slist_next(curr_nick);
         }
-        win_println(window, THEME_DEFAULT, '!', "");
+        win_println(window, THEME_DEFAULT, "!", "");
     } else {
-        win_println(window, THEME_DEFAULT, '!', "No occupants found with role: %s", role);
-        win_println(window, THEME_DEFAULT, '!', "");
+        win_println(window, THEME_DEFAULT, "!", "No occupants found with role: %s", role);
+        win_println(window, THEME_DEFAULT, "!", "");
     }
 }
 
@@ -784,28 +792,28 @@ mucwin_show_role_list(ProfMucWin *mucwin, muc_role_t role)
     if (!occupants) {
         switch (role) {
             case MUC_ROLE_MODERATOR:
-                win_println(window, THEME_DEFAULT, '!', "No moderators found.");
+                win_println(window, THEME_DEFAULT, "!", "No moderators found.");
                 break;
             case MUC_ROLE_PARTICIPANT:
-                win_println(window, THEME_DEFAULT, '!', "No participants found.");
+                win_println(window, THEME_DEFAULT, "!", "No participants found.");
                 break;
             case MUC_ROLE_VISITOR:
-                win_println(window, THEME_DEFAULT, '!', "No visitors found.");
+                win_println(window, THEME_DEFAULT, "!", "No visitors found.");
                 break;
             default:
                 break;
         }
-        win_println(window, THEME_DEFAULT, '-', "");
+        win_println(window, THEME_DEFAULT, "-", "");
     } else {
         switch (role) {
             case MUC_ROLE_MODERATOR:
-                win_println(window, THEME_DEFAULT, '!', "Moderators:");
+                win_println(window, THEME_DEFAULT, "!", "Moderators:");
                 break;
             case MUC_ROLE_PARTICIPANT:
-                win_println(window, THEME_DEFAULT, '!', "Participants:");
+                win_println(window, THEME_DEFAULT, "!", "Participants:");
                 break;
             case MUC_ROLE_VISITOR:
-                win_println(window, THEME_DEFAULT, '!', "Visitors:");
+                win_println(window, THEME_DEFAULT, "!", "Visitors:");
                 break;
             default:
                 break;
@@ -816,16 +824,16 @@ mucwin_show_role_list(ProfMucWin *mucwin, muc_role_t role)
             Occupant *occupant = curr_occupant->data;
             if (occupant->role == role) {
                 if (occupant->jid) {
-                    win_println(window, THEME_DEFAULT, '!', "  %s (%s)", occupant->nick, occupant->jid);
+                    win_println(window, THEME_DEFAULT, "!", "  %s (%s)", occupant->nick, occupant->jid);
                 } else {
-                    win_println(window, THEME_DEFAULT, '!', "  %s", occupant->nick);
+                    win_println(window, THEME_DEFAULT, "!", "  %s", occupant->nick);
                 }
             }
 
             curr_occupant = g_slist_next(curr_occupant);
         }
 
-        win_println(window, THEME_DEFAULT, '-', "");
+        win_println(window, THEME_DEFAULT, "-", "");
     }
 }
 
@@ -836,7 +844,7 @@ mucwin_affiliation_set_error(ProfMucWin *mucwin, const char *const jid, const ch
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_println(window, THEME_ERROR, '!', "Error setting %s affiliation for %s: %s", affiliation, jid, error);
+    win_println(window, THEME_ERROR, "!", "Error setting %s affiliation for %s: %s", affiliation, jid, error);
 }
 
 void
@@ -846,7 +854,7 @@ mucwin_role_set_error(ProfMucWin *mucwin, const char *const nick, const char *co
     assert(mucwin != NULL);
 
     ProfWin *window = (ProfWin*)mucwin;
-    win_println(window, THEME_ERROR, '!', "Error setting %s role for %s: %s", role, nick, error);
+    win_println(window, THEME_ERROR, "!", "Error setting %s role for %s: %s", role, nick, error);
 }
 
 void
@@ -858,10 +866,10 @@ mucwin_info(ProfMucWin *mucwin)
     char *affiliation = muc_affiliation_str(mucwin->roomjid);
 
     ProfWin *window = (ProfWin*) mucwin;
-    win_println(window, THEME_DEFAULT, '!', "Room: %s", mucwin->roomjid);
-    win_println(window, THEME_DEFAULT, '!', "Affiliation: %s", affiliation);
-    win_println(window, THEME_DEFAULT, '!', "Role: %s", role);
-    win_println(window, THEME_DEFAULT, '-', "");
+    win_println(window, THEME_DEFAULT, "!", "Room: %s", mucwin->roomjid);
+    win_println(window, THEME_DEFAULT, "!", "Affiliation: %s", affiliation);
+    win_println(window, THEME_DEFAULT, "!", "Role: %s", role);
+    win_println(window, THEME_DEFAULT, "-", "");
 }
 
 void
@@ -951,3 +959,13 @@ mucwin_unset_message_char(ProfMucWin *mucwin)
         mucwin->message_char = NULL;
     }
 }
+
+static void
+_mucwin_set_last_message(ProfMucWin *mucwin, const char *const id, const char *const message)
+{
+    free(mucwin->last_message);
+    mucwin->last_message = strdup(message);
+
+    free(mucwin->last_msg_id);
+    mucwin->last_msg_id = strdup(id);
+}
