@@ -172,6 +172,7 @@ rosterwin_roster(void)
             _rosterwin_private_chats(layout, orphaned_privchats);
         }
         prefs_free_string(privpref);
+        g_list_free(privchats);
         g_list_free(orphaned_privchats);
     }
 
@@ -353,7 +354,7 @@ _rosterwin_contact(ProfLayoutSplit *layout, PContact contact)
     }
 
     theme_item_t presence_colour = _get_roster_theme(theme_type, presence);
-    int colour;
+    int colour = 0;
     if (prefs_get_boolean(PREF_ROSTER_COLOR_NICK)) {
         colour = theme_hash_attrs(name);
         wattron(layout->subwin, colour);
@@ -562,6 +563,7 @@ _rosterwin_resources(ProfLayoutSplit *layout, PContact contact, int current_inde
 
                 wattron(layout->subwin, theme_attrs(presence_colour));
                 win_sub_print(layout->subwin, unreadmsg->str, FALSE, wrap, current_indent);
+                g_string_free(unreadmsg, TRUE);
                 wattroff(layout->subwin, theme_attrs(presence_colour));
             }
             prefs_free_string(unreadpos);
@@ -617,6 +619,7 @@ _rosterwin_resources(ProfLayoutSplit *layout, PContact contact, int current_inde
 
             wattron(layout->subwin, theme_attrs(presence_colour));
             win_sub_print(layout->subwin, unreadmsg->str, FALSE, wrap, current_indent);
+            g_string_free(unreadmsg, TRUE);
             wattroff(layout->subwin, theme_attrs(presence_colour));
         }
         prefs_free_string(unreadpos);
@@ -633,6 +636,7 @@ _rosterwin_resources(ProfLayoutSplit *layout, PContact contact, int current_inde
 
             wattron(layout->subwin, theme_attrs(presence_colour));
             win_sub_print(layout->subwin, unreadmsg->str, FALSE, wrap, current_indent);
+            g_string_free(unreadmsg, TRUE);
             wattroff(layout->subwin, theme_attrs(presence_colour));
         }
         prefs_free_string(unreadpos);
@@ -1290,4 +1294,3 @@ _filter_contacts_with_presence(GSList *contacts, const char *const presence)
 
     return filtered_contacts;
 }
-
