@@ -61,10 +61,12 @@ static gboolean typing;
 static GTimer *typing_elapsed;
 
 static void _title_bar_draw(void);
-static void _show_self_presence(void);
-static void _show_contact_presence(ProfChatWin *chatwin);
+static void  _show_self_presence(void);
+static int  _calc_self_presence(void);
+static void _show_contact_presence(ProfChatWin *chatwin, int pos, int maxpos);
 static void _show_privacy(ProfChatWin *chatwin);
 static void _show_muc_privacy(ProfMucWin *mucwin);
+static void _show_scrolled(ProfWin *current);
 
 void
 create_title_bar(void)
@@ -182,6 +184,8 @@ title_bar_set_typing(gboolean is_typing)
 static void
 _title_bar_draw(void)
 {
+    int pos;
+    int maxrightpos;
     ProfWin *current = wins_get_current();
 
     werase(win);
@@ -191,34 +195,23 @@ _title_bar_draw(void)
         waddch(win, ' ');
     }
 
-    char *title = NULL;
-    if (current && current->type == WIN_MUC) {
-        char *use_as_name = prefs_get_string(PREF_TITLEBAR_MUC_TITLE);
-
-        if ((g_strcmp0(use_as_name, "name") == 0)) {
-            ProfMucWin *mucwin = (ProfMucWin*) current;
-            assert(mucwin->memcheck == PROFMUCWIN_MEMCHECK);
-
-            if (mucwin->room_name) {
-                title = strdup(mucwin->room_name);
-            }
-        }
-
-        prefs_free_string(use_as_name);
-    }
-
-    if (title == NULL) {
-        title = win_get_title(current);
-    }
+    char *title = win_get_title(current);
 
     mvwprintw(win, 0, 0, " %s", title);
+    pos = strlen(title) + 1;
+
     free(title);
 
+    // presence is written from the right side
+    // calculate it first so we have a maxposition
+    maxrightpos = _calc_self_presence();
+
     if (current && current->type == WIN_CHAT) {
         ProfChatWin *chatwin = (ProfChatWin*) current;
         assert(chatwin->memcheck == PROFCHATWIN_MEMCHECK);
-        _show_contact_presence(chatwin);
+        _show_contact_presence(chatwin, pos, maxrightpos);
         _show_privacy(chatwin);
+        _show_scrolled(current);
 
         if (typing) {
             wprintw(win, " (typing...)");
@@ -227,6 +220,7 @@ _title_bar_draw(void)
         ProfMucWin *mucwin = (ProfMucWin*) current;
         assert(mucwin->memcheck == PROFMUCWIN_MEMCHECK);
         _show_muc_privacy(mucwin);
+        _show_scrolled(current);
     }
 
     _show_self_presence();
@@ -235,6 +229,57 @@ _title_bar_draw(void)
     inp_put_back();
 }
 
+static void
+_show_scrolled(ProfWin *current)
+{
+    if (current && current->layout->paged == 1) {
+        int bracket_attrs = theme_attrs(THEME_TITLE_BRACKET);
+        int scrolled_attrs = theme_attrs(THEME_TITLE_SCROLLED);
+
+        wattron(win, bracket_attrs);
+        wprintw(win, "[");
+        wattroff(win, bracket_attrs);
+
+        wattron(win, scrolled_attrs);
+        wprintw(win, "SCROLLED");
+        wattroff(win, scrolled_attrs);
+
+        wattron(win, bracket_attrs);
+        wprintw(win, "]");
+        wattroff(win, bracket_attrs);
+    }
+}
+
+static int
+_calc_self_presence(void)
+{
+    int tls_start = 0;
+
+    switch (current_presence)
+    {
+        case CONTACT_ONLINE:
+            tls_start = 15;
+            break;
+        case CONTACT_AWAY:
+            tls_start = 13;
+            break;
+        case CONTACT_DND:
+            tls_start = 12;
+            break;
+        case CONTACT_CHAT:
+            tls_start = 13;
+            break;
+        case CONTACT_XA:
+            tls_start = 11;
+            break;
+        case CONTACT_OFFLINE:
+            tls_start = 16;
+            break;
+    }
+
+    return tls_start - 1;
+}
+
 static void
 _show_self_presence(void)
 {
@@ -486,7 +531,7 @@ _show_privacy(ProfChatWin *chatwin)
 }
 
 static void
-_show_contact_presence(ProfChatWin *chatwin)
+_show_contact_presence(ProfChatWin *chatwin, int pos, int maxpos)
 {
     int bracket_attrs = theme_attrs(THEME_TITLE_BRACKET);
     char *resource = NULL;
@@ -497,9 +542,13 @@ _show_contact_presence(ProfChatWin *chatwin)
     } else if (session && session->resource) {
         resource = session->resource;
     }
+
     if (resource && prefs_get_boolean(PREF_RESOURCE_TITLE)) {
-        wprintw(win, "/");
-        wprintw(win, resource);
+        int needed = strlen(resource) + 1;
+        if (pos + needed < maxpos) {
+            wprintw(win, "/");
+            wprintw(win, resource);
+        }
     }
 
     if (prefs_get_boolean(PREF_PRESENCE)) {
