@@ -3,6 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2019 - 2020 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -56,7 +57,6 @@
 #define NO_COLOUR_DATE  16
 #define UNTRUSTED       32
 
-
 // core UI
 void ui_init(void);
 void ui_load_colours(void);
@@ -124,8 +124,7 @@ ProfChatWin* chatwin_new(const char *const barejid);
 void chatwin_incoming_msg(ProfChatWin *chatwin, ProfMessage *message, gboolean win_created);
 void chatwin_receipt_received(ProfChatWin *chatwin, const char *const id);
 void chatwin_recipient_gone(ProfChatWin *chatwin);
-void chatwin_outgoing_msg(ProfChatWin *chatwin, const char *const message, char *id, prof_enc_t enc_mode,
-    gboolean request_receipt);
+void chatwin_outgoing_msg(ProfChatWin *chatwin, const char *const message, char *id, prof_enc_t enc_mode, gboolean request_receipt, const char *const replace_id);
 void chatwin_outgoing_carbon(ProfChatWin *chatwin, ProfMessage *message);
 void chatwin_contact_online(ProfChatWin *chatwin, Resource *resource, GDateTime *last_activity);
 void chatwin_contact_offline(ProfChatWin *chatwin, char *resource, char *status);
@@ -158,9 +157,9 @@ void mucwin_occupant_affiliation_change(ProfMucWin *mucwin, const char *const ni
 void mucwin_occupant_role_and_affiliation_change(ProfMucWin *mucwin, const char *const nick,
     const char *const role, const char *const affiliation, const char *const actor, const char *const reason);
 void mucwin_roster(ProfMucWin *mucwin, GList *occupants, const char *const presence);
-void mucwin_history(ProfMucWin *mucwin, const char *const nick, GDateTime *timestamp, const char *const message);
-void mucwin_outgoing_msg(ProfMucWin *mucwin, const char *const message, const char *const id, prof_enc_t enc_mode);
-void mucwin_incoming_msg(ProfMucWin *mucwin, ProfMessage *message, GSList *mentions, GList *triggers);
+void mucwin_history(ProfMucWin *mucwin, const ProfMessage *const message);
+void mucwin_outgoing_msg(ProfMucWin *mucwin, const char *const message, const char *const id, prof_enc_t enc_mode, const char *const replace_id);
+void mucwin_incoming_msg(ProfMucWin *mucwin, const ProfMessage *const message, GSList *mentions, GList *triggers, gboolean filter_reflection);
 void mucwin_subject(ProfMucWin *mucwin, const char *const nick, const char *const subject);
 void mucwin_requires_config(ProfMucWin *mucwin);
 void mucwin_info(ProfMucWin *mucwin);
@@ -269,6 +268,7 @@ void cons_show_login_success(ProfAccount *account, gboolean secured);
 void cons_show_account_list(gchar **accounts);
 void cons_show_room_list(GSList *room, const char *const conference_node);
 void cons_show_bookmarks(const GList *list);
+void cons_show_bookmarks_ignore(gchar **list, gsize len);
 void cons_show_disco_items(GSList *items, const char *const jid);
 void cons_show_disco_info(const char *from, GSList *identities, GSList *features);
 void cons_show_room_invite(const char *const invitor, const char *const room, const char *const reason);
@@ -319,6 +319,9 @@ void cons_statusbar_setting(void);
 void cons_winpos_setting(void);
 void cons_color_setting(void);
 void cons_os_setting(void);
+void cons_correction_setting(void);
+void cons_executable_setting(void);
+void cons_slashguard_setting(void);
 void cons_show_contact_online(PContact contact, Resource *resource, GDateTime *last_activity);
 void cons_show_contact_offline(PContact contact, char *resource, char *status);
 void cons_theme_properties(void);
@@ -360,8 +363,8 @@ void win_show_subwin(ProfWin *window);
 void win_refresh_without_subwin(ProfWin *window);
 void win_refresh_with_subwin(ProfWin *window);
 
-void win_print(ProfWin *window, theme_item_t theme_item, const char ch, const char *const message, ...);
-void win_println(ProfWin *window, theme_item_t theme_item, const char ch, const char *const message, ...);
+void win_print(ProfWin *window, theme_item_t theme_item, const char *show_char, const char *const message, ...);
+void win_println(ProfWin *window, theme_item_t theme_item, const char *show_char, const char *const message, ...);
 void win_println_indent(ProfWin *window, int pad, const char *const message, ...);
 
 void win_append(ProfWin *window, theme_item_t theme_item, const char *const message, ...);
