@@ -3,6 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2019 - 2020 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -49,6 +50,7 @@
 #include <ncurses.h>
 #endif
 
+#include "log.h"
 #include "config/theme.h"
 #include "config/preferences.h"
 #include "ui/ui.h"
@@ -62,9 +64,9 @@
 
 #define CEILING(X) (X-(int)(X) > 0 ? (int)(X+1) : (int)(X))
 
-static void _win_printf(ProfWin *window, const char show_char, int pad_indent, GDateTime *timestamp,
-    int flags, theme_item_t theme_item, const char *const from, const char *const message, ...);
-static void _win_print(ProfWin *window, const char show_char, int pad_indent, GDateTime *time,
+static void
+_win_printf(ProfWin *window, const char *show_char, int pad_indent, GDateTime *timestamp, int flags, theme_item_t theme_item, const char *const display_from, const char *const from_jid, const char *const message_id, const char *const message, ...);
+static void _win_print_internal(ProfWin *window, const char *show_char, int pad_indent, GDateTime *time,
     int flags, theme_item_t theme_item, const char *const from, const char *const message, DeliveryReceipt *receipt);
 static void _win_print_wrapped(WINDOW *win, const char *const message, size_t indent, int pad_indent);
 
@@ -151,6 +153,8 @@ win_create_chat(const char *const barejid)
     new_win->enctext = NULL;
     new_win->incoming_char = NULL;
     new_win->outgoing_char = NULL;
+    new_win->last_message = NULL;
+    new_win->last_msg_id = NULL;
 
     new_win->memcheck = PROFCHATWIN_MEMCHECK;
 
@@ -164,7 +168,6 @@ win_create_muc(const char *const roomjid)
     int cols = getmaxx(stdscr);
 
     new_win->window.type = WIN_MUC;
-    new_win->window.layout = _win_create_simple_layout();
     ProfLayoutSplit *layout = malloc(sizeof(ProfLayoutSplit));
     layout->base.type = LAYOUT_SPLIT;
 
@@ -200,6 +203,8 @@ win_create_muc(const char *const roomjid)
     new_win->enctext = NULL;
     new_win->message_char = NULL;
     new_win->is_omemo = FALSE;
+    new_win->last_message = NULL;
+    new_win->last_msg_id = NULL;
 
     new_win->memcheck = PROFMUCWIN_MEMCHECK;
 
@@ -294,7 +299,22 @@ win_get_title(ProfWin *window)
     if (window->type == WIN_MUC) {
         ProfMucWin *mucwin = (ProfMucWin*) window;
         assert(mucwin->memcheck == PROFMUCWIN_MEMCHECK);
-        return strdup(mucwin->roomjid);
+
+        gboolean show_titlebar_jid = prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_JID);
+        gboolean show_titlebar_name = prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_NAME);
+        GString *title = g_string_new("");
+
+        if (show_titlebar_name) {
+            g_string_append(title, mucwin->room_name);
+            g_string_append(title, " ");
+        }
+        if (show_titlebar_jid) {
+            g_string_append(title, mucwin->roomjid);
+        }
+
+        char *title_str = title->str;
+        g_string_free(title, FALSE);
+        return title_str;
     }
     if (window->type == WIN_CONFIG) {
         ProfConfWin *confwin = (ProfConfWin*) window;
@@ -487,6 +507,8 @@ win_free(ProfWin* window)
         free(chatwin->enctext);
         free(chatwin->incoming_char);
         free(chatwin->outgoing_char);
+        free(chatwin->last_message);
+        free(chatwin->last_msg_id);
         chat_state_free(chatwin->state);
         break;
     }
@@ -497,6 +519,8 @@ win_free(ProfWin* window)
         free(mucwin->room_name);
         free(mucwin->enctext);
         free(mucwin->message_char);
+        free(mucwin->last_message);
+        free(mucwin->last_msg_id);
         break;
     }
     case WIN_CONFIG:
@@ -714,22 +738,23 @@ win_refresh_without_subwin(ProfWin *window)
 void
 win_refresh_with_subwin(ProfWin *window)
 {
-    int cols = getmaxx(stdscr);
     int subwin_cols = 0;
-
+    int cols = getmaxx(stdscr);
     int row_start = screen_mainwin_row_start();
     int row_end = screen_mainwin_row_end();
+    ProfLayoutSplit *layout = (ProfLayoutSplit*)window->layout;
+
     if (window->type == WIN_MUC) {
-        ProfLayoutSplit *layout = (ProfLayoutSplit*)window->layout;
         subwin_cols = win_occpuants_cols();
-        pnoutrefresh(layout->base.win, layout->base.y_pos, 0, row_start, 0, row_end, (cols-subwin_cols)-1);
-        pnoutrefresh(layout->subwin, layout->sub_y_pos, 0, row_start, (cols-subwin_cols), row_end, cols-1);
     } else if (window->type == WIN_CONSOLE) {
-        ProfLayoutSplit *layout = (ProfLayoutSplit*)window->layout;
         subwin_cols = win_roster_cols();
-        pnoutrefresh(layout->base.win, layout->base.y_pos, 0, row_start, 0, row_end, (cols-subwin_cols)-1);
-        pnoutrefresh(layout->subwin, layout->sub_y_pos, 0, row_start, (cols-subwin_cols), row_end, cols-1);
+    } else {
+        // Other window types don't support subwindows, we shouldn't be here
+        return;
     }
+
+    pnoutrefresh(layout->base.win, layout->base.y_pos, 0, row_start, 0, row_end, (cols-subwin_cols)-1);
+    pnoutrefresh(layout->subwin, layout->sub_y_pos, 0, row_start, (cols-subwin_cols), row_end, cols-1);
 }
 
 void
@@ -754,7 +779,7 @@ win_show_occupant(ProfWin *window, Occupant *occupant)
 
     theme_item_t presence_colour = theme_main_presence_attrs(presence_str);
 
-    win_print(window, presence_colour, '-', "%s", occupant->nick);
+    win_print(window, presence_colour, "-", "%s", occupant->nick);
     win_append(window, presence_colour, " is %s", presence_str);
 
     if (occupant->status) {
@@ -776,9 +801,9 @@ win_show_contact(ProfWin *window, PContact contact)
     theme_item_t presence_colour = theme_main_presence_attrs(presence);
 
     if (name) {
-        win_print(window, presence_colour, '-', "%s", name);
+        win_print(window, presence_colour, "-", "%s", name);
     } else {
-        win_print(window, presence_colour, '-', "%s", barejid);
+        win_print(window, presence_colour, "-", "%s", barejid);
     }
 
     win_append(window, presence_colour, " is %s", presence);
@@ -817,7 +842,7 @@ win_show_occupant_info(ProfWin *window, const char *const room, Occupant *occupa
 
     theme_item_t presence_colour = theme_main_presence_attrs(presence_str);
 
-    win_print(window, presence_colour, '!', "%s", occupant->nick);
+    win_print(window, presence_colour, "!", "%s", occupant->nick);
     win_append(window, presence_colour, " is %s", presence_str);
 
     if (occupant->status) {
@@ -827,11 +852,11 @@ win_show_occupant_info(ProfWin *window, const char *const room, Occupant *occupa
     win_newline(window);
 
     if (occupant->jid) {
-        win_println(window, THEME_DEFAULT, '!', "  Jid: %s", occupant->jid);
+        win_println(window, THEME_DEFAULT, "!", "  Jid: %s", occupant->jid);
     }
 
-    win_println(window, THEME_DEFAULT, '!', "  Affiliation: %s", occupant_affiliation);
-    win_println(window, THEME_DEFAULT, '!', "  Role: %s", occupant_role);
+    win_println(window, THEME_DEFAULT, "!", "  Affiliation: %s", occupant_affiliation);
+    win_println(window, THEME_DEFAULT, "!", "  Role: %s", occupant_role);
 
     Jid *jidp = jid_create_from_bare_and_resource(room, occupant->nick);
     EntityCapabilities *caps = caps_lookup(jidp->fulljid);
@@ -841,7 +866,7 @@ win_show_occupant_info(ProfWin *window, const char *const room, Occupant *occupa
         // show identity
         if (caps->identity) {
             DiscoIdentity *identity = caps->identity;
-            win_print(window, THEME_DEFAULT, '!', "  Identity: ");
+            win_print(window, THEME_DEFAULT, "!", "  Identity: ");
             if (identity->name) {
                 win_append(window, THEME_DEFAULT, "%s", identity->name);
                 if (identity->category || identity->type) {
@@ -863,7 +888,7 @@ win_show_occupant_info(ProfWin *window, const char *const room, Occupant *occupa
         if (caps->software_version) {
             SoftwareVersion *software_version = caps->software_version;
             if (software_version->software) {
-                win_print(window, THEME_DEFAULT, '!', "  Software: %s", software_version->software);
+                win_print(window, THEME_DEFAULT, "!", "  Software: %s", software_version->software);
             }
             if (software_version->software_version) {
                 win_append(window, THEME_DEFAULT, ", %s", software_version->software_version);
@@ -872,7 +897,7 @@ win_show_occupant_info(ProfWin *window, const char *const room, Occupant *occupa
                 win_newline(window);
             }
             if (software_version->os) {
-                win_print(window, THEME_DEFAULT, '!', "  OS: %s", software_version->os);
+                win_print(window, THEME_DEFAULT, "!", "  OS: %s", software_version->os);
             }
             if (software_version->os_version) {
                 win_append(window, THEME_DEFAULT, ", %s", software_version->os_version);
@@ -885,7 +910,7 @@ win_show_occupant_info(ProfWin *window, const char *const room, Occupant *occupa
         caps_destroy(caps);
     }
 
-    win_println(window, THEME_DEFAULT, '-', "");
+    win_println(window, THEME_DEFAULT, "-", "");
 }
 
 void
@@ -899,15 +924,15 @@ win_show_info(ProfWin *window, PContact contact)
 
     theme_item_t presence_colour = theme_main_presence_attrs(presence);
 
-    win_println(window, THEME_DEFAULT, '-', "");
-    win_print(window, presence_colour, '-', "%s", barejid);
+    win_println(window, THEME_DEFAULT, "-", "");
+    win_print(window, presence_colour, "-", "%s", barejid);
     if (name) {
         win_append(window, presence_colour, " (%s)", name);
     }
     win_appendln(window, THEME_DEFAULT, ":");
 
     if (sub) {
-        win_println(window, THEME_DEFAULT, '-', "Subscription: %s", sub);
+        win_println(window, THEME_DEFAULT, "-", "Subscription: %s", sub);
     }
 
     if (last_activity) {
@@ -921,10 +946,10 @@ win_show_info(ProfWin *window, PContact contact)
         int seconds = span / G_TIME_SPAN_SECOND;
 
         if (hours > 0) {
-          win_println(window, THEME_DEFAULT, '-', "Last activity: %dh%dm%ds", hours, minutes, seconds);
+          win_println(window, THEME_DEFAULT, "-", "Last activity: %dh%dm%ds", hours, minutes, seconds);
         }
         else {
-          win_println(window, THEME_DEFAULT, '-', "Last activity: %dm%ds", minutes, seconds);
+          win_println(window, THEME_DEFAULT, "-", "Last activity: %dm%ds", minutes, seconds);
         }
 
         g_date_time_unref(now);
@@ -933,7 +958,7 @@ win_show_info(ProfWin *window, PContact contact)
     GList *resources = p_contact_get_available_resources(contact);
     GList *ordered_resources = NULL;
     if (resources) {
-        win_println(window, THEME_DEFAULT, '-', "Resources:");
+        win_println(window, THEME_DEFAULT, "-", "Resources:");
 
         // sort in order of availability
         GList *curr = resources;
@@ -951,7 +976,7 @@ win_show_info(ProfWin *window, PContact contact)
         Resource *resource = curr->data;
         const char *resource_presence = string_from_resource_presence(resource->presence);
         theme_item_t presence_colour = theme_main_presence_attrs(resource_presence);
-        win_print(window, presence_colour, '-', "  %s (%d), %s", resource->name, resource->priority, resource_presence);
+        win_print(window, presence_colour, "-", "  %s (%d), %s", resource->name, resource->priority, resource_presence);
         if (resource->status) {
             win_append(window, presence_colour, ", \"%s\"", resource->status);
         }
@@ -965,7 +990,7 @@ win_show_info(ProfWin *window, PContact contact)
             // show identity
             if (caps->identity) {
                 DiscoIdentity *identity = caps->identity;
-                win_print(window, THEME_DEFAULT, '-', "    Identity: ");
+                win_print(window, THEME_DEFAULT, "-", "    Identity: ");
                 if (identity->name) {
                     win_append(window, THEME_DEFAULT, "%s", identity->name);
                     if (identity->category || identity->type) {
@@ -987,7 +1012,7 @@ win_show_info(ProfWin *window, PContact contact)
             if (caps->software_version) {
                 SoftwareVersion *software_version = caps->software_version;
                 if (software_version->software) {
-                    win_print(window, THEME_DEFAULT, '-', "    Software: %s", software_version->software);
+                    win_print(window, THEME_DEFAULT, "-", "    Software: %s", software_version->software);
                 }
                 if (software_version->software_version) {
                     win_append(window, THEME_DEFAULT, ", %s", software_version->software_version);
@@ -996,7 +1021,7 @@ win_show_info(ProfWin *window, PContact contact)
                     win_newline(window);
                 }
                 if (software_version->os) {
-                    win_print(window, THEME_DEFAULT, '-', "    OS: %s", software_version->os);
+                    win_print(window, THEME_DEFAULT, "-", "    OS: %s", software_version->os);
                 }
                 if (software_version->os_version) {
                     win_append(window, THEME_DEFAULT, ", %s", software_version->os_version);
@@ -1030,7 +1055,7 @@ win_show_status_string(ProfWin *window, const char *const from,
         presence_colour = THEME_OFFLINE;
     }
 
-    win_print(window, presence_colour, '-', "%s %s", pre, from);
+    win_print(window, presence_colour, "-", "%s %s", pre, from);
 
     if (show)
         win_append(window, presence_colour, " is %s", show);
@@ -1055,10 +1080,51 @@ win_show_status_string(ProfWin *window, const char *const from,
     win_appendln(window, presence_colour, "");
 }
 
+static void
+_win_correct(ProfWin *window, const char *const message, const char *const id, const char *const replace_id, const char *const from_jid)
+{
+    ProfBuffEntry *entry = buffer_get_entry_by_id(window->layout->buffer, replace_id);
+    if (!entry) {
+        log_debug("Replace ID %s could not be found in buffer. Message: %s", replace_id, message);
+        return;
+    }
+
+    if (g_strcmp0(entry->from_jid, from_jid) != 0) {
+        log_debug("Illicit LMC attempt from %s for message from %s with: %s", from_jid, entry->from_jid, message);
+        cons_show("Illicit LMC attempt from %s for message from %s", from_jid, entry->from_jid);
+        return;
+    }
+
+    /*TODO: set date?
+    if (entry->date) {
+        if (entry->date->timestamp) {
+            g_date_time_unref(entry->date->timestamp);
+        }
+        free(entry->date);
+    }
+
+    entry->date = buffer_date_new_now();
+    */
+
+    free(entry->show_char);
+    entry->show_char = prefs_get_correction_char();
+
+    if (entry->message) {
+        free(entry->message);
+    }
+    entry->message = strdup(message);
+
+    if (entry->id) {
+        free(entry->id);
+    }
+    entry->id = strdup(id);
+
+    win_redraw(window);
+}
+
 void
-win_print_incoming(ProfWin *window, const char *const from, ProfMessage *message)
+win_print_incoming(ProfWin *window, const char *const display_name_from, ProfMessage *message)
 {
-    char enc_char = '-';
     int flags = NO_ME;
 
     if (!message->trusted) {
@@ -1069,21 +1135,32 @@ win_print_incoming(ProfWin *window, const char *const from, ProfMessage *message
     {
         case WIN_CHAT:
         {
+            char *enc_char;
             ProfChatWin *chatwin = (ProfChatWin*)window;
+
             if (chatwin->incoming_char) {
-                enc_char = chatwin->incoming_char[0];
+                enc_char = strdup(chatwin->incoming_char);
             } else if (message->enc == PROF_MSG_ENC_OTR) {
                 enc_char = prefs_get_otr_char();
             } else if (message->enc == PROF_MSG_ENC_PGP) {
                 enc_char = prefs_get_pgp_char();
             } else if (message->enc == PROF_MSG_ENC_OMEMO) {
                 enc_char = prefs_get_omemo_char();
+            } else {
+                enc_char = strdup("-");
             }
-            _win_printf(window, enc_char, 0, message->timestamp, flags, THEME_TEXT_THEM, from, "%s", message->plain);
+
+            if (prefs_get_boolean(PREF_CORRECTION_ALLOW) && message->replace_id) {
+                _win_correct(window, message->plain, message->id, message->replace_id, message->from_jid->barejid);
+            } else {
+                _win_printf(window, enc_char, 0, message->timestamp, flags, THEME_TEXT_THEM, display_name_from, message->from_jid->barejid, message->id, "%s", message->plain);
+            }
+
+            free(enc_char);
             break;
         }
         case WIN_PRIVATE:
-            _win_printf(window, '-', 0, message->timestamp, flags, THEME_TEXT_THEM, from, "%s", message->plain);
+            _win_printf(window, "-", 0, message->timestamp, flags, THEME_TEXT_THEM, display_name_from, message->from_jid->barejid, message->id, "%s", message->plain);
             break;
         default:
             assert(FALSE);
@@ -1092,13 +1169,84 @@ win_print_incoming(ProfWin *window, const char *const from, ProfMessage *message
 }
 
 void
-win_print_them(ProfWin *window, theme_item_t theme_item, char ch, int flags, const char *const them)
+win_print_them(ProfWin *window, theme_item_t theme_item, const char *const show_char, int flags, const char *const them)
 {
-    _win_printf(window, ch, 0, NULL, flags | NO_ME | NO_EOL, theme_item, them, "");
+    _win_printf(window, show_char, 0, NULL, flags | NO_ME | NO_EOL, theme_item, them, NULL, NULL, "");
 }
 
 void
-win_println_them_message(ProfWin *window, char ch, int flags, const char *const them, const char *const message, ...)
+win_println_incoming_muc_msg(ProfWin *window, char *show_char, int flags, const ProfMessage *const message)
+{
+    if (prefs_get_boolean(PREF_CORRECTION_ALLOW) && message->replace_id) {
+        _win_correct(window, message->plain, message->id, message->replace_id, message->from_jid->fulljid);
+    } else {
+        _win_printf(window, show_char, 0, message->timestamp, flags | NO_ME, THEME_TEXT_THEM, message->from_jid->resourcepart, message->from_jid->fulljid, message->id, "%s", message->plain);
+    }
+
+    inp_nonblocking(TRUE);
+}
+
+void
+win_print_outgoing_muc_msg(ProfWin *window, char *show_char, const char *const me, const char *const id, const char *const replace_id, const char *const message)
+{
+    GDateTime *timestamp = g_date_time_new_now_local();
+
+    if (prefs_get_boolean(PREF_CORRECTION_ALLOW) && replace_id) {
+        _win_correct(window, message, id, replace_id, me);
+    } else {
+        _win_printf(window, show_char, 0, timestamp, 0, THEME_TEXT_ME, me, me, id, "%s", message);
+    }
+
+    inp_nonblocking(TRUE);
+    g_date_time_unref(timestamp);
+}
+
+void
+win_print_outgoing(ProfWin *window, const char *show_char, const char *const id, const char *const replace_id, const char *const message)
+{
+    GDateTime *timestamp = g_date_time_new_now_local();
+
+    const char *myjid = connection_get_fulljid();
+    if (replace_id) {
+        _win_correct(window, message, id, replace_id, myjid);
+    } else {
+        //TODO my jid
+        _win_printf(window, show_char, 0, timestamp, 0, THEME_TEXT_THEM, "me", myjid, id, "%s", message);
+    }
+
+    inp_nonblocking(TRUE);
+    g_date_time_unref(timestamp);
+}
+
+void
+win_print_history(ProfWin *window, const ProfMessage *const message)
+{
+    g_date_time_ref(message->timestamp);
+
+    char *display_name;
+    int flags = 0;
+    const char *jid = connection_get_fulljid();
+    Jid *jidp = jid_create(jid);
+
+    if (g_strcmp0(jidp->barejid, message->from_jid->barejid) == 0) {
+        display_name = strdup("me");
+    } else {
+        display_name = roster_get_msg_display_name(message->from_jid->barejid, message->from_jid->resourcepart);
+    }
+
+    jid_destroy(jidp);
+
+    buffer_append(window->layout->buffer, "-", 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, NULL, message->plain, NULL, NULL);
+    _win_print_internal(window, "-", 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, message->plain, NULL);
+
+    free(display_name);
+
+    inp_nonblocking(TRUE);
+    g_date_time_unref(message->timestamp);
+}
+
+void
+win_print(ProfWin *window, theme_item_t theme_item, const char *show_char, const char *const message, ...)
 {
     GDateTime *timestamp = g_date_time_new_now_local();
 
@@ -1107,68 +1255,8 @@ win_println_them_message(ProfWin *window, char ch, int flags, const char *const
     GString *fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, message, arg);
 
-    buffer_append(window->layout->buffer, ch, 0, timestamp, flags | NO_ME, THEME_TEXT_THEM, them, fmt_msg->str, NULL, NULL);
-
-    _win_print(window, ch, 0, timestamp, flags | NO_ME, THEME_TEXT_THEM, them, fmt_msg->str, NULL);
-    inp_nonblocking(TRUE);
-    g_date_time_unref(timestamp);
-
-    g_string_free(fmt_msg, TRUE);
-    va_end(arg);
-}
-
-void
-win_println_me_message(ProfWin *window, char ch, const char *const me, const char *const message, ...)
-{
-    GDateTime *timestamp = g_date_time_new_now_local();
-
-    va_list arg;
-    va_start(arg, message);
-    GString *fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
-
-    buffer_append(window->layout->buffer, ch, 0, timestamp, 0, THEME_TEXT_ME, me, fmt_msg->str, NULL, NULL);
-
-    _win_print(window, ch, 0, timestamp, 0, THEME_TEXT_ME, me, fmt_msg->str, NULL);
-    inp_nonblocking(TRUE);
-    g_date_time_unref(timestamp);
-
-    g_string_free(fmt_msg, TRUE);
-    va_end(arg);
-}
-
-void
-win_print_outgoing(ProfWin *window, const char ch, const char *const message, ...)
-{
-    GDateTime *timestamp = g_date_time_new_now_local();
-
-    va_list arg;
-    va_start(arg, message);
-    GString *fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
-
-    buffer_append(window->layout->buffer, ch, 0, timestamp, 0, THEME_TEXT_ME, "me", fmt_msg->str, NULL, NULL);
-
-    _win_print(window, ch, 0, timestamp, 0, THEME_TEXT_ME, "me", fmt_msg->str, NULL);
-    inp_nonblocking(TRUE);
-    g_date_time_unref(timestamp);
-
-    g_string_free(fmt_msg, TRUE);
-    va_end(arg);
-}
-
-void
-win_print_history(ProfWin *window, GDateTime *timestamp, const char *const message, ...)
-{
-    g_date_time_ref(timestamp);
-
-    va_list arg;
-    va_start(arg, message);
-    GString *fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
-
-    buffer_append(window->layout->buffer, '-', 0, timestamp, 0, THEME_TEXT_HISTORY, "", fmt_msg->str, NULL, NULL);
-    _win_print(window, '-', 0, timestamp, 0, THEME_TEXT_HISTORY, "", fmt_msg->str, NULL);
+    buffer_append(window->layout->buffer, show_char, 0, timestamp, NO_EOL, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
+    _win_print_internal(window, show_char, 0, timestamp, NO_EOL, theme_item, "", fmt_msg->str, NULL);
 
     inp_nonblocking(TRUE);
     g_date_time_unref(timestamp);
@@ -1178,7 +1266,7 @@ win_print_history(ProfWin *window, GDateTime *timestamp, const char *const messa
 }
 
 void
-win_print(ProfWin *window, theme_item_t theme_item, const char ch, const char *const message, ...)
+win_println(ProfWin *window, theme_item_t theme_item, const char *show_char, const char *const message, ...)
 {
     GDateTime *timestamp = g_date_time_new_now_local();
 
@@ -1187,29 +1275,9 @@ win_print(ProfWin *window, theme_item_t theme_item, const char ch, const char *c
     GString *fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, message, arg);
 
-    buffer_append(window->layout->buffer, ch, 0, timestamp, NO_EOL, theme_item, "", fmt_msg->str, NULL, NULL);
+    buffer_append(window->layout->buffer, show_char, 0, timestamp, 0, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
+    _win_print_internal(window, show_char, 0, timestamp, 0, theme_item, "", fmt_msg->str, NULL);
 
-    _win_print(window, ch, 0, timestamp, NO_EOL, theme_item, "", fmt_msg->str, NULL);
-    inp_nonblocking(TRUE);
-    g_date_time_unref(timestamp);
-
-    g_string_free(fmt_msg, TRUE);
-    va_end(arg);
-}
-
-void
-win_println(ProfWin *window, theme_item_t theme_item, const char ch, const char *const message, ...)
-{
-    GDateTime *timestamp = g_date_time_new_now_local();
-
-    va_list arg;
-    va_start(arg, message);
-    GString *fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
-
-    buffer_append(window->layout->buffer, ch, 0, timestamp, 0, theme_item, "", fmt_msg->str, NULL, NULL);
-
-    _win_print(window, ch, 0, timestamp, 0, theme_item, "", fmt_msg->str, NULL);
     inp_nonblocking(TRUE);
     g_date_time_unref(timestamp);
 
@@ -1227,9 +1295,9 @@ win_println_indent(ProfWin *window, int pad, const char *const message, ...)
     GString *fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, message, arg);
 
-    buffer_append(window->layout->buffer, '-', pad, timestamp, 0, THEME_DEFAULT, "", fmt_msg->str, NULL, NULL);
+    buffer_append(window->layout->buffer, "-", pad, timestamp, 0, THEME_DEFAULT, "", NULL, fmt_msg->str, NULL, NULL);
+    _win_print_internal(window, "-", pad, timestamp, 0, THEME_DEFAULT, "", fmt_msg->str, NULL);
 
-    _win_print(window, '-', pad, timestamp, 0, THEME_DEFAULT, "", fmt_msg->str, NULL);
     inp_nonblocking(TRUE);
     g_date_time_unref(timestamp);
 
@@ -1247,9 +1315,9 @@ win_append(ProfWin *window, theme_item_t theme_item, const char *const message,
     GString *fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, message, arg);
 
-    buffer_append(window->layout->buffer, '-', 0, timestamp, NO_DATE | NO_EOL, theme_item, "", fmt_msg->str, NULL, NULL);
+    buffer_append(window->layout->buffer, "-", 0, timestamp, NO_DATE | NO_EOL, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
+    _win_print_internal(window, "-", 0, timestamp, NO_DATE | NO_EOL, theme_item, "", fmt_msg->str, NULL);
 
-    _win_print(window, '-', 0, timestamp, NO_DATE | NO_EOL, theme_item, "", fmt_msg->str, NULL);
     inp_nonblocking(TRUE);
     g_date_time_unref(timestamp);
 
@@ -1267,9 +1335,9 @@ win_appendln(ProfWin *window, theme_item_t theme_item, const char *const message
     GString *fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, message, arg);
 
-    buffer_append(window->layout->buffer, '-', 0, timestamp, NO_DATE, theme_item, "", fmt_msg->str, NULL, NULL);
+    buffer_append(window->layout->buffer, "-", 0, timestamp, NO_DATE, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
+    _win_print_internal(window, "-", 0, timestamp, NO_DATE, theme_item, "", fmt_msg->str, NULL);
 
-    _win_print(window, '-', 0, timestamp, NO_DATE, theme_item, "", fmt_msg->str, NULL);
     inp_nonblocking(TRUE);
     g_date_time_unref(timestamp);
 
@@ -1287,9 +1355,9 @@ win_append_highlight(ProfWin *window, theme_item_t theme_item, const char *const
     GString *fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, message, arg);
 
-    buffer_append(window->layout->buffer, '-', 0, timestamp, NO_DATE | NO_ME | NO_EOL, theme_item, "", fmt_msg->str, NULL, NULL);
+    buffer_append(window->layout->buffer, "-", 0, timestamp, NO_DATE | NO_ME | NO_EOL, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
+    _win_print_internal(window, "-", 0, timestamp, NO_DATE | NO_ME | NO_EOL, theme_item, "", fmt_msg->str, NULL);
 
-    _win_print(window, '-', 0, timestamp, NO_DATE | NO_ME | NO_EOL, theme_item, "", fmt_msg->str, NULL);
     inp_nonblocking(TRUE);
     g_date_time_unref(timestamp);
 
@@ -1307,9 +1375,9 @@ win_appendln_highlight(ProfWin *window, theme_item_t theme_item, const char *con
     GString *fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, message, arg);
 
-    buffer_append(window->layout->buffer, '-', 0, timestamp, NO_DATE | NO_ME, theme_item, "", fmt_msg->str, NULL, NULL);
+    buffer_append(window->layout->buffer, "-", 0, timestamp, NO_DATE | NO_ME, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
+    _win_print_internal(window, "-", 0, timestamp, NO_DATE | NO_ME, theme_item, "", fmt_msg->str, NULL);
 
-    _win_print(window, '-', 0, timestamp, NO_DATE | NO_ME, theme_item, "", fmt_msg->str, NULL);
     inp_nonblocking(TRUE);
     g_date_time_unref(timestamp);
 
@@ -1320,19 +1388,25 @@ win_appendln_highlight(ProfWin *window, theme_item_t theme_item, const char *con
 void
 win_print_http_upload(ProfWin *window, const char *const message, char *url)
 {
-    win_print_with_receipt(window, '!', NULL, message, url);
+    win_print_outgoing_with_receipt(window, "!", NULL, message, url, NULL);
 }
 
 void
-win_print_with_receipt(ProfWin *window, const char show_char, const char *const from, const char *const message, char *id)
+win_print_outgoing_with_receipt(ProfWin *window, const char *show_char, const char *const from, const char *const message, char *id, const char *const replace_id)
 {
     GDateTime *time = g_date_time_new_now_local();
 
     DeliveryReceipt *receipt = malloc(sizeof(struct delivery_receipt_t));
     receipt->received = FALSE;
 
-    buffer_append(window->layout->buffer, show_char, 0, time, 0, THEME_TEXT_ME, from, message, receipt, id);
-    _win_print(window, show_char, 0, time, 0, THEME_TEXT_ME, from, message, receipt);
+    const char *myjid = connection_get_fulljid();
+    if (replace_id) {
+        _win_correct(window, message, id, replace_id, myjid);
+    } else {
+        buffer_append(window->layout->buffer, show_char, 0, time, 0, THEME_TEXT_ME, from, myjid, message, receipt, id);
+        _win_print_internal(window, show_char, 0, time, 0, THEME_TEXT_ME, from, message, receipt);
+    }
+
     // TODO: cross-reference.. this should be replaced by a real event-based system
     inp_nonblocking(TRUE);
     g_date_time_unref(time);
@@ -1372,8 +1446,7 @@ win_newline(ProfWin *window)
 }
 
 static void
-_win_printf(ProfWin *window, const char show_char, int pad_indent, GDateTime *timestamp,
-    int flags, theme_item_t theme_item, const char *const from, const char *const message, ...)
+_win_printf(ProfWin *window, const char *show_char, int pad_indent, GDateTime *timestamp, int flags, theme_item_t theme_item, const char *const display_from, const char *const from_jid, const char *const message_id, const char *const message, ...)
 {
     if (timestamp == NULL) {
         timestamp = g_date_time_new_now_local();
@@ -1386,9 +1459,10 @@ _win_printf(ProfWin *window, const char show_char, int pad_indent, GDateTime *ti
     GString *fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, message, arg);
 
-    buffer_append(window->layout->buffer, show_char, pad_indent, timestamp, flags, theme_item, from, fmt_msg->str, NULL, NULL);
+    buffer_append(window->layout->buffer, show_char, pad_indent, timestamp, flags, theme_item, display_from, from_jid, fmt_msg->str, NULL, message_id);
+
+    _win_print_internal(window, show_char, pad_indent, timestamp, flags, theme_item, display_from, fmt_msg->str, NULL);
 
-    _win_print(window, show_char, pad_indent, timestamp, flags, theme_item, from, fmt_msg->str, NULL);
     inp_nonblocking(TRUE);
     g_date_time_unref(timestamp);
 
@@ -1397,15 +1471,15 @@ _win_printf(ProfWin *window, const char show_char, int pad_indent, GDateTime *ti
 }
 
 static void
-_win_print(ProfWin *window, const char show_char, int pad_indent, GDateTime *time,
+_win_print_internal(ProfWin *window, const char *show_char, int pad_indent, GDateTime *time,
     int flags, theme_item_t theme_item, const char *const from, const char *const message, DeliveryReceipt *receipt)
 {
-    // flags : 1st bit =  0/1 - me/not me
-    //         2nd bit =  0/1 - date/no date
-    //         3rd bit =  0/1 - eol/no eol
-    //         4th bit =  0/1 - color from/no color from
-    //         5th bit =  0/1 - color date/no date
-    //         6th bit =  0/1 - trusted/untrusted
+    // flags : 1st bit =  0/1 - me/not me. define: NO_ME
+    //         2nd bit =  0/1 - date/no date. define: NO_DATE
+    //         3rd bit =  0/1 - eol/no eol. define: NO_EOL
+    //         4th bit =  0/1 - color from/no color from. define: NO_COLOUR_FROM
+    //         5th bit =  0/1 - color date/no date. define: NO_COLOUR_DATE
+    //         6th bit =  0/1 - trusted/untrusted. define: UNTRUSTED
     gboolean me_message = FALSE;
     int offset = 0;
     int colour = theme_attrs(THEME_ME);
@@ -1452,7 +1526,7 @@ _win_print(ProfWin *window, const char show_char, int pad_indent, GDateTime *tim
                 wbkgdset(window->layout->win, theme_attrs(THEME_TIME));
                 wattron(window->layout->win, theme_attrs(THEME_TIME));
             }
-            wprintw(window->layout->win, "%s %c ", date_fmt, show_char);
+            wprintw(window->layout->win, "%s %s ", date_fmt, show_char);
             if ((flags & NO_COLOUR_DATE) == 0) {
                 wattroff(window->layout->win, theme_attrs(THEME_TIME));
             }
@@ -1466,7 +1540,9 @@ _win_print(ProfWin *window, const char show_char, int pad_indent, GDateTime *tim
 
         char *color_pref = prefs_get_string(PREF_COLOR_NICK);
         if (color_pref != NULL && (strcmp(color_pref, "false") != 0)) {
-            colour = theme_hash_attrs(from);
+            if (flags & NO_ME || (!(flags & NO_ME) && prefs_get_boolean(PREF_COLOR_NICK_OWN))) {
+                colour = theme_hash_attrs(from);
+            }
         }
         prefs_free_string(color_pref);
 
@@ -1655,14 +1731,20 @@ _win_print_wrapped(WINDOW *win, const char *const message, size_t indent, int pa
 }
 
 void
-win_print_separator(ProfWin *window)
+win_print_trackbar(ProfWin *window)
 {
     int cols = getmaxx(window->layout->win);
 
+    wbkgdset(window->layout->win, theme_attrs(THEME_TRACKBAR));
+    wattron(window->layout->win, theme_attrs(THEME_TRACKBAR));
+
     int i;
     for (i=1; i<cols; i++) {
         wprintw(window->layout->win, "-");
     }
+
+    wattroff(window->layout->win, theme_attrs(THEME_TRACKBAR));
+
     wprintw(window->layout->win, "\n");
 }
 
@@ -1676,12 +1758,12 @@ win_redraw(ProfWin *window)
     for (i = 0; i < size; i++) {
         ProfBuffEntry *e = buffer_get_entry(window->layout->buffer, i);
 
-        if (e->from == NULL && e->message && e->message[0] == '-') {
-            // just an indicator to print the separator not the actual message
-            win_print_separator(window);
+        if (e->display_from == NULL && e->message && e->message[0] == '-') {
+            // just an indicator to print the trackbar/separator not the actual message
+            win_print_trackbar(window);
         } else {
             // regular thing to print
-            _win_print(window, e->show_char, e->pad_indent, e->time, e->flags, e->theme_item, e->from, e->message, e->receipt);
+            _win_print_internal(window, e->show_char, e->pad_indent, e->time, e->flags, e->theme_item, e->display_from, e->message, e->receipt);
         }
     }
 }
@@ -1788,7 +1870,7 @@ win_command_list_error(ProfWin *window, const char *const error)
 {
     assert(window != NULL);
 
-    win_println(window, THEME_ERROR, '!', "Error retrieving command list: %s", error);
+    win_println(window, THEME_ERROR, "!", "Error retrieving command list: %s", error);
 }
 
 void
@@ -1800,7 +1882,7 @@ win_command_exec_error(ProfWin *window, const char *const command, const char *c
     GString *msg = g_string_new(NULL);
     g_string_vprintf(msg, error, arg);
 
-    win_println(window, THEME_ERROR, '!', "Error executing command %s: %s", command, msg->str);
+    win_println(window, THEME_ERROR, "!", "Error executing command %s: %s", command, msg->str);
 
     g_string_free(msg, TRUE);
     va_end(arg);
@@ -1812,17 +1894,17 @@ win_handle_command_list(ProfWin *window, GSList *cmds)
     assert(window != NULL);
 
     if (cmds) {
-        win_println(window, THEME_DEFAULT, '!', "Ad hoc commands:");
+        win_println(window, THEME_DEFAULT, "!", "Ad hoc commands:");
         GSList *curr_cmd = cmds;
         while (curr_cmd) {
             const char *cmd = curr_cmd->data;
-            win_println(window, THEME_DEFAULT, '!', "  %s", cmd);
+            win_println(window, THEME_DEFAULT, "!", "  %s", cmd);
             curr_cmd = g_slist_next(curr_cmd);
         }
-        win_println(window, THEME_DEFAULT, '!', "");
+        win_println(window, THEME_DEFAULT, "!", "");
     } else {
-        win_println(window, THEME_DEFAULT, '!', "No commands found");
-        win_println(window, THEME_DEFAULT, '!', "");
+        win_println(window, THEME_DEFAULT, "!", "No commands found");
+        win_println(window, THEME_DEFAULT, "!", "");
     }
 }
 
@@ -1830,14 +1912,14 @@ void
 win_handle_command_exec_status(ProfWin *window, const char *const command, const char *const value)
 {
     assert(window != NULL);
-    win_println(window, THEME_DEFAULT, '!', "%s %s", command, value);
+    win_println(window, THEME_DEFAULT, "!", "%s %s", command, value);
 }
 
 void
 win_handle_command_exec_result_note(ProfWin *window, const char *const type, const char *const value)
 {
     assert(window != NULL);
-    win_println(window, THEME_DEFAULT, '!', value);
+    win_println(window, THEME_DEFAULT, "!", value);
 }
 
 void
@@ -1859,10 +1941,10 @@ win_insert_last_read_position_marker(ProfWin *window, char* id)
 
     GDateTime *time = g_date_time_new_now_local();
 
-    // the separator will actually be print in win_redraw().
+    // the trackbar/separator will actually be print in win_redraw().
     // this only puts it in the buffer and win_redraw() will interpret it.
     // so that we have the correct length even when resizing.
-    buffer_append(window->layout->buffer, ' ', 0, time, 0, THEME_TEXT, NULL, "-", NULL, id);
+    buffer_append(window->layout->buffer, " ", 0, time, 0, THEME_TEXT, NULL, NULL, "-", NULL, id);
     win_redraw(window);
 
     g_date_time_unref(time);
