@@ -50,6 +50,7 @@
 #include "ui/window_list.h"
 #include "xmpp/xmpp.h"
 #include "xmpp/roster_list.h"
+#include "tools/http_upload.h"
 
 static GHashTable *windows;
 static int current;
@@ -279,7 +280,7 @@ wins_private_nick_change(const char *const roomjid, const char *const oldnick, c
 
         Jid *newjid = jid_create_from_bare_and_resource(roomjid, newnick);
         privwin->fulljid = strdup(newjid->fulljid);
-        win_println((ProfWin*)privwin, THEME_THEM, '!', "** %s is now known as %s.", oldjid->resourcepart, newjid->resourcepart);
+        win_println((ProfWin*)privwin, THEME_THEM, "!", "** %s is now known as %s.", oldjid->resourcepart, newjid->resourcepart);
 
         autocomplete_remove(wins_ac, oldjid->fulljid);
         autocomplete_remove(wins_close_ac, oldjid->fulljid);
@@ -497,12 +498,6 @@ wins_get_current_num(void)
     return current;
 }
 
-void
-wins_close_current(void)
-{
-    wins_close_by_num(current);
-}
-
 void
 wins_close_by_num(int i)
 {
@@ -519,15 +514,7 @@ wins_close_by_num(int i)
         ProfWin *window = wins_get_by_num(i);
         if (window) {
             // cancel upload proccesses of this window
-            GSList *upload_process = upload_processes;
-            while (upload_process) {
-                HTTPUpload *upload = upload_process->data;
-                if (upload->window == window) {
-                    upload->cancel = 1;
-                    break;
-                }
-                upload_process = g_slist_next(upload_process);
-            }
+            http_upload_cancel_processes(window);
 
             switch (window->type) {
             case WIN_CHAT:
@@ -547,7 +534,7 @@ wins_close_by_num(int i)
                         }
                     }
                 }
-
+                autocomplete_free(window->urls_ac);
                 break;
             }
             case WIN_MUC:
@@ -559,6 +546,7 @@ wins_close_by_num(int i)
                 if (mucwin->last_msg_timestamp) {
                     g_date_time_unref(mucwin->last_msg_timestamp);
                 }
+                autocomplete_free(window->urls_ac);
                 break;
             }
             case WIN_PRIVATE:
@@ -566,6 +554,7 @@ wins_close_by_num(int i)
                 ProfPrivateWin *privwin = (ProfPrivateWin*)window;
                 autocomplete_remove(wins_ac, privwin->fulljid);
                 autocomplete_remove(wins_close_ac, privwin->fulljid);
+                autocomplete_free(window->urls_ac);
                 break;
             }
             case WIN_XML:
@@ -637,6 +626,7 @@ wins_new_chat(const char *const barejid)
             autocomplete_add(wins_close_ac, nick);
         }
     }
+    newwin->urls_ac = autocomplete_new();
 
     return newwin;
 }
@@ -651,6 +641,8 @@ wins_new_muc(const char *const roomjid)
     g_hash_table_insert(windows, GINT_TO_POINTER(result), newwin);
     autocomplete_add(wins_ac, roomjid);
     autocomplete_add(wins_close_ac, roomjid);
+    newwin->urls_ac = autocomplete_new();
+
     return newwin;
 }
 
@@ -662,6 +654,7 @@ wins_new_config(const char *const roomjid, DataForm *form, ProfConfWinCallback s
     g_list_free(keys);
     ProfWin *newwin = win_create_config(roomjid, form, submit, cancel, userdata);
     g_hash_table_insert(windows, GINT_TO_POINTER(result), newwin);
+
     return newwin;
 }
 
@@ -675,6 +668,8 @@ wins_new_private(const char *const fulljid)
     g_hash_table_insert(windows, GINT_TO_POINTER(result), newwin);
     autocomplete_add(wins_ac, fulljid);
     autocomplete_add(wins_close_ac, fulljid);
+    newwin->urls_ac = autocomplete_new();
+
     return newwin;
 }
 
@@ -755,6 +750,11 @@ wins_show_subwin(ProfWin *window)
 {
     win_show_subwin(window);
 
+    // only mucwin and console have occupants/roster subwin
+    if (window->type != WIN_MUC && window->type != WIN_CONSOLE) {
+        return;
+    }
+
     ProfWin *current_win = wins_get_current();
     win_refresh_with_subwin(current_win);
 }
@@ -830,7 +830,7 @@ wins_lost_connection(void)
     while (curr) {
         ProfWin *window = curr->data;
         if (window->type != WIN_CONSOLE) {
-            win_println(window, THEME_ERROR, '-', "Lost connection.");
+            win_println(window, THEME_ERROR, "-", "Lost connection.");
 
             // if current win, set current_win_dirty
             if (wins_is_current(window)) {
@@ -851,7 +851,7 @@ wins_reestablished_connection(void)
     while (curr) {
         ProfWin *window = curr->data;
         if (window->type != WIN_CONSOLE) {
-            win_println(window, THEME_TEXT, '-', "Connection re-established.");
+            win_println(window, THEME_TEXT, "-", "Connection re-established.");
 
             // if current win, set current_win_dirty
             if (wins_is_current(window)) {
@@ -1149,3 +1149,36 @@ wins_get_next_unread(void)
     g_list_free(values);
     return NULL;
 }
+
+void
+wins_add_urls_ac(const ProfWin *const win, const ProfMessage *const message)
+{
+    GRegex *regex;
+    GMatchInfo *match_info;
+
+    regex = g_regex_new("https?://\\S+", 0, 0, NULL);
+    g_regex_match (regex, message->plain, 0, &match_info);
+
+    while (g_match_info_matches (match_info))
+    {
+        gchar *word = g_match_info_fetch (match_info, 0);
+
+        autocomplete_add_reverse(win->urls_ac, word);
+        // for people who run profanity a long time, we don't want to waste a lot of memory
+        autocomplete_remove_older_than_max_reverse(win->urls_ac, 20);
+
+        g_free (word);
+        g_match_info_next (match_info, NULL);
+    }
+
+    g_match_info_free (match_info);
+    g_regex_unref (regex);
+}
+
+char*
+wins_get_url(const char *const search_str, gboolean previous, void *context)
+{
+    ProfWin *win = (ProfWin*)context;
+
+    return autocomplete_complete(win->urls_ac, search_str, FALSE, previous);
+}
