@@ -46,13 +46,13 @@ xmlwin_show(ProfXMLWin *xmlwin, const char *const msg)
 
     ProfWin *window = (ProfWin*)xmlwin;
     if (g_str_has_prefix(msg, "SENT:")) {
-        win_println(window, THEME_DEFAULT, '-', "SENT:");
-        win_println(window, THEME_ONLINE, '-', "%s", &msg[6]);
-        win_println(window, THEME_ONLINE, '-', "");
+        win_println(window, THEME_DEFAULT, "-", "SENT:");
+        win_println(window, THEME_ONLINE, "-", "%s", &msg[6]);
+        win_println(window, THEME_ONLINE, "-", "");
     } else if (g_str_has_prefix(msg, "RECV:")) {
-        win_println(window, THEME_DEFAULT, '-', "RECV:");
-        win_println(window, THEME_AWAY, '-', "%s", &msg[6]);
-        win_println(window, THEME_AWAY, '-', "");
+        win_println(window, THEME_DEFAULT, "-", "RECV:");
+        win_println(window, THEME_AWAY, "-", "%s", &msg[6]);
+        win_println(window, THEME_AWAY, "-", "");
     }
 }
 
