@@ -59,6 +59,7 @@
 #include "xmpp/connection.h"
 #include "xmpp/session.h"
 #include "xmpp/iq.h"
+#include "ui/ui.h"
 
 typedef struct prof_conn_t {
     xmpp_log_t *xmpp_log;
@@ -134,8 +135,10 @@ connection_shutdown(void)
 
 jabber_conn_status_t
 connection_connect(const char *const jid, const char *const passwd, const char *const altdomain, int port,
-    const char *const tls_policy)
+    const char *const tls_policy, const char *const auth_policy)
 {
+    long flags;
+
     assert(jid != NULL);
     assert(passwd != NULL);
 
@@ -175,15 +178,43 @@ connection_connect(const char *const jid, const char *const passwd, const char *
     xmpp_conn_set_jid(conn.xmpp_conn, jid);
     xmpp_conn_set_pass(conn.xmpp_conn, passwd);
 
+    flags = xmpp_conn_get_flags(conn.xmpp_conn);
+
     if (!tls_policy || (g_strcmp0(tls_policy, "force") == 0)) {
-        xmpp_conn_set_flags(conn.xmpp_conn, XMPP_CONN_FLAG_MANDATORY_TLS);
+        flags |= XMPP_CONN_FLAG_MANDATORY_TLS;
     } else if (g_strcmp0(tls_policy, "trust") == 0) {
-        xmpp_conn_set_flags(conn.xmpp_conn, XMPP_CONN_FLAG_MANDATORY_TLS);
-        xmpp_conn_set_flags(conn.xmpp_conn, XMPP_CONN_FLAG_TRUST_TLS);
+        flags |= XMPP_CONN_FLAG_MANDATORY_TLS;
+        flags |= XMPP_CONN_FLAG_TRUST_TLS;
     } else if (g_strcmp0(tls_policy, "disable") == 0) {
-        xmpp_conn_set_flags(conn.xmpp_conn, XMPP_CONN_FLAG_DISABLE_TLS);
+        flags |= XMPP_CONN_FLAG_DISABLE_TLS;
     } else if (g_strcmp0(tls_policy, "legacy") == 0) {
-        xmpp_conn_set_flags(conn.xmpp_conn, XMPP_CONN_FLAG_LEGACY_SSL);
+        flags |= XMPP_CONN_FLAG_LEGACY_SSL;
+    }
+
+    if (auth_policy && (g_strcmp0(auth_policy, "legacy") == 0)) {
+#ifdef HAVE_LIBSTROPHE_LT_0_9_3
+        log_warning("Legacy authentication is requested, but it hasn't been "
+                    "built. Update libstrophe and rebuild Profanity.");
+        cons_show("Legacy authentication is requested, but it hasn't been built.");
+#else
+        flags |= XMPP_CONN_FLAG_LEGACY_AUTH;
+#endif /* HAVE_LIBSTROPHE_LT_0_9_3 */
+    }
+
+    xmpp_conn_set_flags(conn.xmpp_conn, flags);
+
+    /* Print debug logs that can help when users share the logs */
+    if (flags != 0) {
+        log_debug("Connecting with flags (0x%lx):", flags);
+#define LOG_FLAG_IF_SET(name) if (flags & name) { log_debug("  " #name); }
+        LOG_FLAG_IF_SET(XMPP_CONN_FLAG_MANDATORY_TLS);
+        LOG_FLAG_IF_SET(XMPP_CONN_FLAG_TRUST_TLS);
+        LOG_FLAG_IF_SET(XMPP_CONN_FLAG_DISABLE_TLS);
+        LOG_FLAG_IF_SET(XMPP_CONN_FLAG_LEGACY_SSL);
+#ifndef HAVE_LIBSTROPHE_LT_0_9_3
+        LOG_FLAG_IF_SET(XMPP_CONN_FLAG_LEGACY_AUTH);
+#endif /* HAVE_LIBSTROPHE_LT_0_9_3 */
+#undef LOG_FLAG_IF_SET
     }
 
 #ifdef HAVE_LIBMESODE
@@ -411,6 +442,18 @@ connection_get_fulljid(void)
     }
 }
 
+char*
+connection_get_barejid(void) {
+    const char *jid = connection_get_fulljid();
+    char *result;
+
+    Jid *jidp = jid_create(jid);
+    result = strdup(jidp->barejid);
+    jid_destroy(jidp);
+
+    return result;
+}
+
 void
 connection_features_received(const char *const jid)
 {
@@ -655,7 +698,7 @@ static void _random_bytes_init(void)
         gsize g_data_size;
         gchar *g_accounts_data = g_key_file_to_data(rndbytes, &g_data_size, NULL);
 
-        gchar *base = g_path_get_basename(rndbytes_loc);
+        gchar *base = g_path_get_dirname(rndbytes_loc);
         gchar *true_loc = get_file_or_linked(rndbytes_loc, base);
         g_file_set_contents(true_loc, g_accounts_data, g_data_size, NULL);
 
