@@ -1286,15 +1286,27 @@ stanza_get_oldest_delay(xmpp_stanza_t *const stanza)
         if (child_name && strcmp(child_name, STANZA_NAME_DELAY) == 0) {
             GDateTime *tmp = _stanza_get_delay_timestamp_xep0203(child);
 
-            if (!oldest || g_date_time_compare(oldest, tmp) == 1)
+            if (oldest == NULL) {
                 oldest = tmp;
+            } else if (g_date_time_compare(oldest, tmp) == 1) {
+                g_date_time_unref(oldest);
+                oldest = tmp;
+            } else {
+                g_date_time_unref(tmp);
+            }
         }
 
         if (child_name && strcmp(child_name, STANZA_NAME_X) == 0) {
             GDateTime *tmp = _stanza_get_delay_timestamp_xep0091(child);
 
-            if (!oldest || g_date_time_compare(oldest, tmp) == 1)
+            if (oldest == NULL) {
                 oldest = tmp;
+            } else if (g_date_time_compare(oldest, tmp) == 1) {
+                g_date_time_unref(oldest);
+                oldest = tmp;
+            } else {
+                g_date_time_unref(tmp);
+            }
         }
     }
 
@@ -1772,13 +1784,21 @@ stanza_create_caps_from_query_element(xmpp_stanza_t *query)
                 formField = field->data;
                 if (formField->values) {
                     if (strcmp(formField->var, "software") == 0) {
-                        software = formField->values->data;
+                        if (software == NULL) {
+                            software = strdup(formField->values->data);
+                        }
                     } else if (strcmp(formField->var, "software_version") == 0) {
-                        software_version = formField->values->data;
+                        if (software_version == NULL) {
+                            software_version = strdup(formField->values->data);
+                        }
                     } else if (strcmp(formField->var, "os") == 0) {
-                        os = formField->values->data;
+                        if (os == NULL) {
+                            os = strdup(formField->values->data);
+                        }
                     } else if (strcmp(formField->var, "os_version") == 0) {
-                        os_version = formField->values->data;
+                        if (os_version == NULL) {
+                            os_version = strdup(formField->values->data);
+                        }
                     }
                 }
                 field = g_slist_next(field);
@@ -1860,6 +1880,10 @@ stanza_create_caps_from_query_element(xmpp_stanza_t *query)
 
     EntityCapabilities *result = caps_create(category, type, name, software, software_version, os, os_version, features);
     g_slist_free_full(features, free);
+    free(software);
+    free(software_version);
+    free(os);
+    free(os_version);
 
     return result;
 }
@@ -2606,3 +2630,98 @@ stanza_create_avatar_retrieve_data_request(xmpp_ctx_t *ctx, const char *stanza_i
 
     return iq;
 }
+
+xmpp_stanza_t*
+stanza_attach_correction(xmpp_ctx_t *ctx, xmpp_stanza_t *stanza, const char *const replace_id)
+{
+    xmpp_stanza_t *replace_stanza = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(replace_stanza, "replace");
+    xmpp_stanza_set_id(replace_stanza, replace_id);
+    xmpp_stanza_set_ns(replace_stanza, STANZA_NS_LAST_MESSAGE_CORRECTION);
+    xmpp_stanza_add_child(stanza, replace_stanza);
+    xmpp_stanza_release(replace_stanza);
+
+    return stanza;
+}
+
+xmpp_stanza_t*
+stanza_create_mam_iq(xmpp_ctx_t *ctx, const char *const jid, const char *const startdate)
+{
+    char *id = connection_create_stanza_id();
+    xmpp_stanza_t *iq = xmpp_iq_new(ctx, STANZA_TYPE_SET, id);
+    free(id);
+    //xmpp_stanza_set_to(iq, jid);
+
+    xmpp_stanza_t *query = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(query, STANZA_NAME_QUERY);
+    xmpp_stanza_set_ns(query, STANZA_NS_MAM2);
+
+    xmpp_stanza_t *x = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(x, STANZA_NAME_X);
+    xmpp_stanza_set_ns(x, STANZA_NS_DATA);
+    xmpp_stanza_set_type(x, "submit");
+
+    // field FORM_TYPE MAM2
+    xmpp_stanza_t *field_form_type = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(field_form_type, STANZA_NAME_FIELD);
+    xmpp_stanza_set_attribute(field_form_type, STANZA_ATTR_VAR, "FORM_TYPE");
+    xmpp_stanza_set_type(field_form_type, "hidden");
+
+    xmpp_stanza_t *value_mam = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(value_mam, STANZA_NAME_VALUE);
+
+    xmpp_stanza_t *mam_text = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_text(mam_text, STANZA_NS_MAM2);
+    xmpp_stanza_add_child(value_mam, mam_text);
+
+    xmpp_stanza_add_child(field_form_type, value_mam);
+
+    // field 'with'
+    xmpp_stanza_t *field_with = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(field_with, STANZA_NAME_FIELD);
+    xmpp_stanza_set_attribute(field_with, STANZA_ATTR_VAR, "with");
+
+    xmpp_stanza_t *value_with = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(value_with, STANZA_NAME_VALUE);
+
+    xmpp_stanza_t *with_text = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_text(with_text, jid);
+    xmpp_stanza_add_child(value_with, with_text);
+
+    xmpp_stanza_add_child(field_with, value_with);
+
+    // field 'start'
+    xmpp_stanza_t *field_start = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(field_start, STANZA_NAME_FIELD);
+    xmpp_stanza_set_attribute(field_start, STANZA_ATTR_VAR, "start");
+
+    xmpp_stanza_t *value_start = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_name(value_start, STANZA_NAME_VALUE);
+
+    xmpp_stanza_t *date_text = xmpp_stanza_new(ctx);
+    xmpp_stanza_set_text(date_text, startdate);
+    xmpp_stanza_add_child(value_start, date_text);
+
+    xmpp_stanza_add_child(field_start, value_start);
+
+    // add and release
+    xmpp_stanza_add_child(iq, query);
+    xmpp_stanza_add_child(query, x);
+    xmpp_stanza_add_child(x, field_form_type);
+    xmpp_stanza_add_child(x, field_with);
+    xmpp_stanza_add_child(x, field_start);
+
+    xmpp_stanza_release(mam_text);
+    xmpp_stanza_release(with_text);
+    xmpp_stanza_release(date_text);
+    xmpp_stanza_release(value_mam);
+    xmpp_stanza_release(value_with);
+    xmpp_stanza_release(value_start);
+    xmpp_stanza_release(field_form_type);
+    xmpp_stanza_release(field_with);
+    xmpp_stanza_release(field_start);
+    xmpp_stanza_release(x);
+    xmpp_stanza_release(query);
+
+    return iq;
+}
