@@ -68,6 +68,8 @@
 #define XMPP_FEATURE_PUBSUB "http://jabber.org/protocol/pubsub"
 #define XMPP_FEATURE_PUBSUB_PUBLISH_OPTIONS "http://jabber.org/protocol/pubsub#publish-options"
 #define XMPP_FEATURE_USER_AVATAR_METADATA_NOTIFY "urn:xmpp:avatar:metadata+notify"
+#define XMPP_FEATURE_LAST_MESSAGE_CORRECTION "urn:xmpp:message-correct:0"
+#define XMPP_FEATURE_MAM2 "urn:xmpp:mam:2"
 
 typedef enum {
     JABBER_CONNECTING,
@@ -91,7 +93,9 @@ typedef struct bookmark_t {
     char *barejid;
     char *nick;
     char *password;
+    char *name;
     gboolean autojoin;
+    int ext_gajim_minimize; //0 - non existant, 1 - true, 2 - false
 } Bookmark;
 
 typedef struct disco_identity_t {
@@ -119,17 +123,32 @@ typedef struct disco_item_t {
 } DiscoItem;
 
 typedef enum {
-    PROF_MSG_ENC_PLAIN,
+    PROF_MSG_ENC_NONE,
     PROF_MSG_ENC_OTR,
     PROF_MSG_ENC_PGP,
     PROF_MSG_ENC_OMEMO
 } prof_enc_t;
 
+typedef enum {
+    PROF_MSG_TYPE_UNINITIALIZED,
+    // regular 1:1 chat
+    PROF_MSG_TYPE_CHAT,
+    // groupchats to whole group
+    PROF_MSG_TYPE_MUC,
+    // groupchat private message
+    PROF_MSG_TYPE_MUCPM
+} prof_msg_type_t;
+
 typedef struct prof_message_t {
-   Jid *jid;
+   Jid *from_jid;
+   Jid *to_jid;
+   /* regular <message id=""> */
    char *id;
    /* </origin-id> XEP-0359 */
    char *originid;
+   /* <replace id> XEP-0308 LMC */
+   char *replace_id;
+   /* for MAM we will need archive_id (stanza-id in XEP-0359) (see database.c) */
    /* The raw body from xmpp message, either plaintext or OTR encrypted text */
    char *body;
    /* The encrypted message as for PGP */
@@ -139,12 +158,13 @@ typedef struct prof_message_t {
    GDateTime *timestamp;
    prof_enc_t enc;
    gboolean trusted;
-   gboolean mucuser;
+   gboolean is_mam;
+   prof_msg_type_t type;
 } ProfMessage;
 
 void session_init(void);
 jabber_conn_status_t session_connect_with_details(const char *const jid, const char *const passwd,
-    const char *const altdomain, const int port, const char *const tls_policy);
+    const char *const altdomain, const int port, const char *const tls_policy, const char *const auth_policy);
 jabber_conn_status_t session_connect_with_account(const ProfAccount *const account);
 void session_disconnect(void);
 void session_shutdown(void);
@@ -155,6 +175,7 @@ jabber_conn_status_t connection_get_status(void);
 char *connection_get_presence_msg(void);
 void connection_set_presence_msg(const char *const message);
 const char* connection_get_fulljid(void);
+char* connection_get_barejid(void);
 char* connection_create_uuid(void);
 void connection_free_uuid(char *uuid);
 #ifdef HAVE_LIBMESODE
@@ -168,13 +189,12 @@ char* connection_jid_for_feature(const char *const feature);
 
 const char* connection_get_profanity_identifier(void);
 
-char* message_send_chat(const char *const barejid, const char *const msg, const char *const oob_url,
-    gboolean request_receipt);
-char* message_send_chat_otr(const char *const barejid, const char *const msg, gboolean request_receipt);
-char* message_send_chat_pgp(const char *const barejid, const char *const msg, gboolean request_receipt);
-char* message_send_chat_omemo(const char *const jid, uint32_t sid, GList *keys, const unsigned char *const iv, size_t iv_len, const unsigned char *const ciphertext, size_t ciphertext_len, gboolean request_receipt, gboolean muc);
-void message_send_private(const char *const fulljid, const char *const msg, const char *const oob_url);
-char* message_send_groupchat(const char *const roomjid, const char *const msg, const char *const oob_url);
+char* message_send_chat(const char *const barejid, const char *const msg, const char *const oob_url, gboolean request_receipt, const char *const replace_id);
+char* message_send_chat_otr(const char *const barejid, const char *const msg, gboolean request_receipt, const char *const replace_id);
+char* message_send_chat_pgp(const char *const barejid, const char *const msg, gboolean request_receipt, const char *const replace_id);
+char* message_send_chat_omemo(const char *const jid, uint32_t sid, GList *keys, const unsigned char *const iv, size_t iv_len, const unsigned char *const ciphertext, size_t ciphertext_len, gboolean request_receipt, gboolean muc, const char *const replace_id);
+char* message_send_private(const char *const fulljid, const char *const msg, const char *const oob_url);
+char* message_send_groupchat(const char *const roomjid, const char *const msg, const char *const oob_url, const char *const replace_id);
 void message_send_groupchat_subject(const char *const roomjid, const char *const subject);
 void message_send_inactive(const char *const jid);
 void message_send_composing(const char *const jid);
@@ -182,7 +202,7 @@ void message_send_paused(const char *const jid);
 void message_send_gone(const char *const jid);
 void message_send_invite(const char *const room, const char *const contact, const char *const reason);
 
-bool message_is_sent_by_us(ProfMessage *message, bool checkOID);
+bool message_is_sent_by_us(const ProfMessage *const message, bool checkOID);
 
 void presence_subscription(const char *const jid, const jabber_subscr_t action);
 GList* presence_get_subscription_requests(void);
@@ -223,6 +243,7 @@ void iq_autoping_check(void);
 void iq_http_upload_request(HTTPUpload *upload);
 void iq_command_list(const char *const target);
 void iq_command_exec(const char *const target, const char *const command);
+void iq_mam_request(ProfChatWin *win);
 
 EntityCapabilities* caps_lookup(const char *const jid);
 void caps_close(void);
@@ -232,8 +253,8 @@ void caps_add_feature(char *feature);
 void caps_remove_feature(char *feature);
 gboolean caps_jid_has_feature(const char *const jid, const char *const feature);
 
-gboolean bookmark_add(const char *jid, const char *nick, const char *password, const char *autojoin_str);
-gboolean bookmark_update(const char *jid, const char *nick, const char *password, const char *autojoin_str);
+gboolean bookmark_add(const char *jid, const char *nick, const char *password, const char *autojoin_str, const char *name);
+gboolean bookmark_update(const char *jid, const char *nick, const char *password, const char *autojoin_str, const char *name);
 gboolean bookmark_remove(const char *jid);
 gboolean bookmark_join(const char *jid);
 GList* bookmark_get_list(void);
