@@ -26,20 +26,21 @@
 
 #include "log.h"
 
-void log_init(log_level_t filter) {}
+void log_init(log_level_t filter, char *log_file) {}
 log_level_t log_get_filter(void)
 {
     return mock_type(log_level_t);
 }
+
 void log_reinit(void) {}
 void log_close(void) {}
 void log_debug(const char * const msg, ...) {}
 void log_info(const char * const msg, ...) {}
 void log_warning(const char * const msg, ...) {}
 void log_error(const char * const msg, ...) {}
-void log_msg(log_level_t level, const char * const area,
-    const char * const msg) {}
-char * get_log_file_location(void)
+void log_msg(log_level_t level, const char * const area, const char * const msg) {}
+
+const char * get_log_file_location(void)
 {
     return mock_ptr_type(char *);
 }
@@ -66,11 +67,6 @@ void chat_log_pgp_msg_in(ProfMessage *message) {}
 void chat_log_omemo_msg_in(ProfMessage *message) {}
 
 void chat_log_close(void) {}
-GSList * chat_log_get_previous(const gchar * const login,
-    const gchar * const recipient)
-{
-    return mock_ptr_type(GSList *);
-}
 
 void groupchat_log_init(void) {}
 void groupchat_log_msg_in(const gchar *const room, const gchar *const nick, const gchar *const msg) {}
