@@ -182,7 +182,7 @@ void cmd_otr_gen_generates_key_for_connected_account(void **state)
     gchar *args[] = { "gen", NULL };
     char *account_name = "myaccount";
     ProfAccount *account = account_new(account_name, "me@jabber.org", NULL, NULL,
-        TRUE, NULL, 0, NULL, NULL, NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+        TRUE, NULL, 0, NULL, NULL, NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 
     will_return(connection_get_status, JABBER_CONNECTED);
     will_return(session_get_account_name, account_name);
@@ -250,6 +250,8 @@ test_cmd_otr_theirfp_from_wintype(win_type_t wintype)
     gchar *args[] = { "theirfp", NULL };
     ProfWin window;
     window.type = wintype;
+    window.layout = NULL;
+    window.urls_ac = NULL;
 
     will_return(connection_get_status, JABBER_CONNECTED);
 
@@ -281,6 +283,8 @@ void cmd_otr_theirfp_shows_message_when_non_otr_chat_window(void **state)
 
     ProfWin window;
     window.type = WIN_CHAT;
+    window.layout = NULL;
+    window.urls_ac = NULL;
     ProfChatWin chatwin;
     chatwin.window = window;
     chatwin.memcheck = PROFCHATWIN_MEMCHECK;
@@ -307,6 +311,8 @@ void cmd_otr_theirfp_shows_fingerprint(void **state)
 
     ProfWin window;
     window.type = WIN_CHAT;
+    window.layout = NULL;
+    window.urls_ac = NULL;
     ProfChatWin chatwin;
     chatwin.window = window;
     chatwin.barejid = recipient;
@@ -333,6 +339,8 @@ test_cmd_otr_start_from_wintype(win_type_t wintype)
     gchar *args[] = { "start", NULL };
     ProfWin window;
     window.type = wintype;
+    window.layout = NULL;
+    window.urls_ac = NULL;
 
     will_return(connection_get_status, JABBER_CONNECTED);
 
@@ -366,6 +374,8 @@ void cmd_otr_start_shows_message_when_already_started(void **state)
 
     ProfWin window;
     window.type = WIN_CHAT;
+    window.layout = NULL;
+    window.urls_ac = NULL;
     ProfChatWin chatwin;
     chatwin.window = window;
     chatwin.barejid = recipient;
@@ -389,6 +399,8 @@ void cmd_otr_start_shows_message_when_no_key(void **state)
 
     ProfWin window;
     window.type = WIN_CHAT;
+    window.layout = NULL;
+    window.urls_ac = NULL;
     ProfChatWin chatwin;
     chatwin.window = window;
     chatwin.barejid = recipient;
@@ -411,6 +423,8 @@ cmd_otr_start_sends_otr_query_message_to_current_recipeint(void **state)
 
     ProfWin window;
     window.type = WIN_CHAT;
+    window.layout = NULL;
+    window.urls_ac = NULL;
     ProfChatWin chatwin;
     chatwin.window = window;
     chatwin.barejid = recipient;
