@@ -26,4 +26,7 @@ void* http_file_put(void *userdata) { return NULL; }
 char* file_mime_type(const char* const file_name) { return NULL; }
 off_t file_size(const char* const file_name) { return 0; }
 
+void http_upload_cancel_processes() {};
+void http_upload_add_upload() {};
+
 #endif
