@@ -11,7 +11,8 @@ void session_init_activity(void) {}
 void session_check_autoaway(void) {}
 
 jabber_conn_status_t session_connect_with_details(const char * const jid,
-    const char * const passwd, const char * const altdomain, const int port, const char *const tls_policy)
+    const char * const passwd, const char * const altdomain, const int port, const char *const tls_policy,
+    const char *const auth_policy)
 {
     check_expected(jid);
     check_expected(passwd);
@@ -34,6 +35,11 @@ const char * connection_get_fulljid(void)
     return mock_ptr_type(char *);
 }
 
+char* connection_get_barejid(void)
+{
+    return mock_ptr_type(char *);
+}
+
 const char * connection_get_domain(void)
 {
     return NULL;
@@ -97,31 +103,35 @@ const char* connection_get_profanity_identifier(void) {
 }
 
 // message functions
-char* message_send_chat(const char * const barejid, const char * const msg, const char *const oob_url,
-    gboolean request_receipt)
+char* message_send_chat(const char *const barejid, const char *const msg, const char *const oob_url, gboolean request_receipt, const char *const replace_id)
 {
     check_expected(barejid);
     check_expected(msg);
     return NULL;
 }
 
-char* message_send_chat_otr(const char * const barejid, const char * const msg, gboolean request_receipt)
+char* message_send_chat_otr(const char * const barejid, const char * const msg, gboolean request_receipt, const char *const replace_id)
 {
     check_expected(barejid);
     check_expected(msg);
     return NULL;
 }
 
-char* message_send_chat_pgp(const char * const barejid, const char * const msg, gboolean request_receipt)
+char* message_send_chat_pgp(const char * const barejid, const char * const msg, gboolean request_receipt, const char *const replace_id)
 {
     return NULL;
 }
 
-void message_send_private(const char * const fulljid, const char * const msg, const char *const oob_url) {}
-char* message_send_groupchat(const char * const roomjid, const char * const msg, const char *const oob_url)
+char* message_send_private(const char *const fulljid, const char *const msg, const char *const oob_url)
 {
     return NULL;
 }
+
+char* message_send_groupchat(const char *const roomjid, const char *const msg, const char *const oob_url, const char *const replace_id)
+{
+    return NULL;
+}
+
 void message_send_groupchat_subject(const char * const roomjid, const char * const subject) {}
 
 void message_send_inactive(const char * const barejid) {}
@@ -132,7 +142,7 @@ void message_send_gone(const char * const barejid) {}
 void message_send_invite(const char * const room, const char * const contact,
     const char * const reason) {}
 
-bool message_is_sent_by_us(ProfMessage *message, bool checkOID) {
+bool message_is_sent_by_us(const ProfMessage *const message, bool checkOID) {
     return TRUE;
 }
 
@@ -236,7 +246,7 @@ gboolean caps_jid_has_feature(const char *const jid, const char *const feature)
     return FALSE;
 }
 
-gboolean bookmark_add(const char *jid, const char *nick, const char *password, const char *autojoin_str)
+gboolean bookmark_add(const char *jid, const char *nick, const char *password, const char *autojoin_str, const char *name)
 {
     check_expected(jid);
     check_expected(nick);
@@ -245,7 +255,7 @@ gboolean bookmark_add(const char *jid, const char *nick, const char *password, c
     return mock_type(gboolean);
 }
 
-gboolean bookmark_update(const char *jid, const char *nick, const char *password, const char *autojoin_str)
+gboolean bookmark_update(const char *jid, const char *nick, const char *password, const char *autojoin_str, const char *name)
 {
     return FALSE;
 }
